/*
Copyright (C) Ivan Ivanov, 2015

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <wx/frame.h>




namespace Khz{namespace BibTex{




/**
@brief The bibtex database generation window. Appears when user selects "Tools->Gen Bibtex" in the menubar.
**/
class BibTexGeneratorStandaloneWindow : public wxFrame
    {
    public:
        /**
        Constructor
        **/
        BibTexGeneratorStandaloneWindow(wxWindow *parent);

    private:


        /**
        Radiobutton. If selected, than bibtex database is going to be generated for books belonging to the particular topic.
        **/
        wxRadioButton *bibtex_gen_topic_radiobutton;

        /**
        Radiobutton. If selected, than bibtex database is going to be generated for books that are currently displayed on the right panel.
        **/
        wxRadioButton *bibtex_gen_right_panel_radiobutton;


        /**
        Radiobutton. If selected, than bibtex database is going to be generated for all of the books.
        **/
        wxRadioButton *bibtex_gen_all_radiobutton;

        /**
        If checked together with #bibtex_gen_topic_radiobutton, than bibtex database
        is going to be generated for all of the books belonging to the inner subtopics
        of the topic that is currently written in #input_for_topic.
        **/
        wxCheckBox *include_subtopics_checkbox;

        /**
        The text input for the topic.
        Bibtex database is going to be generated for books belonging to the topic that is written here.
        Active if #bibtex_gen_topic_radiobutton is selected.
        **/
        wxTextCtrl *input_for_topic;

        /**
        The "Choose" button near of the #input_for_topic.
        Launches the #On_select_where_to_save_database_button_pressed function.
        **/
        wxButton *select_topic_button;

        /**
        Where to save the generated database.
        **/
        wxTextCtrl *where_to_save_database_input;

        /**
        Opens window  that allows to select where to save the generated database. Fills #where_to_save_database_input
        **/
        wxButton *select_path_for_result_button;

        /**
        Ok button
        **/
        wxButton *ok_button;


        /**
        Cancel button
        **/
        wxButton *cancel_button;

        /**
        Text "Topic" above of the #input_for_topic.
        **/
        wxStaticText *topic_text;

        /**
        enables/disables #select_topic_button, #input_for_topic, #include_subtopics_checkbox, #topic_text.
        **/
        void OnRadioButtonChanged(wxCommandEvent& event);

        /**
        Opens window that allow to select where to save the generated bibtex database.
        The selected path  is going be written to #where_to_save_database_input.
        Executes when user presses #select_path_for_result_button.
        **/
        void On_select_where_to_save_database_button_pressed(wxCommandEvent& event);

        /**
        Opens window that allows to select topic. Inserts selected topic into #input_for_topic.
        **/
        void On_select_topic_button_pressed(wxCommandEvent &event);

        /**
        Closes the window. Executes when user presses #cancel_button.
        **/
        void OnCancel(wxCommandEvent &event);


        /**
        Launches the bibtex database generation. Executes when user presses #ok_button.
        **/
        void OnGenBibtexButtonPressed(wxCommandEvent &event);

        /**
        This function prepares the valid "WHERE" statement to query the "main" table of the kolkhoznik database
        to retrieve books belonging to the selected by the user topic.
        The bibtex database is going to be generated for books that are retrieved by this query.

        First, this fuction queries the "topics" table of the database for id(s).
        The query is something like this:
        @code
        "select ID from topics where topic=" + input_for_topic->GetValue().ToStdString()
        @endcode

        Next, this function forms the valid where statement to query the "main" table. Something like
        @code
        std::string where_statement="(ID_of_Topic=3 AND ID_of_Topic=4)"; //this is just example
        @endcode

        @return valid "where" statement
        **/
        std::string PrepareWhereForTopic();


        /**
        Generates the bibtex database
        @param where_statement The valid "WHERE" statement to query the "main" table of the main kolkhoznik database.
        The bibtex database is going to be generated from books retrieved from the main kolkhoznik database by
        this "WHERE" statement.
        The full query  to the database is
        @code
        std::string query= "SELECT * FROM main WHERE " + where_statement;
        //p.s. if where_statement="", than query is going to be "SELECT * FROM main";
        @endcode

        @param placeholder_parameters where_statement could contain placeholders. The placeholder_parameters represents the array
        of the values for these placeholders

        @param where_to_save_bib Full path to the final .bib file.
        **/
        void  GenerateBibtex(std::string &where_statement, std::vector<std::string> &placeholder_parameters, wxString where_to_save_bib);

    };






}} // end of namespace
