/*
Copyright (C) Ivan Ivanov, 2014-2015

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/


#include <wx/spinctrl.h>
#include <wx/hyperlink.h>
#include <wx/checkbox.h>



namespace Khz{
    /**
    @brief classes to form the kolkhoznik configuration window. See detailed description
    of #Khz::CfgWin to understand the general
    hierarchy of the classes




    The top class of the configuration window is #Khz::CfgWin::ConfigureGui.
    The constructor of this class creates a bunch of subwindows, for example
    #Khz::CfgWin::ConfigureGuiSubwindow_main_window,
    #Khz::CfgWin::ConfigureGuiSubwindow_full_text_search_indexer_params.

    Eeach "subwindow" is a window that appear when user changes selection iside of the left
    vertical panel of the configuration window
     (user can select "General", "Main window" and so on subwindows).

    Each subwindow consists of bunch of tabs. Each class that is responsible for a
    tab drawing is named like CfgTab*something*.
    For example it coud be #Khz::CfgWin::CfgTabAdvanced, #Khz::CfgWin::CfgTabFullTextSearchGeneral.


    Every subwindow class is inherited from #Khz::CfgWin::ConfigureGuiSubwindowParent.
    This class provedes two features that all of the subwindows need:

    - The first feature is #Khz::CfgWin::ConfigureGuiSubwindowParent::AskUserToSaveChangesIfAny().
    We should call this function each time when user switch subwindow. This function
    checks if a currently active subwindow have some change of the config parameters.
    If there are some changes, than this function will ask user if he wants to save changes
    (see Khz::CfgWin::ConfigureGui::OnTabChanging()).

    - The second feature is #Khz::CfgWin::ConfigureGuiSubwindowParent::OnTabChanging().
    This functions gets called when user switches tab inside of the subwindow (not switches subwindow,
    but switches a tab inside of it). This function checks if the currently active tab of the subwindow
    have some changes of the config parameters. If there are changes, than this function will ask user
    if he wants to save the changes.


    Each CfgTab*something* class is inherited from #Khz::CfgWin::Configure_gui_tabs_parent. This class
    declares virtual functions and variables that every tab must have, for example
    #Khz::CfgWin::Configure_gui_tabs_parent::write_changes_to_config().


    All of the configuration parameters are stored in a single configuration file.
    The default path to the file is defined like
    @code
    wxString home_dir=wxStandardPaths::Get().GetUserConfigDir() + wxFileName::GetPathSeparator() + ".kolkhoznik";
    wxSTring full_path_to_config = home_dir + wxFileName::GetPathSeparator() + "config.cfg"; // final full path to the config file.
    @endcode
    When user changes config parameters in the configuration window,
    we write new values to the config file.
    Next, we send special event (we use Khz::CfgWin::Configure_gui_tabs_parent::send_reconfigure_panels_event
    to do it) that forces the application to apply changes.
    When application gets this event, it reads the config file again and applies the changes accordingly.

    **/
    namespace CfgWin{



/**
@brief "Advanced" tab of the "Main window"

"Advanced" tab of the "Main window"

General idea: we have two variables:
@code
    int max_num_of_books_to_sort;
    int max_cache_size;
@endcode
When user changes values in inputs (we have two inputs), we read new values from the inputs and change the variables accordingly.
When user presses "Apply" button we write current values of both of the variables to the config file. Next, we send special event
that makes MyMainWindow to reload MyBooksList to apply changes (we use Configure_gui_tabs_parent::send_reconfigure_panels_event for it).
**/
class CfgTabAdvanced : public Configure_gui_tabs_parent
{
public:

    /**
    Constructor
    @param parent standard wxwidgets parent
    @param path_to_config full path to the main config file.
    **/
    CfgTabAdvanced(wxWindow* parent, wxString path_to_config);
private:

    /**
    Default (in case if there is no information in the main config file) value for #max_cache_size
    **/
    int static const default_max_cache=4000;


    /**
    Default (in case if there is no information in the main config file) value for #max_num_of_books_to_sort.
    **/
    int static const default_max_books_to_sort=10000;


    /**
    "Sort always" checkbox. If checked, than MyBooksList::do_not_sort_list_if_books_more_than_this_number =-1,
    as a result books are always sorted.
    **/
    wxCheckBox *sort_always_checkbox;

    /**
    Hyperlink "Details about this options". When user clicks this hyperlink, we show
    popup window explaining what "cache size" and  "Max number of books to sort" are.
    **/
    wxHyperlinkCtrl *CfgTabAdvanced_info_hyperlink;

    /**
    @note configure option

    Current value for maximum number of books that kolkhoznik is allowed to sort.
    Defines MyBooksList::do_not_sort_list_if_books_more_than_this_number
    **/
    int max_num_of_books_to_sort;

    /**
    @note configure option

    Current maximal value of the cache size.
    Defines MyBooksList::max_size_of_cache
    **/
    int max_cache_size;


    /**
    Input for the #max_num_of_books_to_sort. When user changes something in the input, we update #max_num_of_books_to_sort
    (see #on_something_changed  and #on_something_changed2).
    **/
    wxSpinCtrl  *max_num_of_books_to_sort_input;

    /**
    Input for the #max_cache_size. When user changes something in the input, we update #max_cache_size
    (see #on_something_changed  and #on_something_changed2).
    **/
    wxSpinCtrl *max_cache_size_input;

    /**
    Standard wxWidgets sizer
    **/
    wxBoxSizer    *only_sizer;

    /**
    Executes when user presses up/down arrow of any of the input.
    Gets values from  #max_num_of_books_to_sort_input and #max_cache_size_input,
    updates #max_num_of_books_to_sort and
    #max_cache_size
    accordingly.
    **/
    void on_something_changed(wxSpinEvent& event);


    /**
    Executes when user changes number inside of one of the inputs by using keyboard.
    Gets values from  #max_num_of_books_to_sort_input and #max_cache_size_input,
    updates #max_num_of_books_to_sort and #max_cache_size
    accordingly.
    **/
    void on_something_changed2(wxCommandEvent& event);

    /**
    Executes when user clicks the "Sort always" checbox.
    If checkbox is checked, than sets #max_num_of_books_to_sort to -1. Otherwise sets #max_num_of_books_to_sort to #default_max_books_to_sort.
    **/
    void on_sort_always_checkbox_clicked(wxCommandEvent& event);

    /**
    "Apply" button"
    **/
    wxButton *apply_button;


    /**
    Writes changes to the config file. Writes current values of  #max_num_of_books_to_sort and  #max_cache_size into the config file.
    **/
    void write_changes_to_config();

    /**
    Executes when user presses "Apply" button. Calls #write_changes_to_config (),
    Khz::CfgWin::Configure_gui_tabs_parent::send_reconfigure_panels_event ()
    **/
    void on_apply_pressed(wxCommandEvent& event);

    /**
    Executes when user presses "Ok" button. Calls #write_changes_to_config (),
    Khz::CfgWin::Configure_gui_tabs_parent::send_reconfigure_panels_event (),
    closes the configure gui window.
    **/
    void on_ok_pressed(wxCommandEvent& event);

    /**
    Closes the configure gui window
    **/
    void on_cancel_pressed(wxCommandEvent& event);

    /**
    See Configure_gui_tabs_parent::restore_values
    **/
    void restore_values();


    /**
    Shows popup window explaining configuration options when user clicks #CfgTabAdvanced_info_hyperlink
    **/
    void on_info_hyperlink_clicked(wxHyperlinkEvent& event);
};


}} // namespaces
