/*
Copyright (C) Ivan Ivanov, 2014-2015

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/




#include <wx/listbox.h>
#include <wx/checkbox.h>
#include <wx/textctrl.h>
#include <wx/button.h>


namespace Khz{namespace CfgWin{



/**
@brief "Extensions" tab of the "General"

@brief "Extensions" tab of the "General". User can set the default application to open a specific file extension here
(for djvu, pdf and so on files).

General idea: we have two variables:
@code
    std::vector<wxString> array_of_extensions;
    std::vector<wxString> array_of_applications;
@endcode
When user changes a value of an input, we read new value from the input and change the variables accordingly.
When user presses "Apply" button, we write current values of both of the variables to the config file.
Next, we send special event that makes MyMainWindow to reload MyBooksList to apply changes
(we use Configure_gui_tabs_parent::send_reconfigure_panels_event for it).
@note see #array_of_extensions for more details.
**/
class CfgTabDefaultApps : public Configure_gui_tabs_parent
{
public:

    /**
    Constructor, creates tab
    @param parent standard wxWidgets parent
    @param path_to_config full path to the main config file
    **/
    CfgTabDefaultApps(wxWindow* parent, wxString path_to_config);


private:
    /**
    main window sizer
    **/
    wxBoxSizer    *only_sizer;

    /**
    Apply button
    **/
    wxButton *apply_button;

    /**
    Delete extension button
    **/
    wxButton *delete_ext_button;


    /**
    Edit extension button
    **/
    wxButton *edit_extension_button;

    /**
    Add extension button
    **/
    wxButton *add_extension_button;

    /**
    Writes current values of  #array_of_extensions and #array_of_applications to the config file.
    **/
    void write_changes_to_config();

    /**
    Writes changes to the config file, disables apply button, sends an event making MyMainWindow to reconfigure panels
    (calls Configure_gui_tabs_parent::send_reconfigure_panels_event for it).
    **/
    void on_apply_pressed(wxCommandEvent& event);

    /**
    Executes when user presses "Ok" button. Calls #write_changes_to_config (),
    Khz::CfgWin::Configure_gui_tabs_parent::send_reconfigure_panels_event (),
    closes the configure gui window.
    **/
    void on_ok_pressed(wxCommandEvent& event);


    /**
    Closes the configuration window
    **/
    void on_cancel_pressed(wxCommandEvent& event);

    /**
    See Configure_gui_tabs_parent::restore_values
    **/
    void restore_values();

    /**
    current array of extensions, user can see these extensions in the #list_of_extensions  wxListBox (left panel).
    Also, we have the #array_of_applications variable. This array(more precisely, vector) has the
    same dimension as the #array_of_extensions. For example, we can have vectors with two elements:
    @code
    array_of_extensions[0]="djvu";
    array_of_extensions[1]="pdf";

    array_of_applications[0]="windjvuview"
    array_of_applications[1]="AdobeReader"
    @endcode
    Suppose that user double clicked a djvu book with such configuration. As a result,
    kolkhoznik will run command
    @code
    windjvuview "/full/path/to/file".
    @endcode
    **/
    std::vector<wxString> array_of_extensions;



    /**
    See #array_of_extensions
    **/
    std::vector<wxString> array_of_applications;

    /**
    reads config file and generates #array_of_extensions based on it
    **/
    void gen_array_of_extensions_from_config();

    /**
    Listbox with all of the extensions
    **/
    wxListBox *list_of_extensions;

    /**
    User can see/modify an
     application for the currently selected (in the #list_of_extensions wxListBox) extension here.
    **/
    wxTextCtrl *app_name;

    /**
    Full command text entry
    **/
    wxTextCtrl *full_command;

    /**
    Disables/enables #full_command entry.
    **/
    wxCheckBox *custom_command_switch;

    //###################################################

    /**
    Executes when user presses the #add_extension_button button.
    Adds an extension to the #array_of_extensions, makes this extension visible in the #list_of_extensions listbox.
    Also, adds an empy element to the #array_of_applications array (we change value of this empty element when user
                                                    types something to the #app_name entry or to the #full_command entry)
    **/
    void on_add_new_extension(wxCommandEvent& event);



    /**
    Executes when user presses the #delete_ext_button button.
    Removes extension from #array_of_extensions, removes according element from #array_of_applications, updates
     #list_of_extensions listbox accordingly.
     **/
    void on_delete_extension(wxCommandEvent& event);


    /**
    Executes when user presses the "Edit" extension button. Edits element of #array_of_extensions.
    **/
    void on_edit_extension(wxCommandEvent& event);

    /**
    Executes when user changes selection in the extensions listbox (see #on_sel_changed).
    Puts right values into the    #app_name, #full_command, #custom_command_switch elements
    based on a new selected extension. Extracts values for
    currently selected extension from #array_of_extensions, #array_of_applications to do it.
    **/
    void put_data_to_text_entries(unsigned int item_num);

    /**
    Executes when user changes selection in the extensions listbox. In general, just calls #put_data_to_text_entries.
    **/
    void on_sel_changed(wxCommandEvent& event);


    /**
    Executes when user checks or unchecks the "Custom command" checkbox (#custom_command_switch).
    Enables or disables the "Full command" text entry (#full_command).
    **/
    void on_custom_command_switch_checkbox_changed(wxCommandEvent& event);

    /**
    Updates #array_of_applications accordingly
    **/
    void on_text_changed_in_application_entry(wxCommandEvent& event);

    /**
    Updates #array_of_applications accordingly
    **/
    void on_text_changed_in_full_command_entry(wxCommandEvent& event);
};

}} //namespaces
