/*
Copyright (C) Ivan Ivanov, 2014-2015

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/



#include <wx/listbox.h>
#include <wx/panel.h>
#include <wx/checkbox.h>
#include <wx/textctrl.h>
#include <wx/button.h>
#include <wx/bmpbuttn.h>
#include <vector>
#include <wx/sizer.h>
#include "Configure_tabs_parent.h"

namespace Khz{namespace CfgWin{




/**
@brief "Visible Fields" tab of the "Main window"

"Visible Fields" tab of the "Main window".

General idea: we have two variables:

@code
    std::vector<wxString> choosen_fields_array;  //fields in the database
    std::vector<wxString> visible_captions_array; //visible captions
@endcode

Assume for example that we want to display only author and title of the book. Let caption of the title column be "Book's title", and
caption of the author column be "Book's author". Than we have:

@code
choosen_fields_array[0]="Author";  //column name in the database;
choosen_fields_array[1]="Title";  //column name in the database.

visible_captions_array[0]="Book's author";  //any string that we want
visible_captions_array[1]="Book's title";   //any string that we want
@endcode

When user changes a value of an input, we read new value from the
input and change the variables accordingly. When user presses "Apply" button,
we write current values of both of the variables to the config file. Next,
we send special event that makes MyMainWindow to reload MyBooksList to apply changes
(we use Configure_gui_tabs_parent::send_reconfigure_panels_event for it).


Additional:
We heave two listboxes: left, #full_list_of_fields; and right, #used_list_of_fields.
#full_list_of_fields displays all of the available fields(Author, title and so on) based on the #full_fields_array.
 By pressing button with right arrow, user can copy currently selected item to the right listbox that displays
 selected fields (#used_list_of_fields).
 Also, pressing button with right arrow adds additional element to the #choosen_fields_array, #visible_captions_array.
 We have two additional arrays:

@code
    full_fields_display_names; //size is equal to full_fields_array
    choosen_fields_array_display_names; // size is equal to choosen_fields_array
@endcode

These arrays define visible strings for each of the listboxes.
These strings are just visible in the configure gui window strings, nothing more.
**/
class CfgTabFieldsChooser : public Configure_gui_tabs_parent
{
public:
    /**
    Constructor
    @param parent standard wxwidgets parent
    @param path_to_config full path to the main config file.
    **/
    CfgTabFieldsChooser(wxWindow* parent, wxString path_to_config);
private:


    /**
    All of the available fields. User can only choose from this fields.
    **/
    std::vector<wxString> full_fields_array;
    /**
    See detailed description of this class
    **/
    std::vector<wxString> choosen_fields_array;
    /**
    captions for choosen fields. Size of the array must be the same as #choosen_fields_array. See detailed description of this class for more info.
    **/
    std::vector<wxString> visible_captions_array;

    /**
    Visible (in the #full_list_of_fields lisbox) names of the fields of the #full_fields_array.
    These are just visible in the configuration window names, they have no affection on the values that
    are to be written into the config file.
    **/
    std::vector<wxString> full_fields_display_names;


    /**
    Visible in the lisbox (with all of the choosen fields) names of the fields of the #choosen_fields_array.
    These are just visible in the configure gui window names, they have no affection on the values that
    are to be written into the config file.
    **/
    std::vector<wxString> choosen_fields_array_display_names;

    /**
    We have two panels: #right_panel and #right_panel_outer. They have different colors,
    we put one panel inside of the another. As a result, a colored border appear.
    Something like this:
    @code
      ________________
      |              |
      |    ______    |
      |    |     |   |
      |    |     |   |
      |    |_____|   |
      |              |
      |______________|
    @endcode
    Here #right_panel is inside of the #right_panel_outer.
    **/
    wxPanel *right_panel;


    /**
    See #right_panel
    **/
    wxPanel *right_panel_outer;



    /**
    Main window sizer
    **/
    wxBoxSizer    *only_sizer;


    /**
    Listbox with all of the available fields. User can select a field and move it to the
    listbox with selected fields
    **/
    wxListBox *full_list_of_fields;

    /**
    Listbox with visible fields (right listbox).
    **/
    wxListBox *used_list_of_fields;

    /**
    Button with right arrow. When user pressed this button, #on_button_move_right_pressed gets executed.
    **/
    wxBitmapButton *button_move_right;
    //wxBitmapButton *button_move_left;

    /**
    User can change the order of the fields in the #used_list_of_fields listbox.
    We move the currenly selected item by one position up when user presses this button (this is done by the #on_button_up_pressed function).
    **/
    wxBitmapButton *button_move_up;



    /**
    Similar to the #button_move_up, but moves item down
    **/
    wxBitmapButton *button_move_down;


    /**
    Cancel button
    **/
    wxButton *cancel_button;

    /**
    Apply button
    **/
    wxButton *apply_button;


    /**
    Ok button
    **/
    wxButton *ok_buttoon;


    /**
    Removes the currently selected in the right listbox item
    **/
    wxButton *remove_button;


    /**
    "Visible caption of the field" input.
    Assume that the j-th item of  the  #used_list_of_fields is currently selected.
    Than the #visible_name_input input must contain the value of the #visible_captions_array[j].
    **/
    wxTextCtrl *visible_name_input;

    /**
    Generates #full_fields_array and #full_fields_display_names.
    **/
    void generate_full_arrays();

    /**
    reads config file and generates
            #choosen_fields_array,
            #visible_captions_array,
            #choosen_fields_array_display_names
    **/
    void generate_visible_arrays_from_config();


    /**
    Writes #visible_captions_array[index] to the #visible_name_input. This happens when user
    changes selection in the "Visible fields" listbox.
    **/
    void write_string_to_text_entry(unsigned int index);


    /**
    Executes when user change selection in the #used_list_of_fields.
    In general, calls #write_string_to_text_entry
    **/
    void on_used_list_of_fields_sel_changed(wxCommandEvent& event);



    /**
    Executes when user presses button with right arrow.
    By pressing button with right arrow user can copy currently selected in the #full_list_of_fields
    item to the right listbox that displays
    selected fields (#used_list_of_fields).
    Also, it adds additional element to the #choosen_fields_array, #visible_captions_array.
    **/
    void on_button_move_right_pressed(wxCommandEvent& event);


    /**
    removes the currently selected item from the #used_list_of_fields listbox. Also, removes according element
    from #choosen_fields_array and #visible_captions_array
    **/
    void on_button_remove_pressed(wxCommandEvent& event);

    /**
    Executes when user presses the #button_move_up button.
    Moves the currently selected (in the #used_list_of_fields listbox) item one position up. Also, changes the order
    of the elements in #choosen_fields_array, #visible_captions_array accordingly (affects order of the fields).
    **/
    void on_button_up_pressed(wxCommandEvent& event);

    /**
    Similar to the #on_button_up_pressed function, but moves selected item down.
    **/
    void on_button_down_pressed(wxCommandEvent& event);


    /**
    updates #visible_captions_array accordingly to changes
    **/
    void on_text_changed_in_visible_fields_entry(wxCommandEvent& event);


    /**
    Vrites current values of #choosen_fields_array, #visible_captions_array into the config file.
    **/
    void write_changes_to_config();//

    /**
    Writes changes to the config file, disables apply button, sends event that makes MyMainWindow to reconfigure panels
    (calls Configure_gui_tabs_parent::send_reconfigure_panels_event for it).
    **/
    void on_apply_pressed(wxCommandEvent& event);//

    /**
    Executes when user presses "Ok" button. Calls #write_changes_to_config (),
    Khz::CfgWin::Configure_gui_tabs_parent::send_reconfigure_panels_event (),
    closes the configure gui window.
    **/
    void on_ok_pressed(wxCommandEvent& event);//

    /**
    Closes the configuration window
    **/
    void on_cancel_pressed(wxCommandEvent& event);//

    /**
    See Configure_gui_tabs_parent::restore_values
    **/
    void restore_values();//
};

}}// namespaces
