/*
Copyright (C) Ivan Ivanov, 2015-2016

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/



#include "Configure_tabs_parent.h"
#include <wx/button.h>
#include <wx/spinctrl.h>
#include <wx/choice.h>
#include <vector>
#include <wx/checkbox.h>
namespace Khz{namespace CfgWin{

/**
@brief "General" tab of the "Full text search window"

allows to configure appearance of the full text search window.

General idea: user can add or edit something. when user presses "Apply" button
we write changes to the config file and call
Khz::CfgWin::Configure_gui_tabs_parent::send_reconfigure_panels_event ()
**/
class CfgTabFullTextSearchGeneral : public Configure_gui_tabs_parent
{
public:
    /**
    constructor
    @param parent standard wxwidgets parent
    @param path_to_config full path to the main config file
    **/
    CfgTabFullTextSearchGeneral(wxWindow* parent, wxString path_to_config);
private:

    /**
    writes changes to the main config file
    **/
    void write_changes_to_config();


    /**
    See Configure_gui_tabs_parent::restore_values
    **/
    void restore_values();


    /**
    Executes when user presses "Apply" button. Calls #write_changes_to_config (),
    Khz::CfgWin::Configure_gui_tabs_parent::send_reconfigure_panels_event ()
    **/
    void on_apply_pressed(wxCommandEvent &event);

    /**
    Executes when user presses "Ok" button. Calls #write_changes_to_config (),
    Khz::CfgWin::Configure_gui_tabs_parent::send_reconfigure_panels_event (),
    closes the configure gui window.
    **/
    void on_ok_pressed(wxCommandEvent &event);


    /**
    Closes the configuration window
    **/
    void on_cancel_pressed(wxCommandEvent &event);


    /**
    "Apply" button
    **/
    wxButton *apply_button;

    /**
    "Show menubar" checkbox
    **/
    wxCheckBox *show_menubar_checkbox;

    /**
    Sets
    #left_right_border,
    #scrollrate,
    #samples_number,
    #samples_size
     to values
    that are reflecting states of the corresponding inputs
    **/
    void read_spinctrl_and_textctrl_values();

    /**
    executes when user changes state of any of the spinctrl by
    pressing up or down arrow.
    Reads values from all of the spinctrls and
    updates inner variables accordingly
    **/
    void on_some_spinctrl_or_textctrl_changed(wxSpinEvent& event);

    /** Executes when user clicks the "show menubar" checkbox **/
    void on_show_menubar_checkbox_clicked(wxCommandEvent &event);


    /**
    executes when user changes state of any of the spinctrls by a keyboard.
    Reads values from all of the spinctrls and
    updates all of the inner variables accordingly
    **/
    void on_some_spinctrl_or_textctrl_changed2(wxCommandEvent& event);

    /**
    "left/right border" input
    **/
    wxSpinCtrl *left_right_border_input;


    /**
    #write_changes_to_config () uses this var when writing to the config file.
    We change this value when user change value of #left_right_border_input
    **/
    int left_right_border;


    /**
    "icons size" input
    **/
    wxChoice *icons_size_input;

    /**
    #write_changes_to_config () uses this var when writing to the config file.
    We change this value when user change value of #icons_size_input

    if #icons_size_input set to "no icons" than this var=-1
    **/
    int icons_size;

    /**
    We use this vector to fill #icons_size_input when necessary.
    Int is the size of the icons (-1 for no icons), wxString is a
    visible by the user string. Let us consider an example.
    In the constructor of this class we have:
    @code

    // form the icons_sizes_vector;
    icons_sizes_vector.push_back(std::make_pair(-1, wxString("No icons")));
    icons_sizes_vector.push_back(std::make_pair(16, wxString("16")));
    icons_sizes_vector.push_back(std::make_pair(20, wxString("20")));
    icons_sizes_vector.push_back(std::make_pair(22, wxString("22")));
    icons_sizes_vector.push_back(std::make_pair(32, wxString("32")));
    icons_sizes_vector.push_back(std::make_pair(48, wxString("48")));

    // fill icons_size_input;
    for(unsigned int i=0;i<icons_sizes_vector.size();i++)
        {
        icons_size_input->Append(icons_sizes_vector[i].second);
        if(icons_sizes_vector[i].first==icons_size)
            {
            icons_size_input->SetSelection (i);
            }
        }
    @endcode

    when user changes selection inside of the #icons_size_input,
    we do something like this:
    @code
    icons_size=icons_sizes_vector[icons_size_input->GetSelection()].first;
    @endcode
    Finally, when writing into the config file we do something like this:
    @code
    config->Write( wxT("FULL_TEXT_SEARCH/icons_size"), icons_size );
    @endcode
    **/
    std::vector<std::pair<int, wxString> >icons_sizes_vector;


    /**
    Executes when user changes selection of the  #icons_size_input.
    Sets #icons_size to the new value.

    See also #icons_sizes_vector
    **/
    void on_icons_size_input_changed(wxCommandEvent& event);

    /**
    "scrollrate" input
    **/
    wxSpinCtrl *scrollrate_input;


    /**
    #write_changes_to_config () uses this var when writing to the config file.
    We change this value when user change value of #scrollrate_input
    **/
    int scrollrate;


    /**
    The "samples number" input
    **/
    wxSpinCtrl *samples_number_input;


    /**
    #write_changes_to_config () uses this var when writing to the config file.
    We change this value when user change value of #samples_number_input
    **/
    int samples_number;

    /**
    #write_changes_to_config () uses this var when writing to the config file.
    We change this value when user change value of #show_menubar_checkbox
    **/
    bool show_menubar;

    /**
    "samples size" input
    **/
    wxSpinCtrl *samples_size_input;


    /**
    #write_changes_to_config () uses this var when writing to the config file.
    We change this value when user change value of #samples_size_input
    **/
    int samples_size;

};


}}// namespaces
