/*
Copyright (C) Ivan Ivanov, 2015

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "Configure_tabs_parent.h"
#include <wx/button.h>
#include <wx/listbox.h>
#include <wx/bmpbuttn.h>
#include <wx/stattext.h>
#include <vector>

namespace Khz{namespace CfgWin{

/**
@brief "Stemmers order" tab of the "Full text search window"

General idea: user can add or edit something. when user presses "Apply" button
we write changes to the config file and call
Khz::CfgWin::Configure_gui_tabs_parent::send_reconfigure_panels_event ()
**/
class CfgTabFullTextSearchStemmers : public Configure_gui_tabs_parent
{
public:
    /**
    constructor
    @param parent standard wxwidgets parent
    @param path_to_config path to the main config file
    **/
    CfgTabFullTextSearchStemmers(wxWindow* parent, wxString path_to_config);
private:


    /**
    writes changes to the config file
    **/
    virtual void write_changes_to_config();


    /**
    See Configure_gui_tabs_parent::restore_values
    **/
    virtual void restore_values();




    /**
    Executes when user presses "Apply" button. Calls #write_changes_to_config (),
    Khz::CfgWin::Configure_gui_tabs_parent::send_reconfigure_panels_event ()
    **/
    virtual void on_apply_pressed(wxCommandEvent &event);



    /**
    Executes when user presses "Ok" button. Calls #write_changes_to_config (),
    Khz::CfgWin::Configure_gui_tabs_parent::send_reconfigure_panels_event (),
    closes the configure gui window.
    **/
    virtual void on_ok_pressed(wxCommandEvent &event);

    /**
    closes the configuration window without saving changes
    **/
    virtual void on_cancel_pressed(wxCommandEvent &event);


    /**
    "Apply" button
    **/
    wxButton *apply_button;

    /**
    lisbox filled with supported by the xapian stemmes
    **/
    wxListBox *stemmers_list;


    /**
    button with arrow pointing up.
    When pressed, moves selected inside of the #stemmers_list stemmer
    one position up ( see #on_button_up_pressed () )
    **/
    wxBitmapButton *button_move_up;

    /**
    similar to #button_move_up, but move selected stemmer one position down
    **/
    wxBitmapButton *button_move_down;


    /**
    Bold string indicating the default stemmer, for example
    it could be "Default stemmer: english".

    We change this string when user presses #set_to_default_button
    **/
    wxStaticText *default_stemmer_text;

    /**
    Gets default stemmer from the config file.
    **/
    std::string GetDefaultStemmerFromConfig();


    /**
    Gets available stemmers list either from the config file
    or from xapian.
    Basically this function does the following:
    - first, it extracts stemmers list from the config file.
    - second, it creates list of supported by the xapain stemmers. GetStemmersFromXapian() is used for this.
    - third, if checks if extracted from the config file stemmers list
    contain the same stemmers that are supported by the xapian (order does not matter).
    - finally. If extracted from the config file stemmers list
    contain the same stemmers that are supported by the xapian, than this function returns
    the extracted from the config file stemmers list. Otherwise it returns stemmers
    returned by #GetStemmersFromXapian ()
    **/
    std::vector<std::string> GetStemmers();

    /**
    used inside of the #GetStemmers () function. See #GetStemmers ().
    **/
    std::vector<std::string> GetStemmersFromXapian();


    /**
    Checks if the @a stemmers parameter contains the same stemmers that are supported by xapian.
    We use this function inside of #GetStemmers().
    @param stemmers list of stemmers to check.
    **/
    bool AreStemmersGood(std::vector<std::string> stemmers);


    /**
    "Set to default" button. Executes #OnButtonSetToDefaultPressed ().
    **/
    wxButton *set_to_default_button;

    /**
    moves the currently selected stemmer one position up.
    executes when user presses #button_move_up
    **/
    void on_button_up_pressed(wxCommandEvent& event);


    /**
    Similar to  #on_button_up_pressed ()
    **/
    void on_button_down_pressed(wxCommandEvent& event);


    /**
    Enables/disables #button_move_up/#button_move_down when necessary
    **/
    void on_stemmers_sel_changed(wxCommandEvent& event);

    /**
    Sets the currently selected stemmer to default.
    Basically, it sets #default_stemmer and changes #default_stemmer_text accordingly
    **/
    void OnButtonSetToDefaultPressed(wxCommandEvent & event);


    /**
    #write_changes_to_config () uses this var when writing to the config file.
    See #OnButtonSetToDefaultPressed () and #set_to_default_button.
    **/
    std::string default_stemmer;

};


}}// namespaces
