/*
Copyright (C) Ivan Ivanov, 2014-2015

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/



#include <wx/choice.h>
#include <wx/spinctrl.h>
#include <wx/checkbox.h>
#include <vector>
#include <wx/hyperlink.h>


namespace Khz{namespace CfgWin{


/**
@brief "Icons" Tab of the "Main window"

"Icons" Tab of the "Main window".

General idea: we have two variables:
@code
    int books_list_icons_size;
    int library_tree_icons_size;
@endcode
When user changes values in inputs, we read new values from the inputs and change the variables accordingly.
When user presses "Apply" button, we write current values of both of the variables to the config file. Next, we send special event
that makes MyMainWindow to reload MyBooksList and TreeCtrlLibarary to apply changes (we use Configure_gui_tabs_parent::send_reconfigure_panels_event for it).
**/
class CfgTabIconsChooser : public Configure_gui_tabs_parent
{
public:
    /**
    @param parent standard wxwidgets parent
    @param path_to_config full path to the main config file
    **/
    CfgTabIconsChooser(wxWindow* parent, wxString path_to_config);
private:
    /**
    Hyperlink "How do i change icon theme". When user clicks this hyperlink we show
    popup window explaining how to change icons theme
    **/
    wxHyperlinkCtrl *change_icons_info_hyperlink;

    /**
    Current value of the icons size of the books list
    **/
    int books_list_icons_size;

    /**
    Current value of the icons size of the library tree
    **/
    int library_tree_icons_size;

    /**
    Executes when user changes value in the #library_tree_icons_size_input. Reads new value and updates
    #library_tree_icons_size accordingly
    **/
    void on_library_tree_icons_size_input_changed(wxCommandEvent& event);


    /**
    Executes when user changes value in the #books_list_icons_size_input. Reads new value and updates
    #books_list_icons_size accordingly
    **/
    void on_books_list_icons_size_input_changed(wxCommandEvent& event);


    /**
    Reads value of the  #library_tree_icons_size_input_custom and updates
    #library_tree_icons_size accordingly
    **/
    void on_library_tree_icons_size_input_custom_changed();


    /**
    Reads value of the #books_list_icons_size_input_custom and updates
    #books_list_icons_size accordingly
    **/
    void on_books_list_icons_size_input_custom_changed();



    /**
    Executes when user changes value of the #library_tree_icons_size_input_custom by pressing up/down arrow.
    Just calls #on_library_tree_icons_size_input_custom_changed.
    @param event is generated by wxWidgets automatically when user presses arrow of the wxSpinCtrl.
    **/
    void on_library_tree_icons_size_input_custom_changed1(wxSpinEvent& event);


    /**
    Executes when user changes value of the #books_list_icons_size_input_custom by pressing up/down arrow.
    Just calls #on_books_list_icons_size_input_custom_changed.
    @param event is generated by wxWidgets automatically when user presses arrow of the wxSpinCtrl.
    **/
    void on_books_list_icons_size_input_custom_changed1(wxSpinEvent& event);


    /**
    Executes when user changes value of the #library_tree_icons_size_input_custom by typing new value by means of the keyboard.
    Just calls #on_library_tree_icons_size_input_custom_changed.
    @param event is generated by wxWidgets automatically when user types new value by means of the keyboard.
    **/
    void on_library_tree_icons_size_input_custom_changed2(wxCommandEvent& event);




    /**
    Executes when user changes value of the #books_list_icons_size_input_custom by typing new value by means of the keyboard.
    Just calls #on_books_list_icons_size_input_custom_changed.
    @param event is generated by wxWidgets automatically when user types new value by means of the keyboard.
    **/
    void on_books_list_icons_size_input_custom_changed2(wxCommandEvent& event);



    /**
    Shows popup window explaining how to change icons theme. Executes when user clicks #change_icons_info_hyperlink.
    **/
    void on_change_icons_info_hyperlink_clicked(wxHyperlinkEvent& event);


    /**
    User can choose icons size of the books list here (not custom, but predefined size)
    **/
    wxChoice  *books_list_icons_size_input;


    /**
    User can choose icons size of the library tree here (not custom, but predefined size)
    **/
    wxChoice *library_tree_icons_size_input;



    /**
    Checkbox, enables/disables "Custom" input for the library tree icons size
    **/
    wxCheckBox *allow_tree_custom;


    /**
    Checkbox, enables/disables "Custom" input for the books list icons size
    **/
    wxCheckBox *allow_books_custom;


    /**
    Custom input for the books list icons size
    **/
    wxSpinCtrl  *books_list_icons_size_input_custom;


    /**
    Custom input for the library tree icons size
    **/
    wxSpinCtrl *library_tree_icons_size_input_custom;

    /**
    Vector of predefined icons sizes for the #books_list_icons_size_input.
    First element of the pair (int) is size of the icon. Second element of the pair (wxString) is
    value that is to be shown to the user (For example, pairs could be:  (16, "16"), (-1, "No icons") ).

    P.S. For "No icons" size is -1.
    **/
    std::vector<std::pair<int,wxString>> books_list_icons_sizes_vector;


    /**
    Vector of predefined icons sizes for the #library_tree_icons_size_input.
    First element of the pair (int) is size of the icon. Second element of the pair (wxString) is
    value that is to be shown to the user (For example, pairs could be:  (16, "16"), (-1, "No icons") ).

    P.S. For "No icons" size is -1.
    **/
    std::vector<std::pair<int,wxString>> library_tree_icons_sizes_vector;

    /**
    Main window sizer
    **/
    wxBoxSizer    *only_sizer;

    /**
    Executes when user checks/unchecks #allow_tree_custom. Enables/disables "Custom" input for the library tree icons size.
    **/
    void on_allow_tree_custom_icons_checkbox_changed(wxCommandEvent& event);


    /**
    Executes when user checks/unchecks #allow_books_custom. Enables/disables "Custom" input for the books list icons size.
    **/
    void on_allow_books_custom_checkbox_changed(wxCommandEvent& event);


    /**
    Apply button
    **/
    wxButton *apply_button;

    /**
    Writes changes to the config file. Writes current values of  #books_list_icons_size and  #library_tree_icons_size into the config file.
    **/
    void write_changes_to_config();

    /**
    Executes when user presses "Apply" button. Calls #write_changes_to_config (),
    Khz::CfgWin::Configure_gui_tabs_parent::send_reconfigure_panels_event ()
    **/
    void on_apply_pressed(wxCommandEvent& event);


    /**
    Executes when user presses "Ok" button. Calls #write_changes_to_config (),
    Khz::CfgWin::Configure_gui_tabs_parent::send_reconfigure_panels_event (),
    closes the configure gui window.
    **/
    void on_ok_pressed(wxCommandEvent& event);


    /**
    Closes the configuration window
    **/
    void on_cancel_pressed(wxCommandEvent& event);


    /**
    See Configure_gui_tabs_parent::restore_values
    **/
    void restore_values();
};



}} // namespaces
