/*
Copyright (C) Ivan Ivanov, 2015

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/



#include <wx/fileconf.h>
#include <wx/string.h>

namespace Khz{


/**
@brief manages the main configuration file
**/
class ConfigManager
{
public:
    /**
    Checks if the config file exists.
    If config file does not exist, than creates the default one.
    See #is_path_to_config_set also.
    **/
    static void create_config_file_if_not_presented();

    /**
    Puts default config params (generated by #Khz::default_config_params) to the
    config file if they are not there
    **/
    static void put_default_values_to_config_file_if_necessary();

    /**
    Sets #path_to_config to the default value.

    Let us explain a little. As #path_to_config is static, we need to set it only once (
    we do it in Khz::Executables::kolkhoznik::OnInit(),
    Khz::Executables::KolkhoznikFullTextSearchIndexer::OnInit() ).

    Once #path_to_config is set, we can use #GetPathToConfigFile() (it just returns #path_to_config)
    inside of any class. Due to the fact that #path_to_config is static, #GetPathToConfigFile()
    will return the same value everywhere.

    So, when initializing the application, we call #SetStaticPathToConfig().
    After that we use #GetPathToConfigFile() everywhere.
    **/
    static void SetStaticPathToConfig();

    /**
    Sets #path_to_config to the specified value. See SetStaticPathToConfig() (without parameters)
    **/
    static void SetStaticPathToConfig(wxString path);

    /**
    Returns #path_to_config. See SetStaticPathToConfig().
    **/
    static wxString GetPathToConfigFile(){return path_to_config;};

private:

    /**
    Used internally. Let's describe a simple example.
    - When programmer calls #SetStaticPathToConfig(), this var gets set to true.
    - Next, #create_config_file_if_not_presented() have the followind lines of the code:
    @code
    void ConfigManager::create_config_file_if_not_presented()
    {
    ///////////////////////////////////////////////////////////////////////////////////////////////////
    if(!is_path_to_config_set)
        {
        wxString message="ConfigManager: before dealing with config you have to set path_to_config";
        wxMessageBox(message, "Error", wxOK|wxICON_ERROR);
        return;
        }
    // rest of the function
    //code is omitted here
    }
    ////////////////////////////////////////////////////////////////////////////////////////////////////
    @endcode

    So, kolkhoznik would not allow to create the configuration file if #path_to_config is not set.
    **/
    static bool is_path_to_config_set;

    /**
    Full path to the main config file. See SetStaticPathToConfig().
    **/
    static wxString path_to_config;
};
}
