/*
Copyright (C) Ivan Ivanov, 2015

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/



#include <string>
#include<vector>

namespace Khz{
    /**
    @brief Full Text Search classes (gui and engine)
    **/
    namespace FTS {
        /**
        @brief Full text search engine classes (not gui classes)
        **/
        namespace Engine{



/**
@brief Extracts archive by executing external command. Used inside of MainTextExtractor

Each ArchiveExtractorByCommand class object is supposed to extract the only one archive type.
Let's consider an example:

@code
ArchiveExtractorByCommand zip_extractor("%e7z x %f -o%o");
//%e denotes the executable, %f will be replaced by the full path to the archive,
// %o will be replaced by the output dir
*
std::string output_dir="/home/ivan/some_dir"; // we going to extracto archive to folder "/home/ivan/some_dir"
std::string path_to_zip="/home/ivan/test.zip";
// the external command would be "7z x /home/ivan/test.zip -o/home/ivan/some_dir"

zip_extractor.try_to_extract(path_to_zip, output_dir);
// now /home/ivan/some_dir should contain the extracted from the archive files.
// if this dir is empty, than it means that something  went wrong.
@endcode


@note #try_to_extract does not delete the extracted archive. So, you have to delete the
content of the \a ouput_dir manually.
@note we can set the full path to the external executable. See #external_applications.
**/
class ArchiveExtractorByCommand
{
public:

    /**
    Constructor
    @param command Full command to extract the archive. See this class detailed description.
    **/
    ArchiveExtractorByCommand(std::string command);

    /**
    Vector of external applications that are used to extract the archive.
    The first element of the pair is the name of the executable, the second
    element of the pair is the full path to the executable
    ( if executable is in PATH, than we can use just the name of the executable
    instead of the full path)

    @note CURRENTLY #external_applications consists of only one element, so only external_applications[0] exists.
    Below we assume this implicitly.

    @note The second element of the pair could be empty (it could be cleared by #IfWorking). If this is the case,
    than kolkhoznik will show window asking user to provide the full path to the executable.
    As a result, the second element will be filled by the provided by the user value (see the text below for details)

    Let me explain "WHY" we have this.
    All of the archive extractors are constructed by the #Khz::FTS::Engine::MainTextExtractor class.
    The procedure is the following: first, #Khz::FTS::Engine::MainTextExtractor constructs all of the archive extractors.
    When the constructor #ArchiveExtractorByCommand(std::string command) gets executed,
    it fills #pattern and #external_applications. if, for example,  \a command (the constructor parameter)
    is "%e7z x %f -o%o", than #pattern would become "%e x %f -o%o", #external_applications wold be filled like this:
    @code
    external_applications.push_back(std::make_pair("7z", "7z"));
    @endcode
    The full command to extract the archive would be (generated by #construct_full_command)
    @code
    //pattern is "%e x %f -o%o"
    std::string full_command=pattern;
    Khz::Libs::Mstr::replaceAll(full_command, "%e", "\""+external_applications[0].second+"\""); // external_applications[0].second = "7z"
    Khz::Libs::Mstr::replaceAll(full_command, "%f", "\""+path_to_archive+"\"");
    Khz::Libs::Mstr::replaceAll(full_command, "%o", "\""+output_dir+"\"");
    // so, external_applications[0].second = "7z" represents the executable
    // full_command= "7z x /home/ivan/test.zip -o/home/ivan/some_dir"


    //P.S. external_applications could have more than one element,
    // but let us assume for now that it consists only of one element.
    @endcode
    Next, #Khz::FTS::Engine::MainTextExtractor checks if all of the archive extractors are working.
    So, #IfWorking of all of the extractors gets executed.
    As the #IfWorking docs say, #IfWorking can clear all of the second elements of the pairs of #external_applications.
    If one of the extractors is not working, than
    #Khz::FTS::Engine::MainTextExtractor
    would ask user to provide the full path to the executable for all of the executables listed in #external_applications
    whose second element of the pair is empty (CURRENTLY #IfWorking clears all of the second elements(actually, we have only
    one element CURRENTLY),
    but this can be changed in future).
    So, #Khz::FTS::Engine::MainTextExtractor would show a message to user saying something like this:
    @code
std::string message = "oh my god, the archive extractor that is supposed"+
     					 "to extract the 7z files is not working. It is probably"+
						 " that your system does not have some needed executables."+
						 " You have missing  the followind executables: ";
std::string temp_str="";

    // for the simplicity we assume that all of second elements are empty
    // real code is more complicated and can exclude executables with non-empty
    // second elements of the external_applications
    // but CURRENTLY it does not matter due to the fact that
    // IfWorking clears all of the second elements of the external_applications.
for (int i=0;i<external_applications.size();i++)
	{
	temp_str=temp_str+external_applications[i].first+"\n";
	}

message = message + temp_str;
    @endcode
    Next, #Khz::FTS::Engine::MainTextExtractor will show inputs
    asking user to provide the full path to the missing executables. User will
    fill the inputs. #Khz::FTS::Engine::MainTextExtractor will use the provided by
    the user values to fill #external_applications[i].second vector. Something like this:
    @code
    //pseudocode here
    // for the simplicity we assume that all of second elements are empty
    // real code is more complicated and can exclude executables with non-empty
    // second elements of the external_applications
    // but CURRENTLY it does not matter due to the fact that
    // IfWorking clears all of the second elements of the external_applications.
    for (int i=0;i<external_applications.size();i++)
		{
		external_applications[i].second = Input[i]->get_filled_bu_the_user_full_path_to_exe();
		}
    @endcode
    Next, #Khz::FTS::Engine::MainTextExtractor will call #IfWorking () once again.
    Let me note that now the full command to extract the archive would be (generated by #construct_full_command() )
    @code
    //pattern is "%e x %f -o%o"
    std::string full_command=pattern;
    Khz::Libs::Mstr::replaceAll(full_command, "%e", "\""+external_applications[0].second+"\""); // external_applications[0].second = "/full/path/to/7z"
    // !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    // !!!!!!! now external_applications[0].second is not "7z", but "/full/path/to/7z".
    // !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

    Khz::Libs::Mstr::replaceAll(full_command, "%f", "\""+path_to_archive+"\"");
    Khz::Libs::Mstr::replaceAll(full_command, "%o", "\""+output_dir+"\"");

    // !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    // full_command = "/full/path/to/7z x /home/ivan/test.zip -o/home/ivan/some_dir",
    // NOT "7z x /home/ivan/test.zip -o/home/ivan/some_dir"  !!!!!
    // !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    @endcode

    So, the #external_applications vector allows us to set the full path to the executable.
    This is "WHY" we have it.
    **/
    std::vector<std::pair<std::string, std::string>> external_applications;

    /**
    Checks if the extractor is working.
    Let us explain a little:
    - First, this functions extracts \a test_archive to \a output_dir; it is supposed that \a test_archive is an archive that
    contain only one file.
    - Second, it reads content of the \a path_to_file_with_md5 file.
    It is assumed that this file contains string that is md5 of the file that is stored inside of the
    \a test_archive.
    - Finally, this function calculated md5 of the extracted file and compares it with content of the
    \a path_to_file_with_md5. If md5 sums are the same this function returns true.
     If md5 sums are NOT the same,  this function clears all of the second
    elements of #external_applications and returns false.

    P.S. If archive was not extracted properly (i.e. \a output_dir is empty), than this
    function returns false.

    @note this function does not delete the extracted archive. So, you have to delete the
    content of the \a ouput_dir manually.
    **/
    bool IfWorking(std::string test_archive, std::string output_dir, std::string path_to_file_with_md5);


    /**
    Removes existing dir (with it's content). For example, if we created an empy output dir for #IfWorking, than
    we can remove this dir after #IfWorking call finished.
    **/
    static void remove_dir_if_exists(std::string full_path_to_dir);


    /**
    Extracts archive to the specified dir.
    See ArchiveExtractorByCommand detailed description for more info
    **/
    void try_to_extract(std::string path_to_archive, std::string output_dir);
private:

	/**
	construct the full command to extract the archive.
	The code of this function is something like this:
	@code
	std::string ArchiveExtractorByCommand::construct_full_command
	    (
            std::string path_to_archive,
            std::string output_dir,
            std::string executable
            )
		{
		if (pattern=="") return "";

		std::string output=pattern;
		Khz::Libs::Mstr::replaceAll(output, "%e", "\""+executable+"\"");
		Khz::Libs::Mstr::replaceAll(output, "%f", "\""+path_to_archive+"\"");
		Khz::Libs::Mstr::replaceAll(output, "%o", "\""+output_dir+"\"");

		return output;
		}
	@endcode
	See #external_applications for details !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	**/
    std::string construct_full_command(std::string path_to_archive, std::string output_dir, std::string executable);

    /**
    Pattern that is used by #construct_full_command. If, for example, the constructor argument is "%e7z x %f -o%o",
    than pattern would be "%e x %f -o%o"
    See also #external_applications
    **/
    std::string pattern;

    /**
    Just executes an external command
    **/
    bool execute_command(std::string cmd);


    /**
    Checks if the file exists. If it does not exist,
    than this function shows dialog telling user about the problem.
    **/
    bool FileExist(std::string file);
};


}}}//namespace
