/*
Copyright (C) Ivan Ivanov, 2015-2016

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/



#include <string>
#include <xapian.h>
#include <boost/function.hpp>
#include <boost/thread.hpp>
#include <boost/thread/mutex.hpp>
namespace Khz{ namespace FTS { namespace Engine{

/**
@brief performs the full text search database query.

Let's provide a usage example:
@code
std::string full_path_to_xapian_db="/home/ivan/db";
FullTextSearchDbQuery database_asker(full_path_to_xapian_db);

database_asker.InitNewSearch("english", "stupid monkey", 10, Xapian::Query(), 3, 10); //search for "stupid monkey"
database_asker.GiveMePage(1);

for (unsigned int i=0; i<database_asker.GetCurPageSize();i++)
    {
    std::cout << *(database_asker.GetAuthor(i)) << " " << *(database_asker.GetTitle(i))  << "\n\n";
    // prints author and title of the books.
    }


// the second page
database_asker.GiveMePage(2);
for (unsigned int i=0; i<database_asker.GetCurPageSize();i++)
    {
    std::cout << *(database_asker.GetAuthor(i)) << " " << *(database_asker.GetTitle(i))  << "\n\n";
    // prints author and title of the books.
    }






// new search string
database_asker.InitNewSearch("english", "smart monkey", 10, Xapian::Query(), 3, 10); //search for "smart monkey"
database_asker.GiveMePage(1);

for (unsigned int i=0; i<database_asker.GetCurPageSize();i++)
    {
    std::cout << *(database_asker.GetAuthor(i)) << " " << *(database_asker.GetTitle(i))  << "\n\n";
    // prints author and title of the books.
    }
@endcode
**/
class FullTextSearchDbQuery
    {
    public:
        /**
        constructor
        @param path_to_xapian_readable_db full path to the directory with index
        **/
        FullTextSearchDbQuery(std::string path_to_xapian_readable_db);


        /**
        @brief stores data that are necessary to display results of a full text search

        Stores all data that are necessary to display a page with the search results.
        An object of this class is supposed to be created by #Khz::FTS::Engine::FullTextSearchDbQuery::GetCurrentResults
        **/
        class CurrentPageData
            {
            public:
            /**
            the same as #Khz::FTS::Engine::FullTextSearchDbQuery::GetAuthor
            **/
            std::string GetAuthor(int i) {return authors[i];};

            /**
            the same as #Khz::FTS::Engine::FullTextSearchDbQuery::GetTitle
            **/
            std::string GetTitle(int i) {return titles[i];};

            /**
            the same as #Khz::FTS::Engine::FullTextSearchDbQuery::GetTopic
            **/
            std::string GetTopic(int i) {return topics[i];};

            /**
            the same as #Khz::FTS::Engine::FullTextSearchDbQuery::GetExtension
            **/
            std::string GetExtension(int i) {return extensions[i];};

            /**
            the same as #Khz::FTS::Engine::FullTextSearchDbQuery::GetYear
            **/
            std::string GetYear(int i) {return years[i];};

            /**
            the same as #Khz::FTS::Engine::FullTextSearchDbQuery::GetRelPath
            **/
            std::string GetRelPath(int i) {return relpaths[i];};

            /**
            the same as #Khz::FTS::Engine::FullTextSearchDbQuery::GetFileName
            **/
            std::string GetFileName(int i) {return filenames[i];};

            /**
            the same as #Khz::FTS::Engine::FullTextSearchDbQuery::GetWeight
            **/
            int GetWeight(int i) {return weights_in_percents[i];};

            /**
            the same as #Khz::FTS::Engine::FullTextSearchDbQuery::GetMd5
            **/
            std::string GetMd5(int i) {return md5s[i];};


            /**
            the same as #Khz::FTS::Engine::FullTextSearchDbQuery::GetHash
            **/
            std::string GetHash(int i) {return hashs[i];};

            /**
            the same as #Khz::FTS::Engine::FullTextSearchDbQuery::GetAbstract
            **/
            std::vector<std::pair<std::string, int>> GetAbstract(int i) {return abstracts[i];};


            /**
            the same as #Khz::FTS::Engine::FullTextSearchDbQuery::GetCurPageSize
            **/
            int GetCurPageSize() {return matches_size;};


            /**
            the same as #Khz::FTS::Engine::FullTextSearchDbQuery::GetMatchesEstimated
            **/
            long long int GetMatchesEstimated(){return matches_estimated;};


            private:

                /**
                copy of #Khz::FTS::Engine::FullTextSearchDbQuery::authors
                **/
                std::vector<std::string> authors;

                /**
                copy of #Khz::FTS::Engine::FullTextSearchDbQuery::titles
                **/
                std::vector<std::string> titles;

                /**
                copy of #Khz::FTS::Engine::FullTextSearchDbQuery::topics
                **/
                std::vector<std::string> topics;

                /**
                copy of #Khz::FTS::Engine::FullTextSearchDbQuery::extensions
                **/
                std::vector<std::string> extensions;

                /**
                copy of #Khz::FTS::Engine::FullTextSearchDbQuery::years
                **/
                std::vector<std::string> years;

                /**
                copy of #Khz::FTS::Engine::FullTextSearchDbQuery::relpaths
                **/
                std::vector<std::string> relpaths;


                /**
                copy of #Khz::FTS::Engine::FullTextSearchDbQuery::filenames
                **/
                std::vector<std::string> filenames;

                /**
                copy of #Khz::FTS::Engine::FullTextSearchDbQuery::weights_in_percents
                **/
                std::vector<int> weights_in_percents;

                /**
                copy of #FullTextSearchDbQuery::md5s
                **/
                std::vector<std::string> md5s;

                /**
                copy of #Khz::FTS::Engine::FullTextSearchDbQuery::hashs
                **/
                std::vector<std::string> hashs;


                /**
                copy of #Khz::FTS::Engine::FullTextSearchDbQuery::abstracts
                **/
                std::vector<std::vector<std::pair<std::string, int>>> abstracts;


                /**
                copy of #Khz::FTS::Engine::FullTextSearchDbQuery::matches_size
                **/
                int matches_size;

                /**
                copy of #Khz::FTS::Engine::FullTextSearchDbQuery::matches_estimated
                **/
                long long int matches_estimated;


                friend class FullTextSearchDbQuery;

            };


        /**
        Copies current vectors #authors, #titles, #topics, #extensions and so on
        into a #Khz::FTS::Engine::FullTextSearchDbQuery::CurrentPageData object.
        **/
        CurrentPageData GetCurrentResults()
            {
            CurrentPageData output;
            output.authors=this->authors;
            output.titles=this->titles;
            output.topics=this->topics;
            output.extensions=this->extensions;
            output.years=this->years;
            output.relpaths=this->relpaths;
            output.filenames=this->filenames;
            output.weights_in_percents=this->weights_in_percents;
            output.md5s=this->md5s;
            output.hashs=this->hashs;
            output.abstracts=this->abstracts;
            output.matches_size=this->matches_size;
            output.matches_estimated=this->matches_estimated;
            return output;
            }

        /**
        suggest a spelling correction for the whole query (a single word or several words divided by spaces)
        If there are no suggestions that returns "" (an empty string). Otherwise returns the corrected query.
        @note does not work currently because indexing with spelling dictionary takes too long
        **/
        std::string SuggestSpellingCorrection(std::string str);





        /**
        Initializes a new search. SEE THE USAGE EXAMPLE.

        In general this function does the following: It parses the search string and fills #query. #query is
        going to be used by  #GiveMePage() to query the database to get the search results belonging
        to the particular page.

        @param language Supported by xapian stemmer
        @param query_string search string as it was entered into the search input.
        @param _num_of_books_per_page number of books per page
        @param restriction_query additional query to narrow the search results.
        Currently we use this to restrict search by topics. The query to restrict search
        by topics is generated by Khz::FTS::Gui::FullTextSearchTopicsList::GetXapianQuery().
        @param _samples_number When displaying the search results, we show small part
        of the book's text  related to the search string (so called abstract). \a _samples_number
        is the number of the fragments that we want to show. This parameter gets copied to #samples_number
        @param _sample_size size of the fragment of the abstract (words number). This parameter gets_copied
        to #sample_size.
        @return true if there were no errors(xapian database exists and valid), false otherwise.
        **/
        bool InitNewSearch(std::string language, std::string query_string,
                            Xapian::doccount _num_of_books_per_page, Xapian::Query restriction_query,
                            unsigned int _samples_number, unsigned int _sample_size);



        /**
        The same as #GiveMePage, but this function runs in the parallel thread. When called,
        this function only launches the search and exits, it does not wait for search to complete.
        When searching is complete, this function calls \em finished_callback. Your \em finish_callback can
        draw search results as in the class usage example.

        This function is useful for complex queries. As it runs in the parallel thread, it does not
        freeze the interface of the application. We even could perform several searches  in parallel(of course,
        we need more than one #FullTextSearchDbQuery objects. The same #FullTextSearchDbQuery cannot perform
        parallel searches.)
        @param page_num page number
        @param finished_callback Callback. This function gets called when searching is finished.
        @param *output_status Output. Gets set to true if there were no errors(xapian database exists and valid).
        In case of errors gets set to false.
        @note Be sure that only one instance of #GiveMePageParallelThread is running at the same time. You must
        not call #InitNewSearch while #GiveMePageParallelThread is working, this would cause a crash.
        You should use this function like this: when user presses the search button, you should disable the
        search button and call #GiveMePageParallelThread. Your  \em  finished_callback should enable the
        search button. So, user would not be able to do anything while #GiveMePageParallelThread is executing (
        but user can switch to another tab and perform a search there. We are currently using this scheme.
        **/
        void GiveMePageParallelThread
                    (   int page_num,
                        boost::function<void()> &finished_callback,
                        bool *output_status
                    );

        /**
        Extracts books belonging to the particular page from the database (performs the actual searching).
        See FullTextSearchDbQuery usage example.

        @return true if there were no errors(xapian database exists and valid), false otherwise.
        @note We do not have something like GetTotalPages(). The reason for this is the following:
        When searching, xapian gets from the database only necessary number of matches, it does not
        look through the whole database. This works very fast. On the other hand,  in order to calculate the total number
        of the pages, Xapian would require to look through the whole database, it would take too long.

        @note So, how to find out if there are no more pages(we have found everything we could)? Very simple: if
        some page is empty (#GetCurPageSize returns zero) than it means that we have found everything we could and there are no more results
        **/
        bool GiveMePage(int page_num);

        // /*
        // returns #total_num_of_books (total number of books matching the search string).
        // Can be used only AFTER #InitNewSearch() was called.
        // */
        // int GetTotalMatches(){return total_num_of_books;};

        /**
        Just returns #matches_estimated
        **/
        long long int GetMatchesEstimated(){return matches_estimated;};

        /**
        Returns author of the i'th book on the current page.
        See FullTextSearchDbQuery usage example.

        The code of the function is very simple:
        @code
        std::string* GetAuthor(Xapian::doccount i){ return &(authors[i]);};
        @endcode
        **/
        std::string* GetAuthor(Xapian::doccount i){ return &(authors[i]);};


        /*
        Returns id (in the main kolkhoznik sqlite database) of the i'th book on the current page.

        The code of this function is very simple:
        @code
        long long int* GetId(long long int i){return &(ids[i]);};
        @endcode
        */
        //long long int* GetId(long long int i){return &(ids[i]);};


        /**
        returns md5 of the file
        The code of this function is very simple:
        @code
        std::string * GetMd5(long long int i) {return &(md5s[i]);};
        @endcode
        **/
        std::string * GetMd5(long long int i) {return &(md5s[i]);};


        /**
        returns books's hash. Hash is defined like this:
        @code
        std::string my_hash=Khz::Libs::Mstr::sha1(Topic+std::string("_")+val_to_db.Author+std::string("_")+val_to_db.Extension+std::string("_")+
                                            val_to_db.Publisher+std::string("_")+val_to_db.Language+std::string("_")+val_to_db.Title+std::string("_")+
                                            val_to_db.ISBN+std::string("_")+val_to_db.Orientation+std::string("_")+
                                            filename+std::string("_")+RelPath);
        @endcode
        The code of this function is very simple:
        @code
        std::string *GetHash(long long int i} {return &(hashs[i]);};
        @endcode
        **/
        std::string *GetHash(long long int i) {return &(hashs[i]);};

        /**
        Returns weight(in percent) of the i'th book on the current page.

        The code of this function is very simple:
        @code
        int* GetWeight(Xapian::doccount i){ return &(weights_in_percents[i]);};
        @endcode
        **/
        int* GetWeight(Xapian::doccount i){ return &(weights_in_percents[i]);};

        /**
        Returns topic to whom  the i'th book on the current page belongs to.

        The code of this function is very simple:
        @code
        std::string* GetTopic(Xapian::doccount i){return &(topics[i]);};
        @endcode
        **/
        std::string* GetTopic(Xapian::doccount i){return &(topics[i]);};

        /**
        Returns the extension of the file of the i'th book on the current page.

        The code of the function is very simple:
        @code
        std::string* GetExtension(Xapian::doccount i){ return &(extensions[i]);};
        @endcode
        **/
        std::string* GetExtension(Xapian::doccount i){ return &(extensions[i]);};


        /**
        Returns publication date of the i'th book on the current page.

        The code of the function is very simple:
        @code
        std::string* GetYear(Xapian::doccount i){return &(years[i]);};
        @endcode
        **/
        std::string* GetYear(Xapian::doccount i){return &(years[i]);};


        /**
        Returns relative path
        (relatively to the kolkhoz library dir, for example "/B_Biology/BH_Human")
        of the i'th book on the current page.

        The code of the function is very simple:
        @code
        std::string* GetRelPath(Xapian::doccount i){return &(relpaths[i]);};
        @endcode
        **/
        std::string* GetRelPath(Xapian::doccount i){return &(relpaths[i]);};


        /**
        Returns filename of the i'th book on the current page.

        The code of the function is very simple:
        @code
        std::string* GetFileName(Xapian::doccount i){return &(filenames[i]);};
        @endcode
        **/
        std::string* GetFileName(Xapian::doccount i){return &(filenames[i]);};


        /**
        The code of the function is very simple:
        @code
        std::vector<std::pair<std::string, int>>* GetAbstract(Xapian::doccount i){ return &(abstracts[i]);};
        @endcode

        Returns abstract (snippet) (small part
        of the book's text  related to the search string) of the i'th book on the current page.
        The first element of the pair is the abstract word, the
        second element of the pair (int) could be 0, 1, -1.
        If int=1 than word is a search term, you probably want to highlight it.
        If int=-1 than word is samples delimeter (three dots).
        If int=0 than word is a regular word.

        Let's provide a usage example:
        @code
        std::vector<std::pair<std::string, int>> abstract_words = *(database_asker.GetAbstract(1));
        std::string abstract="";
        for (unsigned int i=0;i< abstract_words.size();i++)
            {
            if( abstract_words[i].second==0)
                {
                abstract=abstract+ abstract_words[i].first + " ";
                }
            else if(abstract_words[i].second==1 )
                {
                abstract=abstract+ "<span style='color:red'>" + abstract_words[i].first +"</span> ";
                }
            else if( abstract_words[i].second==-1 )
                {
                abstract=abstract+ " ... ";
                }
            }
        @endcode

        **/
        std::vector<std::pair<std::string, int>>* GetAbstract(Xapian::doccount i){ return &(abstracts[i]);};


        // /*
        // Total  number of pages for the current state.
        // Just returns #total_pages.
        // */
        // int GetTotalPages(){return total_pages;};


        /**
        The number of the books on the current page. See FullTextSearchDbQuery usage example.
        **/
        int GetCurPageSize(){return matches_size;};


        /**
        Returns title of the i'th book on the current page.

        The code of the function is very simple:
        @code
        std::string * GetTitle(Xapian::doccount i){ return &(titles[i]);};
        @endcode
        **/
        std::string * GetTitle(Xapian::doccount i){ return &(titles[i]);};

        /**
        Returns #error_message. #error_message can be filled by constructor, by #InitNewSearch()
        and so on in case of error during the database query.
        **/
        std::string get_error_message(){return error_message;};


        /**
        returns #everything_is_ok
        **/
        bool IsOk(){return everything_is_ok;};


        /**
        Connects to the new xapian database. Use #InitNewSearch() after calling this function.
        **/
        void ReconnectToDB(std::string path_to_xapian_readable_db);
    private:

        /**
        See \a _samples_number parameter of #InitNewSearch().
        Used to exchange data between #InitNewSearch() and #GiveMePage().
        **/
        unsigned int samples_number;


        /**
        See \a _sample_size parameter of #InitNewSearch().
        Used to exchange data between #InitNewSearch() and #GiveMePage().
        **/
        unsigned int sample_size;


        /**
        Normally this var is true. It
        can be set to false by constructor or by #InitNewSearch() in case of error during the database query
        (constructor sets var to false if xapian database does not exist).
        If this is the case, than #GiveMePage() would not do anything, it would just return false.
        (If #InitNewSearch() did not fill #query than we cannot generate page)
        **/
        bool everything_is_ok;


        /**
        Database to get results from.
        **/
        Xapian::Database xapian_database;



        /**
        Gets filled by #InitNewSearch(). #query is
        going to be used by  #GiveMePage() to query the database to get the search results belonging
        to the particular page
        **/
        Xapian::Query query;


        // /**
        // total number of pages.  Filled by #InitNewSearch().
        // **/
        // int total_pages;


        /**
        Number of book on the current page. Renewed every time when #GiveMePage() gets called.
        **/
        int matches_size;



        // /**
        // Total number of books matching the search string. Filled by #InitNewSearch()
        // **/
        // Xapian::doccount total_num_of_books;


        /**
        Number of books per page.
        Used to exchange data between #InitNewSearch() and #GiveMePage().
        **/
        int num_of_books_per_page;


        /**
        vector of the authors of the books on the current pages.
        Gets filled by #GiveMePage(). See also #GetAuthor().
        **/
        std::vector<std::string> authors;

        /**
        vector of the topics of the books on the current pages.
        Gets filled by #GiveMePage(). See also #GetTopic().
        **/
        std::vector <std::string> topics;


        /**
        vector of the extensions of the books on the current pages.
        Gets filled by #GiveMePage(). See also #GetExtension().
        **/
        std::vector<std::string> extensions;


        /**
        vector of the publication dates of the books on the current pages.
        Gets filled by #GiveMePage(). See also #GetYear().
        **/
        std::vector<std::string> years;


        /**
        vector of the relative paths of the books on the current pages.
        Gets filled by #GiveMePage(). See also #GetRelPath().
        **/
        std::vector<std::string> relpaths;



        /**
        vector of the filenames of the books on the current pages.
        Gets filled by #GiveMePage(). See also #GetFileName().
        **/
        std::vector<std::string> filenames;




        /**
        vector of the titles of the books on the current pages.
        Gets filled by #GiveMePage(). See also #GetTitle().
        **/
        std::vector<std::string> titles;


        /**
        vector of the weights of the books on the current pages.
        Gets filled by #GiveMePage(). See also #GetWeight().
        **/
        std::vector<int> weights_in_percents;


        /**
        vector of the md5s of the books on the current page.
        Gets filled by #GiveMePage(). See also #GetMd5().
        **/
        std::vector<std::string> md5s;

        /**
        vector of the hashs of the books on the current page.
        Gets filled by #GiveMePage(). See also #GetHash().
        **/
        std::vector<std::string> hashs;

        /**
        vector of the abstracts of the books on the current pages.
        Gets filled by #GiveMePage(). See #GetAbstract() for more info.
        **/
        std::vector<std::vector<std::pair<std::string, int>>> abstracts;

        /**
        Generates abstract (small part
        of the book's text  related to the search string)
        @param docid document to generate abstract
        @param enquire enquire that was used to find the document.
        @param imaxoccs the number of the fragments of the abstract.
        @param ictxwords size of the fragment of the abstract (words number).
        @param output the produced abstract. See #GetAbstract() for more info.

        P.S. This function is based on the Recoll (full text search app) Query::Native::makeAbstract function.
        Query::Native::makeAbstract is a mess. I have added a little of my own mess,
        as a result  the MakeAbstract function is a COMPLETE mess.
        **/
        int MakeAbstract(Xapian::docid docid,  Xapian::Enquire &enquire,
                         unsigned int imaxoccs, unsigned int ictxwords, std::vector<std::pair<std::string, int>> &output);







        /**
        Gets array of the matched terms for the particular document
        (But terms specifying title, author, topic are excluded from the  output)
        @param enquire enquire that was used to find the document
        @param xdocid the id of the document
        @param terms output

        Used inside oth #MakeAbstract

        This function is based on the Recoll (full text search app) Query::Native::GetMatchTerms function.
        **/
        bool GetMatchTerms( Xapian::Enquire &enquire,unsigned long xdocid, std::vector<std::string>& terms);




        /**
        See #termfreqs

        P.S. This function is based on the Recoll (full text search app) Query::Native::setDbWideQTermsFreqs function.
        Call this function only once only for a given query.

        Here is what the Recoll docs say:

        Retrieve db-wide frequencies for the query terms and store them in
        the query object. This is done at most once for a query, and the data is used
        while computing abstracts for the different result documents.
        **/
        void  setDbWideQTermsFreqs(Xapian::Database &xapian_database, Xapian::Query &query);



        /**
        Used inside of #MakeAbstract().

        Determines quality of the every term in \a terms for the particular document.
        Fills \a byQ.

        I do not understand completely what does this function do, this function is
        based on the Recoll (full text search app) Query::Native::QualityTerms function.
        Here is what the Recoll docs say:

        Compute matched terms quality coefficients for a matched document by
        retrieving the Within Document Frequencies and multiplying by
        overal term frequency, then using log-based thresholds.
        2012: it's not too clear to me why exactly we do the log thresholds thing.
        Preferring terms wich are rare either or both in the db and the document
        seems reasonable though
        To avoid setting a high quality for a low frequency expansion of a
        common stem, which seems wrong, we group the terms by
        root, compute a frequency for the group from the sum of member
        occurrences, and let the frequency for each group member be the
        aggregated frequency.
        **/
        double QualityTerms(Xapian::docid docid,
                                            const std::vector<std::string>& terms,
                                            std::multimap<double, std::string >& byQ);




        /**
        Let's consider an example:
        @code
        std::vector<unsigned int> some_vector;
        some_vector.push_back(1);  // element number 0
        some_vector.push_back(5);  // element number 1
        some_vector.push_back(25); // element number 2

        int num= FindClosesInt(4, some_vector);
        // num=1 due to the fact that 5 (element number 1) is closest to 4.
        @endcode
        **/
        unsigned int FindClosestInt(unsigned int int_num, std::vector<unsigned int> &some_vector);

        /**
        If we are using stemmer
        Used by #MakeAbstract();
        **/
        bool using_stemmer;


        /**
        Removes terms related to topic, author, title from the terms list.
        These term have special prefixes.
        **/
        void NoPrefixList(const std::vector<std::string> &in_qterms, std::vector<std::string> &out_qterms);


        /**
        See #get_error_message()
        **/
        std::string error_message;


        /**
        Database wide terms frequency for the query. Gets filled by #setDbWideQTermsFreqs();
        #setDbWideQTermsFreqs() gets called by InitNewSearch();
        Used by #MakeAbstract().
        **/
        std::map<std::string, double> termfreqs;


        /**
        The number of the ESTIMATED!!! (see Xapian::Mset::get_matches_estimated () ) matches for the current query.
        Gets filled by #GiveMePage, something like this:
        @code
        Xapian::MSet matches = enquire.get_mset(this->num_of_books_per_page*(page_num-1), this->num_of_books_per_page);
        if(page_num==1 or (this->num_of_books_per_page*(page_num))>=matches_estimated)
            {
            this->matches_estimated=matches.get_matches_estimated();
            }
        @endcode
        So, we recalculate #matches_estimated only if this is really necessary
        **/
        long long int matches_estimated;


        /**
        Used only inside of #GiveMePageParallelThreadInternal to prevent simultaneous execution of #GiveMePageParallelThreadInternal
        **/
        boost::mutex global_mutex;

        /**
        Used only inside of #GiveMePageParallelThread. #GiveMePageParallelThread only launches the parallel thread. This
        function actually gets executed in this thread.
        **/
        void GiveMePageParallelThreadInternal
                    (   int page_num,
                        boost::function<void()> &finished_callback,
                        bool *output_status
                    );





    };



}}}//namespace
