/*
Copyright (C) Ivan Ivanov, 2015-2016

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/



#include <fstream>
#include <wx/filename.h>
#include <xapian.h>
#include <wx/progdlg.h>
#include "DEBUGMESSAGE.h"
#include <boost/function.hpp>
#include <vector>
namespace Khz{ namespace FTS { namespace Engine{

/**
@brief Main full text search indexer.

- Reads the main kolkhoznik sqlite database line by line, index every book there.
- Supports incremental index (no reindexing of the already indexed books).
- Supports multithreaded indexing


General idea we have one thread (this thread is represented by  the FillQueueOfBooksToIndex() function)
that fills the books queue #the_queue.

Multiple indexing threads (they represented by the GetBookFromQueueAndIndexIt() function)
remove books from the queue and index them.


Usage example: (very simplified, of course you should not hardcode paths
                in the real apllications)



@code
FullTextSearchIndexerWindow::FullTextSearchIndexerWindow(wxWindow *parent, const wxString& title)
: wxFrame(parent, wxID_ANY, title)
{
callback_update_progress=boost::bind(&Khz::FTS::Gui::FullTextSearchIndexerWindow::UpdateProgressDialogCallBack, this, _1, _2, _3);
callback_finish_indexing=boost::bind(&Khz::FTS::Gui::FullTextSearchIndexerWindow::FinishIndexingCallBAck, this);
//rest of the code is omitted
}
@endcode

@code
void FullTextSearchIndexerWindow::UpdateProgressDialogCallBack(int percents, std::string message, bool *stop)
    {
    std::cout << "Indexed: " << pecents << " percents\n";
    std::cout << "Message is " << message << "\n";
    if(user_pressed_cancel_button) *stop=true;
    }
@endcode


@code
void FullTextSearchIndexerWindow::FinishIndexingCallBAck()
    {
    std::cout << "Indexing is finished!!!";
    }
@endcode

@code
FullTextSearchIndexeWindow::Index(wxCommandEvent &event)
{
// path to the full text search database
std::string xapian_path="/home/ivan/full_text_search_db";

// where should we save log
std:string path_to_log="/home/ivan/indexing_log.txt";

// path to the main kolkhoznik sqlite database
std::string path_to_database="/home/ivan/kolkhoznik_db.sqlite";

// path to the kolkhoz library directory
std::string path_to_library="/home/ivan/my_kolkhoz_library";


// EMPTY!!! temporary dir to extract archives
std::string temp_dir="/home/ivan/temp_dir";

//text layer extractors external commands
//see Khz::FTS::Engine::TextFromFileExtractorByCommand
std::vector<std::pair<std::string, std::string> > text_extractors_commands;
text_extractors_commands.push_back(std::make_pair("djvu", "%edjvutxt %f"));
text_extractors_commands.push_back(std::make_pair("pdf", "%epdftotext -enc UTF-8 %f -"));


//archive extractors external commands
//see Khz::FTS::Engine::ArchiveExtractorByCommand
std::vector<std::pair<std::string, std::string> > archive_extractors_commands;
text_extractors_commands.push_back(std::make_pair("zip", "%e7z x %f -o%o"));
text_extractors_commands.push_back(std::make_pair("gz", "%%e7z x %f -o%o"));


// create the indexer
Khz::FTS::Engine::FullTextSearchIndexer text_indexer
                                    (
                                    xapian_path,
                                    path_to_log,
                                    path_to_database,
                                    path_to_library,
                                    text_extractors_commands,
                                    archive_extractors_commands,
                                    temp_dir,
                                    callback_update_progress, // goes by reference
                                    callback_finish_indexing, //goes by reference
                                    1000 // cache size
                                    );

if(!text_indexer.AreTextExtractorsWorking())
    {
    // AreTextExtractorsWorking() would show error message, so no messages here
    return;
    }
if(!text_indexer.log_ok)
    {
    wxMessageBox("Cannot open the specified log file", wxT("Error"), wxOK | wxICON_ERROR);
    return;
    }
if(!text_indexer.xapian_ok)
    {
    wxMessageBox("Cannot open the specified full text search index database", wxT("Error"), wxOK | wxICON_ERROR);
    return;
    }
if(text_indexer.unknown_error)
    {
    wxString message= "Unknown error occured. Check that just one instance of the indexer is running and that you have enough free space on your hard drive";
    wxMessageBox(message, wxT("Error"), wxOK | wxICON_ERROR);
    return;
    }


// number of indexing threads
int threads_number = 3;


// the indexing threads must be created first
for (int i=0; i< threads_number; i++)
        {
        indexing_threads.push_back
            (
            new boost::thread
                (
                &Khz::FTS::Engine::FullTextSearchIndexer::GetBookFromQueueAndIndexIt, text_indexer
                )
            );

        text_indexer->Incr_num_of_currently_working_threads();
        }
    // IMPORTANT!!! indexing threads MUST be crated before queue_prepare_thread
    queue_prepare_thread= new boost::thread(&Khz::FTS::Engine::FullTextSearchIndexer::FillQueueOfBooksToIndex, text_indexer);
}
@endcode
**/
class FullTextSearchIndexer
{
public:

    /**
    Constructor
    @param xapian_path full path to the xapian database directory (xapian saves it's database to the directory,
    not to the file). Directory can be empty or it could contain already existing xapian database.
    If database already exists than indexer would propose to create an incremental index
    @param path_to_log where to save the log file (simple txt file). If exists than will be overwritten.
    @param path_to_main_database full path to the main kolkhoznik database. We are gonna read which books
    to index from this database.
    @param _path_to_library full path to dir with books.
    @param text_extractors_commands see the usage example
    @param archive_extractors_commands see the usage example
    @param temp_dir EMPTY temporary directory. We are gonna to unpack archives there.
    @param _update_progress_bar_callback Callback function that is used to update progress bar.
    FullTextSearchIndexer calls this function every time when progress bar is supposed to be
    updated. See #update_progress_bar_callback for more info.
    P.S. See the usage example too.
    @param _indexing_finished_callback This function gets executed when the the indexing is finished.
    You can show a message informing user that the indexing is finished and so on.
     P.S. See the usage example.
    @param _cache_size Defines #cache_size. See #cache_size for more info.
    **/
    FullTextSearchIndexer
        (
        std::string xapian_path,
        std::string path_to_log,
        std::string path_to_main_database,
        std::string _path_to_library,
        std::vector<std::pair<std::string, std::string> > text_extractors_commands,
        std::vector<std::pair<std::string, std::string> > archive_extractors_commands,
        std::string temp_dir,
        boost::function<void(int, std::string, bool *)> &_update_progress_bar_callback,
        boost::function<void()> &_indexing_finished_callback,
        unsigned int _cache_size
        );

    /**
    The main thread that puts books to the queue.

    Let us describe briefly how it works.
    This function  queries the main kolkhoznik database (uses #database_asker to do it) and
    extracts books line by line. It puts extracted book to #the_queue, at the same time it fills #status_queue.
    When books gets inserted to #the_queue, "true" gets inserted to the #status_queue. When there no more books
    in the main database, this functions does not insert anything to #the_queue, but keeps inserting "false"
    to the #status_queue. When inserting "false" to the #status_queue this function checks #num_of_currently_working_threads.
    If #num_of_currently_working_threads == 0, than this thread shots himself down (return gets executed)

    Now let's describe how #num_of_currently_working_threads can turn to zero. When creating indexer threads,
    one have to call #Incr_num_of_currently_working_threads() each time when a new indexing thread gets created
    (see usage example)
    So, at the beginning of the indexing process #num_of_currently_working_threads is equal to the number of the
    indexing thread. During the indexing process, each indexing
    thread (GetBookFromQueueAndIndexIt()) is working like this:
    first, it removes one element from #status_queue. It this element is "true", than indexing thread
    removes one book from #the_queue and indexes it. If element of #status_queue is false (there no
    more books in the main kolkhoznik database), than this function decreases #num_of_currently_working_threads by 1
    ( #num_of_currently_working_threads\-\- gets called, #num_of_currently_working_threads is
     protected by #num_of_currently_working_threads_mutex mutex) and shots itself down (return gets executed).
    So, in the end of the indexing process, all of the indexing threads should shot itself down, as a result
    #num_of_currently_working_threads would be equal to zero.

    Once again: when we creating indexing threads, we call #Incr_num_of_currently_working_threads() each time
    when we create a new thread. So, at the beginning of the indexing process #num_of_currently_working_threads
    is equal to the number of working threads. At the end of the indexing process, each indexing thread
    shots itself down and decreases #num_of_currently_working_threads by 1. When all of the indexing threads
    are dead, #num_of_currently_working_threads is equal to zero. So, when
     #num_of_currently_working_threads gets equal to zero, the main thread that puts books to the queue
    could safely shot itself down too.

    When we creating the indexer, we should created the indexing threads first ( #GetBookFromQueueAndIndexIt()).
    AFTER indexing threads
    are created, we should create the thread that puts books to the queue (#FillQueueOfBooksToIndex()).
    If we reverse the order, than #FillQueueOfBooksToIndex() would shot itself down
    instantly as #num_of_currently_working_threads would be zero.
    **/
    void FillQueueOfBooksToIndex();





    /**
    The indexing thread

    Extract book from #the_queue and index it.  Works together with #FillQueueOfBooksToIndex().
    See #FillQueueOfBooksToIndex() for more info.
    **/
    void GetBookFromQueueAndIndexIt();





    /**
    The code of this function is very simple:
    @code
    void  Incr_num_of_currently_working_threads()
        {
        num_of_currently_working_threads++;
        };
    @endcode
    See #FillQueueOfBooksToIndex() to understand "WHY".
    **/
    void  Incr_num_of_currently_working_threads()
        {
        num_of_currently_working_threads++;
        };





    /**
    The code of this function is very simple:
    @code
    bool AreTextExtractorsWorking()
        {
        return text_extractor.extractors_working();
        };
    @endcode
    So, see #Khz::FTS::Engine::MainTextExtractor::extractors_working() for details.
    **/
    bool AreTextExtractorsWorking()
        {
        return text_extractor.extractors_working();
        };




    /**
    Destructor
    **/
    ~FullTextSearchIndexer()
        {
        sqlite3_close(db);
        delete database_asker;
        delete xapian_writable_db;
        for(unsigned int i=0;i<readable_databases_pointers.size();i++)
            {
            delete readable_databases_pointers[i];
            }
        };





    /**
    Constructor sets #log_ok to false if it cannot open the specified log file (see constructor parameters).
    If log file was opened properly, than #log_ok would be true

    See FullTextSearchIndexer usage example to understand how to use it.
    **/
    bool log_ok;




    /**
    Constructor sets #xapian_ok false if it cannot open the specified xapian database (see constructor parameters).
    If database was opened properly, than #xapian_ok would be true.

    See FullTextSearchIndexer usage example to understand how to use it.
    **/
    bool xapian_ok;




    /**
    Indexer can set this var to true if there was error while writing to the xapian database.
    It could happen, for example, if there not free space on the hard drive.

    See FullTextSearchIndexer usage example to understand how to use it.
    **/
    bool unknown_error;




    /**
    Removes obsolete books from the xapian database.
    As one can guess, sometimes books gets removed from the library.
    So, we need to remove them from the xapian database too when performing
    an incremental index.

    We do it like this:
    When generating the kolkhoznik sqlite (not xapian) database we
    add the "Hash" row to the main table of the database. This Hash is defined like this:
    @code
     std::string hash=Khz::Libs::Mstr::sha1(Topic+"_"+Author+"_"+Extension+"_"+
                                            Publisher+"_"+Language+"_"+Title+"_"+
                                            ISBN+"_"+Orientation+"_"+
                                            filename+"_"+RelPath);
    @endcode

    While generating the xapian database we add two values to the document, something like this:
    @code
    doc.add_value (0, md5_of_file);
    doc.add_value(1, Hash);
    @endcode


    So, BEFORE we start the full text search index generation process, we call #RemoveObsoleteBooksFromDb().
    This function iterates trough all of the documents in the xapian database and extracts md5 and Hash.
    Next, it queries the main sqlite database and gets the number of documents with the same md5 and hash.
    If this number is zero (the document is presented in the xapian database, but is not presented in
    the main sqlite database), than we delete this document from the xapian database.
    **/
    void RemoveObsoleteBooksFromDb();




private:
    /**
    The indexing process is running like this:<br>

             First, we index books without saving data to the hard disk (data are stored in RAM)<br>
             Second, after enough data has been accumulated, we write data to the hard disk.<br>
             The reason for this is the following: Xapian writes data to the hard disk much faster
             if data are written by big pieces (for example, it writes 1000 books per query much  faster than
             100 books per 10 queries (i have no idea why).<br>br>
             So, the cache size is the number of the books that Xapian writes to the database per query.

    @note protected by #xapian_database_access_mutex
    **/
    unsigned int cache_size;
    /**
    If this values becomes true, than we would shot down the indexing.
    See #update_progress_bar_callback for more info.
    **/
    bool force_stop;

    /**
    protects #force_stop;
    **/
    boost::mutex force_stop_mutex;
    /**
    Extracts text layer from books. Unpacks archives if necessary.

    - If file is just a book (for example pdf, djvu), than  just extracts
    the text layer from it.
    - If file is an archive, than  unpacks
    the archive (recursively, so all of the subarchives get unpacked too)  and
    extract text layers from all of the files. The final text is the
    result of the concatenation of the text layers.
    - In case if the text layer extraction was not successful, writes message to #log_file.

    @param one_book one element of the #Khz::Libs::DB::mapDB vector (for example,
    one can get the author of the book by one_book["Author"],
    relative path to the file (relatively to the library dir) by
    one_book["RelPathToFile"] and so on).
    See #Khz::Libs::DB::mapDB
    @param Text_Extractor This requires an explanation. When constructing the
    FullTextSearchIndexer object we create #text_extractor that is an object
    of the Khz::FTS::Engine::MainTextExtractor class. Next, when "user" calls #AreTextExtractorsWorking()
    (See FullTextSearchIndexer usage example) we check if #text_extractor is working
    (See Khz::FTS::Engine::MainTextExtractor::extractors_working()).
    Finally, when user creates indexer threads (See FullTextSearchIndexer usage example),
    each thread make it's own copy of the WORKING text extractor (without several copies we would not
                                                                  be able to extract text layer in parallel).
    So, \a Text_Extractor is a copy of #text_extractor.

    @return 1 if the text layer was extracted successfully; -1 if file does not exist or we do not know how
    extract the text  layer from the filetype; 0 if an error during the execution of the external command occurred.
    **/
    int ExtractTextFromFile(std::map<std::string, std::string> one_book, MainTextExtractor &Text_Extractor );




    /**
    Extracts the text layer from the book (see #ExtractTextFromFile) and
    writes data to #xapian_writable_db. Checks if book is already indexed. If it is,
    than this function do nothing.
    @param one_book one element of the #Khz::Libs::DB::mapDB vector (for example,
    one can get the author of the book by one_book["Author"],
    relative path to the file (relatively to the library dir) by
    one_book["RelPathToFile"] and so on).
    See #Khz::Libs::DB::mapDB
    @param Text_Extractor This require an explanation. When constructing the
    FullTextSearchIndexer object we create #text_extractor that is an object
    of the Khz::FTS::Engine::MainTextExtractor class. Next, when "user" calls #AreTextExtractorsWorking()
    (See FullTextSearchIndexer usage example) we check if #text_extractor is working
    (See Khz::FTS::Engine::MainTextExtractor::extractors_working()).
    Finally, when user creates indexer threads (See FullTextSearchIndexer usage example),
    each thread make it's own copy of the WORKING text extractor (without several copies we would not
                                                                  be able to extract text layer in parallel).
    So, \a Text_Extractor is a copy of #text_extractor.
    @param xapian_readable_database Xapian::Database  object
    pointing to the xapian database that we are creating. Let's explain a little.
    Our indexer have only one instance of Xapian::WritableDatabase (see #xapian_writable_db)
    and multiple instances of Xapian::Database. The reason is the following: xapian
    does not allow simultaneous writing to the database from the multiple threads,
    but it does allow simultaneous reading. So, each indexing thread creates it's own
    copy of Xapian::Database. This is used to read the database to find out if
    the book is already indexed.
    **/
    bool IndexBook
        (
        std::map<std::string, std::string> one_book,
        MainTextExtractor &Text_Extractor,
        int xapian_readable_database_num
        );



    /**
    maximal size of    #the_queue and #status_queue (they both have the same size).
    Always protected by #the_queue_access_mutex
    **/
    unsigned int max_queue_size=100;



    /**
    Points to the main kolkhoznik (sqlite) database.
    Used by #FillQueueOfBooksToIndex(). See #FillQueueOfBooksToIndex() also.
    **/
    sqlite3 *db;



    /**
    Number of the currently working INDEXING threads. See #FillQueueOfBooksToIndex()
    to understand how we use it
    **/
    int num_of_currently_working_threads;
    /**
    Protects access to #num_of_currently_working_threads.
    **/
    boost::mutex num_of_currently_working_threads_mutex;


    /**
    Full path to the kolkhoz library directory. We set it inside of the constructor (see \a _path_to_library
                                                                                     constructor parameter)
    Used by #ExtractTextFromFile to construct the full path to the book from the relative path
    **/
    std::string path_to_library;
    /**
    protects #path_to_library;
    **/
    boost::mutex path_to_library_mutex;



    /**
    Queue with books. Books gets inserted to the queue by The main thread that puts books to the queue
    (#FillQueueOfBooksToIndex() ). The indexing threads ( #GetBookFromQueueAndIndexIt() )
    removes books from the queue and index it. See #FillQueueOfBooksToIndex() for more info.
    **/
    std::queue<    std::map<std::string, std::string>   > the_queue;



    /**
    The status queue. Used to shot down the indexing threads. See #FillQueueOfBooksToIndex() for more info.
    **/
    std::queue<bool> status_queue;


    /**
    Protects access to #status_queue and #the_queue
    **/
    boost::mutex the_queue_access_mutex;


    //boost::mutex the_database_access_mutex;

    /**
    Protects access to #xapian_writable_db (xapian does not allow simultaneous writing).
    **/
    boost::mutex xapian_database_access_mutex;


    //boost::mutex extract_text_mutex;

    /**
    Used inside of #SayHi() only for debug. See #SayHi.
    **/
    boost::mutex hi_mutex;


    /**
    Protects access to #unknown_error.
    **/
    boost::mutex unknow_error_mutex;


    /**
    Used only for debug. Just prints the message.
    The code of the function is very simple:
    @code
    void SayHi(std::string text)
        {
        KHZNIK_DEBUGMESSAGE
            (
            boost::mutex::scoped_lock lock_hi(hi_mutex);
            std::cout << "\n" << text << "\n";
            )
        }
    @endcode
    **/
    void SayHi(std::string text)
        {
        KHZNIK_DEBUGMESSAGE
            (
            //boost::mutex::scoped_lock lock_hi(hi_mutex);
            std::cout << "\n" << text << "\n";
            )
        }



    /**
    Log file
    **/
    std::ofstream log_file;


    /**
    protects log_file;
    **/
    boost::mutex log_file_mutex;

    /**
    Used by #FillQueueOfBooksToIndex() to notify indexing threads (#FillQueueOfBooksToIndex())
    that book was added to #the_queue.
    Let's explain a little. It could happen than #the_queue would get empty
    (If indexers threads work faster than the main thread
    (#FillQueueOfBooksToIndex() ) that puts books to #the_queue.).
    If this happens, than indexer threads would go for a sleep.
    They sleep until
    @code
    the_condition_variable_book_added_to_queue.notify_one();
    @endcode
    gets called by #FillQueueOfBooksToIndex() when book is added to #the_queue;
    **/
    boost::condition_variable the_condition_variable_book_added_to_queue;


    /**
    Used by #GetBookFromQueueAndIndexIt() to notify the main thread that puts
    books to #the_queue ( #FillQueueOfBooksToIndex() ) that book was removed
    from #the_queue.
    Let's explain a little. It could happen that the size of #the_queue would
    become equal to #max_queue_size (If indexer threads work slower than the
    main thread that puts books to the queue). If this happens, than the main
    thread that puts books to #the_queue would go for a sleep.
    It sleeps until
    @code
    the_condition_variable_book_removed_from_queue.notify_one();
    @endcode
    gets called by #GetBookFromQueueAndIndexIt() when book is removed from #the_queue.
    **/
    boost::condition_variable the_condition_variable_book_removed_from_queue;



    /**
    Used by #FillQueueOfBooksToIndex()  to extract books from the main
    kolkhoznik (sqlite) database.

    @note used only inside of #FillQueueOfBooksToIndex, does not need mutex protection
    (we run only one thread of #FillQueueOfBooksToIndex ).
    **/
    Khz::Libs::DB::get_books_from_db_by_query *database_asker;

    /**
    Text extractor to extract the text layer from books.
    But this extractor is not used to extract the text layer.
    So, WTF??
    When constructing the FullTextSearchIndexer class object,
    we initialize #text_extractor. Before using indexer, "user"
    have to call #AreTextExtractorsWorking() (see FullTextSearchIndexer usage example).
    #AreTextExtractorsWorking() checks if #text_extractor is working.
    It it does not work, than we should never start the indexing ((see FullTextSearchIndexer usage example).).
    Now let's consider what happens if #text_extractor is working.
    When user creates indexer threads (See FullTextSearchIndexer usage example),
    each thread (#GetBookFromQueueAndIndexIt() )
    make it's own copy of the WORKING #text_extractor (without several copies we would not
            be able to extract the text layer in parallel).
    So #text_extractor is used to make copies. It is not used to extract the text layer directly.
    **/
    MainTextExtractor text_extractor;
    /**
    protects #text_extractor
    **/
    boost::mutex text_extractor_mutex;


    /**
    List of the supported by the xapian stemmers.
    The first element of the pair is the stemmer,
    the second element of the pair is an empty string.
    Used by IndexBook() to set the Xapian::TermGenerator stemmer, something like this:
    @code
    std::string language =  one_book["Language"];
    std::transform(language.begin(),language.end(), language.begin(), ::tolower); //convert to lower case
    if (StemLanguagesMap.count(language) != 0)
        {
        stemmer=Xapian::Stem(language);
        }
    else if(language=="") // default is english
        {
        stemmer= Xapian::Stem("english");
        }
    else
        {
        stemmer=Xapian::Stem(); // no stemming
        }
    indexer.set_stemmer(stemmer); //indexer is an instance of the Xapian::TermGenerator class.
    @endcode
    **/
    std::map<std::string, std::string> StemLanguagesMap;
    /**
    protects #StemLanguagesMap
    **/
    boost::mutex StemLanguagesMap_mutex;


    /**
    Generates #StemLanguagesMap.
    **/
    void GenXapianStemLanguagesMap();



    /**
    Used by indexing threads (#GetBookFromQueueAndIndexIt()) to write
    indexing data to the xapian database.
    Access to #xapian_writable_db is protected by #xapian_database_access_mutex
    due to the fact than xapian does not support simultaneous writing.
    **/
    Xapian::WritableDatabase *xapian_writable_db;


    /**
    Path to the xapian database. Used by #GetBookFromQueueAndIndexIt()
    to construct  Xapian::Database class object. This object gets to
    #IndexBook() as a parameter. See #IndexBook() for more info.
    **/
    std::string path_to_xapian_database;
    /**
    protects #path_to_xapian_database
    **/
    boost::mutex path_to_xapian_database_mutex;


    /**
    see #IncrCommitCounters
    **/
    double commit_counter=0;
    /**
    protects #commit_counter;
    **/
    boost::mutex commit_counter_mutex;

    /**
    Total number of books in the main kolkhoznik (sqlite) database.
    Used to update the progress bar.
    **/
    long long int total_num_of_books;

    /**
    protects #total_num_of_books
    **/
    boost::mutex total_num_of_books_mutex;


    /**
    Number of books whose indexing is completed.
    Used to update the progress bar.
    **/
    //////////

    long long int processed_books;
    /**
    Used inside of #GetBookFromQueueAndIndexIt() to prevent
    #update_progress_bar_callback to be called simultaneously.
    Protects #processed_books.
    **/
    boost::mutex processed_books_mutex;


    /**
    Copy of the constructor \a _update_progress_bar_callback
    parameter.
    Int is the progress in percent , std::string is message (something like
                                    "processed 100 books, remains 1000 books")
    We execute this function each time when one book indexing finishes.
    You should provide pointer to the callback in the constructor.
    Used to update a progress bar.

    Let's describe the third parameter (bool).
    We use it to stop the indexing if user presses the "Cancel" button of the progress bar.
    We do it like this: we call #update_progress_bar_callback like this:
    @code
    update_progress_bar_callback(temp_num, message, &force_stop);
    @endcode
    So, "external wold" (class that has created the indexer) can change value of #force_stop.
    #FillQueueOfBooksToIndex() checks #force_stop from time to time. If #force_stop is true, than
    #FillQueueOfBooksToIndex() puts "false" to #status_queue, as a result all of the indexing threads would finish.
    So, the indexing process would stop.

    @note DO NOT CHANGE #force_stop in the callback function if you do not want to stop the indexing (
    i.e. do not set it to "false"). The reason for this is the following: #force_stop is also used internally
    to stop the indexing in case of database writing error. If you set #force_stop to "false" manually,
    than indexing would not stop in case of error, what is definitely not good.
    **/
    boost::function<void(int, std::string, bool*)> update_progress_bar_callback;

    /**
    Copy of the _indexing_finished_callback constructor parameter.
    This function gets executed when indexing is finished.
    **/
    boost::function<void()> indexing_finished_callback;

    KHZNIK_DEBUGMESSAGE(int debug_temp_num;)

    /**
    Writes data to hard disk in enough data have been accumulated. (if #commit_counter >= #cache_size).
    If simultaneously called by separate threads, this function can freeze the application.
    This is why we always protect this function by #CommitChangesToDbIfNecessary_mutex.
    @param force_commit if true, than we perform commit even if #commit_counter < #cache_size.
    #commit_counter will be set to zero
    **/
    void CommitChangesToDbIfNecessary(bool force_commit=false);


    /**
    protects #CommitChangesToDbIfNecessary. #See CommitChangesToDbIfNecessary for more info
    **/
    boost::mutex CommitChangesToDbIfNecessary_mutex;

    /**
    The code of the function is very simple:
        @code
       void IncrCommitCounter(long long int length_of_the_extracted_doc)
        {
        boost::mutex::scoped_lock commit_lock(commit_counter_mutex);
        commit_counter++;
        this->total_length_of_extracted_docs=
            this->total_length_of_extracted_docs + length_of_the_extracted_doc;
        }
        @endcode
    So, this function increases #commit_counter by  1 and updates #total_length_of_extracted_docs.
      The operation is protected by #commit_counter_mutex.

    #commit_counter and #total_length_of_extracted_docs are used to periodically write data to the hard disk.

    Something like this: ( see #CommitChangesToDbIfNecessary for the actual code)
    @code
    boost::mutex::scoped_lock lock_xapian(xapian_database_access_mutex);
    xapian_writable_db->add_document(doc);
    commit_counter++;
    if(commit_counter>=100 or total_length_of_extracted_docs > 100000)
        {
        xapian_writable_db->commit();
        commit_counter=0;
        total_length_of_extracted_docs=0;
        }
            lock_xapian.unlock();
    @endcode
    So, in the example above(this is just example) we commit every 100 books or every 100000 bytes.

    Let me explain a little. We used to use only #commit_counter before, we did not use #total_length_of_extracted_docs.
    But we run into a very serious problem: if we set our indexer to commit every 1000 docs, than
    the indexing process runs  smoothly while indexing small files.
     But one moment it hits a bunch of bigger documents,
     as a result the RAM usage increases drastically, we even can run out of the available memory.

     In order to fix this problem we use #total_length_of_extracted_docs (terms number, see Xapian::Document::termlist_count).
     We commit(by default) every 1000 documents or every 1000*length_of_typical_book bytes.
     length_of_typical_book is determined experimentally (see #CommitChangesToDbIfNecessary for the actual code).
    **/
    void IncrCommitCounters(unsigned long long int length_of_the_extracted_doc)
        {
        boost::mutex::scoped_lock commit_lock(commit_counter_mutex);
        commit_counter++;
        this->total_length_of_extracted_docs=
            this->total_length_of_extracted_docs + length_of_the_extracted_doc;
        }


    /**
    While indexing a book, we lock this mutex to get NON-exclusive access. The whole #IndexBook function
    is protected by this mutex, like this:
    @code
    bool FullTextSearchIndexer::IndexBook
                    (
                     std::map<std::string, std::string> one_book,
                     MainTextExtractor &Text_Extractor,
                     int xapian_readable_database_num,
                     std::string md5_of_kznik_db
                    )
    {
    boost::shared_lock<boost::shared_mutex> lock_commits(commit_mutex); // no commit during book's indexing
    // some stuff here
    }
    @endcode
    This means that we can read the xapian database during the indexing process
    (to check if the book is already in the database).
    Also we can write to #xapian_writable_db (of course, writing is exclusively protected by
    #xapian_database_access_mutex).
    When performing commit to hard disk (see #CommitChangesToDbIfNecessary) we EXCLUSIVELY lock this mutex.
    This completely stops indexing (we cannot modify #xapian_writable_db (it would be a bad idea
    to modify it during writing to hard drive), we cannot read
    anything  (reading can lead to exception as the database is changing)

    So, #CommitChangesToDbIfNecessary waits for all of the #IndexBook functions (different threads) to finish,
    only then it starts commiting.
    **/
    boost::shared_mutex commit_mutex;


    /**
    Each thread have it's own readable xapian database. This database is used to check
    out if the book that we are indexing is already in the database.
    So, each indexing thread (more precisely, #GetBookFromQueueAndIndexIt) creates it's own
    copy of the Xapian::Database object (readable db, the database is created when starting the thread).
    Pointer to the database is stored inside of #readable_databases_pointers. Each thread knows index of
    it's readable database. The database can be accessed by #readable_databases_pointers [index].
    ##CommitChangesToDbIfNecessary reopens all of the databases listed in #readable_databases_pointers
    after commit is completed.
    **/
    std::vector<Xapian::Database *> readable_databases_pointers;


    /**
    See #IncrCommitCounters
    **/
    unsigned long long int total_length_of_extracted_docs;


    /**
    The function accepts three vectors, something like this:
    | ids | md5s | hashs |
    |--: |--:  |--:   |
    |id 1|md5 1|hash 1|
    |id 2|md5 2|hash 2|
    |id 3|md5 3|hash 3|
     Checks if book with particular md5 and hash should be removed from the
    full text search index. If should, than the corresponding element from ids gets
    pushed to vector_to_add_id_for_removal. This function
    does this for all of the elements in the above three vectors.
    @param ids id of the book in the xapian db
    @param md5s md5 of the file
    @param hashs hash. Calculated ala sha1("title"+"_"+"author"+"_"+"topic"+...)
    @param vector_to_add_id_for_removal we push ids of the books that should be removed here.


    Let me explain a little how this function works. When creating the main sqlite database,
    we add a column with the corresponding md5 of the file as well as a column with the hash.
    When creating the xapian database, we save md5 and hash as document's values (see
    Xapian::document::get_value() ).
    Before we start indexing, we remove all of the obsolete books from the xapian database (if the incremental index option is specified).
    We do it like this: we cycle through all of the documents in the xapian database and extract ids, md5s, hashs.
    Then we query the main sqlite database and check for each book if a book with this md5 and hash is presented in the sqlite database.
    If it is not presented, than we should remove this book from the full text search index. So we push the id of the book
     to vector_to_add_id_for_removal. #RemoveObsoleteBooksFromDb would later remove all of the books that
    are marked for removal from the xapian database.
    **/
    void PrepareListOfDocsToRemove
                (
                std::vector<Xapian::docid> *ids,
                std::vector<std::string> *md5s,
                std::vector<std::string> *hashs,
                std::vector<Xapian::docid> *vector_to_add_id_for_removal
                );
};



}}}// namespaces
