/*
Copyright (C) Ivan Ivanov, 2015

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/



#include "TextFromFileExtractor_parent.h"
#include "SelectAppsToExtractTextWindow.h"
#include "ArchiveExtractorByCommand.h"


namespace Khz{ namespace FTS { namespace Engine{

/**
@brief Extracts text layer from books. Unpacks archives if necessary.

- If file is just a book (for example pdf, djvu), than this class just extracts
the text layer from it.
- If file is an archive, than this class unpacks
the archive (recursively, so all of the subarchives get unpacked too)  and
extract text layers from all of the files. The final text is the
result of the concatenation of the text layers.
**/
class MainTextExtractor
{
public:
    /**
    Constructor
    @param temp_dir EMPTY!!! temporary directory to extract archives.
    @param text_extractors_commands commands to extract the text layer.
    The first element of the pair is extension, the second element of the pair
    is command that will be used as a parameter of the #Khz::FTS::Engine::TextFromFileExtractorByCommand constructor.
    See  #Khz::FTS::Engine::TextFromFileExtractorByCommand for details.
    @param archive_extractors_commands commands to extract archives.
    The first element of the pair is extension, the second element of the pair
    is command that will be used as a parameter of the #Khz::FTS::Engine::ArchiveExtractorByCommand constructor.
    See  #Khz::FTS::Engine::ArchiveExtractorByCommand for details.
    **/
    MainTextExtractor
                    (
                    std::string temp_dir,
                    std::vector<std::pair<std::string, std::string> > &text_extractors_commands,
                    std::vector<std::pair<std::string, std::string> > &archive_extractors_commands
                    );

    /**
    Copy constructor
    **/
    MainTextExtractor(const MainTextExtractor &obj);
    /**
    Destructor
    **/
    ~MainTextExtractor();


    /**
    Checks if all of the text layer extractors and all of the archive Extractors are working
    (see #Khz::FTS::Engine::TextFromFileExtractorByCommand::IfWorking,
    #Khz::FTS::Engine::ArchiveExtractorByCommand::IfWorking).
    If some of the extractors are not working, than this function calls #ask_user_to_install_something().
    #ask_user_to_install_something() shows gui window (#Khz::FTS::Gui::SelectAppsToExtractTextWindow)
    asking user to provide the full path
    to the executable for all of the extractors that
    are not working. When user do this, #ask_user_to_install_something() reads provided by the user values and fills
    \a external_applications
    vector for all of the extractors (see #Khz::FTS::Engine::TextFromFileExtractorByCommand::external_applications,
    #Khz::FTS::Engine::ArchiveExtractorByCommand::external_applications). After this is done, this function calls
    itself recursively once again and everything starts from the beginning.

    P.S. We start indexing only if all of the extractors (text layer extractor, archive extractors) are working.

    P.P.S read #Khz::FTS::Engine::ArchiveExtractorByCommand::external_applications for more detailed description.
    **/
    bool extractors_working();

    /**
    Extracts text layer from \a full_path_to_file. If the file is an archive, than this function unpacks it and
    extracts text layers from all of the files in the archive (strings gets concatenated)
    @param unique_id Some unique id. We use this only if \a full_path_to_file is archive. The unique id
    is used as a directory name to extract all of the files from the archive
    (we create this dir inside of #temp_dir_with_last_separator)
    Currently we use ID from the main kolkhoznik database.
    @note \a unique_id MUST NOT have "_" symbols. See #exctact_archive_recursivelely_if_necessary
    @param extension extension of the file
    @param full_path_to_file full path to file (with extension).

    @return 1 if successful,  -1 if we do not know how to
    extract the text  layer from the filetype; 0 if an error during the execution of the external command occurred


    @note BEFORE calling this function you MUST check if \a full_path_file exists.
    Otherwise this function would show a popup window displaying the error message.
    The popup window is modal, so indexing would stop until user closes the window.
    **/
    int extract_text(std::string unique_id, std::string extension, std::string full_path_to_file);

    /**
    Pointer to the extracted text. We use this to access the result of the text extraction.
    This var gets renewed every time #extract_text gets called.
    **/
    std::string * ptr_to_extracted_text;


        /**
    Returns the length of the extracted text, in terms of bytes.
    **/
    long long int GetLengthOfExtractedText()
        {
        return ptr_to_extracted_text->length();
        }



private:
    /**
    list of the text layer extractors. The first element of the pair is an extension,
    the second element of the pair is pointer to the extractor that extract
    text layer from files with the given extension.
    **/
    std::vector<std::pair<std::string, TextFromFileExtractor_parent * > > text_extractors_list;


    /**
    list of the archive extractors. The first element of the pair is an extension,
    the second element of the pair is pointer to the extractor that extract
    archive with the specified extension.
    **/
    std::vector<std::pair<std::string, ArchiveExtractorByCommand* > >    archive_extractors_list;

    /**
    subsidiary vector to exchange data between #extractors_working() and #ask_user_to_install_something().
    Used only inside of these two functions.
    **/
    std::vector<std::pair<std::string, std::string> > empty_apps;

    /**
    subsidiary vector to exchange data between #extractors_working() and #ask_user_to_install_something().
    Used only inside of these two functions.
    **/
    std::map<std::string, int> empty_apps_helper;



    /**
    Used inside of #extractors_working(). See #extractors_working().
    **/
    bool ask_user_to_install_something();



    /**
    Temporary directory where we are gonna to unpack archives to.
    It's just \a temp_dir constructor parameter with the last
    separator appended.
    **/
    std::string temp_dir_with_last_separator;

    /**
    Generates list of files in the directory recursively (looks inside of subdirs).
    @param full_path_to_dir full path to the directory
    @param output we put output here. The first element of the pair  is  full path to the file,
    the second element of the pair is extension of the file.
    **/
    void gen_list_of_files_in_dir(std::string full_path_to_dir, std::vector<std::pair<std::string, std::string>> &output);



    /**
    Used only if file that we are indexing is an archive holding multiple files.
    In this case we unpack the archive (recursively, so subarchives gets unpacked too),
    than we extract the text layer from all of the files inside of this archive.
    The final text is the result of the concatenation of the text layers from different files.
    So, this var hold the result of such concatenation.

    P.S. If file that we are indexing is an archive holding multiple files, than we set
    @code
    this->ptr_to_extracted_text=&(this->additional_extracted_text);
    @endcode
    Otherwise we set #ptr_to_extracted_text to Khz::FTS::Engine::TextFromFileExtractor_parent::GetPtrToExtractedText(),
    something like this
    @code
    this->ptr_to_extracted_text =extractor_used_to_extract_text_for_this_filetype->GetPtrToExtractedText();
    @endcode
    **/
    std::string additional_extracted_text;


    /**
    Extracts archive and all it's subarchives.
    The archive gets extracted only if an external command is specified for this
    filetype ( <strong class="paramname">input_file</strong>.second is the extension of the file,
             see the parameters list below).
    @note list of supported archives is defined during the construction time,
    see <strong class="paramname">archive_extractors_commands</strong> of the #MainTextExtractor.

    The archive gets extracted to
    @code
    std::string output_dir= output_dir_with_last_separator + unique_id;
    @endcode
    Subarchives gets extracted to dirs that are named something like this
    @code
    std::string subarchive_output_dir1= output_dir_with_last_separator + unique_id + "_" +some_int_num;
    // some_int_num denotes some integer value;

    std::string subarchive_output_dir2= output_dir_with_last_separator +
        unique_id + "_" +some_int_num
            + "_" +some_int_num2;
    @endcode
    So, \a unique_id must  not contain any of the "_" symbols, otherwise collision could occur.
    @param output_dir_with_last_separator see text above
    @param unique_id see text above
    @param input_file full path to the archive. The first element of the pair
    is full path to the file, the second element of the pair is extension.
    @param produced_files OUTPUT!!! list of files that were produced during the archive
    unpacking (this list includes all of the files from subarchives).
    The first element of the pair is the full path to the file,
    the second element of the pair is the extension.
    @param produced_dirs OUTPUT!!! directories (inside of \a output_dir_with_last_separator)
    that were created during the archive extraction. We normally want to delete
    all of these dirs(with content) after we extracted the text layer
    from all of the produced files.
    **/
    bool exctact_archive_recursivelely_if_necessary
        (
        std::string output_dir_with_last_separator,
        std::string unique_id,
        std::pair<std::string,std::string> input_file,
        std::vector<std::pair<std::string, std::string>> &produced_files,
        std::vector<std::string> &produced_dirs
        );


    /**
    When extracting double archives (let's consider tar.gz for example),
    we extract them in two runs: First we extract .gz, after than we extract .tar.
    To make .gz extractor (Khz::FTS::Engine::ArchiveExtractorByCommand) to extract
    tar.gz, we should "deceive" extractor and tell him that the extension of the
    archive is gz.
    So, this function gets only the last extension (for "tar.gz" it would return "gz")
    **/
    std::string get_extension_from_doulbe_extension(const std::string &ext);

};



}}}//namespaces
