/*
Copyright (C) Ivan Ivanov, 2015

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/



#include "TextFromFileExtractor_parent.h"


namespace Khz{ namespace FTS { namespace Engine{


/**
@brief extracts text layer by executing external command. Used inside of MainTextExtractor

Let us consider the usage example
@code
TextFromFileExtractorByCommand djvu_extractor("%edjvutxt %f");
//executable starts with %e, %f denotes the full path to file.

std::string full_path_to_djvu = "/home/ivan/test.djvu";
djvu_extractor.extract_text(full_path_to_djvu);

std::string extracted_text= *( djvu_extractor.GetPtrToExtractedText() );



std::string file2=/home/ivan/test2.djvu;
djvu_extractor.extract_text(file2);
std::string extracted_text2 = *( djvu_extractor.GetPtrToExtractedText() );
@endcode

@note the external command MUST return a UTF8 encoded string.
@note we can set the full path to the external executable. See #external_applications.
**/
class TextFromFileExtractorByCommand : public TextFromFileExtractor_parent
{
public:

    /**
    constructor
    @param command the command to extract the text layer, for example "%edjvutxt %f".
    \%e denotes the executable, \%f denotes the full path to the file.
    See this class detailed description for more info.
    **/
    TextFromFileExtractorByCommand(std::string command);



    virtual bool extract_text (std::string filename);





    /**
    Checks if the extractor is working.
    This function extracts a text layer from \a test_file and compares it with
    the content of the \a  file_with_expected_result.
    If content of the \a file_with_expected_result is substring
    of the extracted text, than this function returns true.  Otherwise,
    this function clears all of the second elements of #external_applications and returns false
    @param test_file full path to the sample file.
    @param file_with_expected_result full path to the .txt file with expected result.
    **/
    virtual bool IfWorking (std::string test_file, std::string file_with_expected_result);


private:
    /**
    Pattern that is used by #construct_full_command. If, for example, the constructor argument is "%edjvutxt %f",
    than pattern would be "%e %f"
    See #Khz::FTS::Engine::ArchiveExtractorByCommand::external_applications to understand "WHY"

    P.S. #Khz::FTS::Engine::ArchiveExtractorByCommand::external_applications is analog of #external_applications.
    Everything that is written in #Khz::FTS::Engine::ArchiveExtractorByCommand::external_applications holds true for
    this class too.
    **/
    std::string pattern;

    /**
    constructs the full command to extract the text layer.
	The code of this function is something like this:
    @code
    std::string TextFromFileExtractorByCommand::construct_full_command(
            std::string path_to_file, std::string executable)
    {
    if (pattern=="") return "";

    std::string output=pattern;
    Khz::Libs::Mstr::replaceAll(output, "%e", "\""+executable+"\"");
    Khz::Libs::Mstr::replaceAll(output, "%f", "\""+path_to_file+"\"");

    return output;
    }
    @endcode
    See #Khz::FTS::Engine::ArchiveExtractorByCommand::external_applications to understand "WHY"

    P.S. #Khz::FTS::Engine::ArchiveExtractorByCommand::external_applications is analog of #external_applications.
    Everything that is written in #Khz::FTS::Engine::ArchiveExtractorByCommand::external_applications holds true for
    this class too.
    **/
    std::string construct_full_command(
            std::string path_to_file, std::string executable);
};


}}}// namespaces
