/*
Copyright (C) Ivan Ivanov, 2015

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/




#include <boost/thread.hpp>
#include <queue>
#include <boost/thread/mutex.hpp>
#include <boost/thread/condition.hpp>
#include "Mdb_fuctions.h"
#include "get_books_from_db_by_query.h"
#include "MainTextExtractor.h"
#include "FullTextSearchIndexer.h"
#include "MyProgressDialog.h"
#include <wx/hyperlink.h>
namespace Khz{ namespace FTS {

    /**
    @brief Full text search gui classes
    **/
    namespace Gui{


/**
@brief The main indexer window.

Allows user to enter a path to the xapian database, path to dir with books and so on.
Launches the full text search indexing.
**/
class FullTextSearchIndexerWindow  : public wxFrame
{
public:
    /**
    @brief Constructor, creates the window
    @param parent standard wxwidgets parent
    @param title the title of the window.
    **/
    FullTextSearchIndexerWindow(wxWindow *parent, const wxString& title);


    /**
    destructor
    **/
    ~FullTextSearchIndexerWindow(){delete config;};
    //wxSize DoGetBestClientSize( ) const;
private:


    /**
    Executes when user presses #cancel_button.
    Closes the window.
    **/
    void OnCancelPressed(wxCommandEvent& event);


    /**
    Launches the indexing.
    Creates an instance of the Khz::FTS::Engine::FullTextSearchIndexer class, something like this:
    @code
    text_indexer= new Khz::FTS::Engine::FullTextSearchIndexer
                                    (
                                    xapian_path.ToStdString(),
                                    path_to_log,
                                    path_to_database.ToStdString(),
                                    path_to_library.ToStdString(),
                                    gen_text_extractors_commands(),
                                    gen_archive_extractors_commands(),
                                    temp_dir.ToStdString(),
                                    callback_update_progress, // goes by reference
                                    callback_finish_indexing //goes by reference
                                    );
    @endcode
    Next, it creates necessary threads:
    @code
    for (int i=0; i< threads_number; i++)
        {
        indexing_threads.push_back
            (
            new boost::thread
                (
                &Khz::FTS::Engine::FullTextSearchIndexer::GetBookFromQueueAndIndexIt, text_indexer
                )
            );

        text_indexer->Incr_num_of_currently_working_threads();
        }
    // IMPORTANT!!! indexing threads MUST be crated before queue_prepare_thread
    queue_prepare_thread= new boost::thread(&Khz::FTS::Engine::FullTextSearchIndexer::FillQueueOfBooksToIndex, text_indexer);
    @endcode
    Executes when user presses #ok_button.

    P.S. See #Khz::FTS::Engine::FullTextSearchIndexer for more info
    **/
    void Index(wxCommandEvent &event);


    /**
    Input that allows user to specify the number of the indexing threads.
    **/
    wxSpinCtrl *threads_number_input;

    /**
    Executes when user clicks "what is this" hyperlink near of #threads_number_input.
    Shows message explaining user what the threads number is."
    **/
    void OnWhatIsThreadsNumberHyperlinkClicked(wxHyperlinkEvent &event);


    /**
    The "cache size" input;
    **/
    wxSpinCtrl* cache_size_input;

    /**
    Executes when user clicks "what is this" hyperlink near of #cache_size_input.
    Shows message explaining user what the cache size is."
    **/
    void OnWhatIsCacheSizeHyperlinkClicked(wxHyperlinkEvent &event);


    /**
    Input that allows user to specify the full path to the directory with books.
    **/
    wxTextCtrl *library_dir_input;


    /**
    Input that allows user to specify path to the main(sqlite) kolkhoznik database.
    **/
    wxTextCtrl *the_main_database_path_input;


    /**
    Input that allows user to specify where to save the log file.
    **/
    wxTextCtrl *where_to_store_log_input;



    /**
    Input to specify where to save the generated index.
    **/
    wxTextCtrl *xapian_database_path_input;


    /**
    "Choose" log file button. Launches #DoKolhozLogPathChoose().
    **/
    wxButton *choose_path_to_log_button;



    /**
    Launches #Index().
    **/
    wxButton *ok_button;


    /**
    Closes the window. Executes #OnCancelPressed()
    **/
    wxButton *cancel_button;


    /**
    Input that allows user to select the temp dir where archives are
    gonna to be unpacked into.
    **/
    wxTextCtrl *temp_dir_input;


    /**
    Opens dir selection dialog that allows to specify the
    directory with books. Puts data to #library_dir_input.
    Executes when user presses the "Choose" button near of the #library_dir_input.
    **/
    void DoKolhozPathChoose(wxCommandEvent& event);



    /**
    Opens the file selection dialog that allows user to select the main
    kolkhoznik (sqlite) database.
    Puts data to #the_main_database_path_input. Executes when user
    presses "Choose" button near of the #the_main_database_path_input.
    **/
    void DoKolhozDatabasePathChoose(wxCommandEvent &event);


    /**
    Opens the file selection dialog that allows user to select a log file
    to write to. Puts the full path to the log file into #where_to_store_log_input.
    Executes when user presses #choose_path_to_log_button.
    **/
    void DoKolhozLogPathChoose(wxCommandEvent &event);


    /**
    Opens dir selection dialog that allows user to specify the directory
    for the full text search index. Puts data to #xapian_database_path_input.
    Executes when user presses "Choose" button near of the #xapian_database_path_input.
    **/
    void DoXapianDatabasePathChoose(wxCommandEvent &event);

    /**
    Opens dir selection dialog that allows user to specify the temporary dir
    where we are gonna to extract archives. Puts data to #temp_dir_input.
    Executes when user presses "Choose" button near of the #temp_dir_input.
    **/
    void DoTempDirPathChoose(wxCommandEvent &event);



    /**
    Config associated with the main config file.
    **/
    wxFileConfig *config;


    /**
    Reads the main config file and generates the text extractors commands for the
    #Khz::FTS::Engine::FullTextSearchIndexer constructor.
    See \a text_extractors_commands parameter of the #Khz::FTS::Engine::FullTextSearchIndexer constructor.
    **/
    std::vector<std::pair<std::string, std::string>> gen_text_extractors_commands();


    /**
    Reads the main config file and generates the archive extractor commands for the
    #Khz::FTS::Engine::FullTextSearchIndexer constructor.
    See \a archive_extractors_commands parameter of the #Khz::FTS::Engine::FullTextSearchIndexer constructor.
    **/
    std::vector<std::pair<std::string, std::string>> gen_archive_extractors_commands();

    /**
    Shows message box explaining which filetypes we can index.
    **/
    void show_info_message_showing_which_filetypes_we_can_index();


    /**
    Progress bar
    **/
    MyProgressDialog *progress_bar;

    /**
    Gets executed when the indexing is finished.
    We bind this function to #callback_finish_indexing, something like:
    @code
    callback_finish_indexing=boost::bind(&Khz::FTS::Gui::FullTextSearchIndexerWindow::FinishIndexingCallBAck, this);
    @endcode
    Next, it goes to the #Khz::FTS::Engine::FullTextSearchIndexer constructor (see the
    \a _indexing_finished_callback parameter of the #Khz::FTS::Engine::FullTextSearchIndexer class).
    Khz::FTS::Engine::FullTextSearchIndexer executes this function when the indexing is finished.

    This function just sends a event that causes the #FinishIndexing function execution. #FinishIndexing
    closes #progress_bar and shows a message informing user that the indexing is finished. We cannot close
    #progress_bar from inside of the #FinishIndexingCallBAck function: it causes an error on Linux (but it
    works on Windows, i have no idea why). So, to ensure the Linux compatibility, we just send a event, nothing
     more.
    **/
    void FinishIndexingCallBAck();


    /**
    Updates progress bar #progress_bar. We bind this function to #callback_update_progress,
    something like this:
    @code
    callback_update_progress=boost::bind(&Khz::FTS::Gui::FullTextSearchIndexerWindow::UpdateProgressDialogCallBack, this, _1, _2);
    @endcode
    Next, it goes to the &Khz::FTS::Engine::FullTextSearchIndexer constructor (see the
    \a _update_progress_barcallback parameter of the #Khz::FTS::Engine::FullTextSearchIndexer class).
    Khz::FTS::Engine::FullTextSearchIndexer executes this function when we need to update a progress bar.

    This function just sends a event that causes the #UpdateProgressBar function execution. #UpdateProgressBar
    updates #progress_bar. We cannot update
    #progress_bar from inside of the #UpdateProgressDialogCallBack function: it causes an error on Linux (but it
    works on Windows, i have no idea why). So, to ensure the Linux compatibility, we just send a event, nothing
    more.

    @param percents progress in percents (will be provided by &Khz::FTS::Engine::FullTextSearchIndexer)
    @param message (will be provided by &Khz::FTS::Engine::FullTextSearchIndexer)
    @param stop if #we_would_like_to_stop_indexing is true, than this function will set
    \a stop to "true" (we get POINTER). As a result, #Khz::FTS::Engine::FullTextSearchIndexer::force_stop
    would become "true". Generally speaking, by changing this parameter we signal #Khz::FTS::Engine::FullTextSearchIndexer
    that user wants to stop indexing. See
    #Khz::FTS::Engine::FullTextSearchIndexer::force_stop for more info.
    **/
    void UpdateProgressDialogCallBack(int percents, std::string message, bool *stop);


    /**
    array of the indexing threads. See Khz::FTS::Engine::FullTextSearchIndexer for more info.
    **/
    std::vector<boost::thread *> indexing_threads;

    /**
    thread that prepares queue of books to index. See Khz::FTS::Engine::FullTextSearchIndexer for more info.
    **/
    boost::thread *queue_prepare_thread;

    /**
    See #UpdateProgressDialogCallBack
    **/
    boost::function<void(int, std::string, bool*)> callback_update_progress;


    /**
    See #FinishIndexingCallBAck()
    **/
    boost::function<void()> callback_finish_indexing;


    /**
    Pointer to the main indexer. We create the indexer inside of the Index(wxCommandEvent &event) function.
    **/
    Khz::FTS::Engine::FullTextSearchIndexer *text_indexer;


    /**
    The number of the indexing threads. We fill it like this:
    @code
    threads_number = threads_number_input->GetValue();
    @endcode
    **/
    int threads_number;

    /**
    Updates #progress_bar.
    See #UpdateProgressDialogCallBack for more info.
    **/
    void UpdateProgressBar(wxCommandEvent &event);

    /**
    Shows message informing user that the indexing is finished. See FinishIndexingCallBAck() for more info.
    **/
    void FinishIndexing(wxCommandEvent &event);

    /**
    If user presses the "stop indexing" button of #progress_bar than we set this value to true;
    Next, if this value is true, than #UpdateProgressDialogCallBack would "tell" #text_indexer to stop indexing.
    (#UpdateProgressDialogCallBack would set #Khz::FTS::Engine::FullTextSearchIndexer::force_stop to true, see
    #Khz::FTS::Engine::FullTextSearchIndexer::force_stop for more info).
    **/
    bool we_would_like_to_stop_indexing;



    void UpdateProgressBar2(int percents, std::string message);
};



}}}//namespaces
