/*
Copyright (C) Ivan Ivanov, 2015-2016

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/




#ifndef KOLKHOZNIK_FULLTEXTSEARCHRESULTS_HEADER_INCLUDED
#define KOLKHOZNIK_FULLTEXTSEARCHRESULTS_HEADER_INCLUDED

#include <wx/panel.h>
#include <wx/srchctrl.h>
#include <wx/button.h>
#include "LanguageSelectorWxChoice.h"
#include <xapian.h>
#include "MyPaginationClass.h"
#include <wx/sizer.h>
#include <wx/stattext.h>
#include <wx/htmllbox.h>
#include "MywxHtmlWindow.h"
#include "FullTextSearchDbQuery.h"
#include "DEBUGMESSAGE.h"
#include <wx/checkbox.h>
#include "FullTextSearchTopicsList.h"
#include <wx/spinctrl.h>


namespace Khz{ namespace FTS { namespace Gui{



/**
@brief Draws search input, search results and so on.

This class is the main class that manages the full text search user interface.
It is placed inside of the #Khz::FTS::Gui::FullTextSearchWindow.  #Khz::FTS::Gui::FullTextSearchWindow
purpose is to interact with the main kolkhoznik window, to remember the size of the full
text search window. FullTextSearchResult on it's turn draw the search input, the search results and so on.
 @note VERY IMPORTANT!!!!. This class launches a parallel thread while searching. Do not destroy the object
 of this class if a searching process is going on right now, this would definitely lead to a segfault.
 Use #IsIdle() to check if the searching is going on right now.


#######################################################################
#######################################################################
#              THE PAGE GENERATION PROCEDURE:                         #
#######################################################################
#######################################################################

 when user presses the search button, the #OnSearchButtonPressed function gets executed. #OnSearchButtonPressed  calls
 #InitNewSearch. In general, #InitNewSearch prepares #database_asker and #database_asker_2: the search string
 gets parsed, the query that will be used to get data from the database gets constructed (see Khz::FTS::Engine::FullTextSearchDbQuery::InitNewSearch for
 more info). After #InitNewSearch is finished, #InitPageGeneration gets executed. #InitPageGeneration disables all of the available buttons and launches the
 searching. To do this #InitPageGeneration
 calls #database_asker->GiveMePageParallelThread (see #Khz::FTS::Engine::FullTextSearchDbQuery::GiveMePageParallelThread). After this,
 #InitPageGeneration returns, it does not wait for searching to complete (we perform searching in a parallel thread).

 When searching is complete, #database_asker calls #display_search_results_callback
 (#display_search_results_callback goes as the \em finished_callback parameter to
  #Khz::FTS::Engine::FullTextSearchDbQuery::GiveMePageParallelThread).  Actually, #display_search_results_callback() is bound to
  display_search_results_callback_void(), something like this:
    @code
    this->display_search_results_callback=boost::bind(&Khz::FTS::Gui::FullTextSearchResults::display_search_results_callback_void, this);
    @endcode
    #display_search_results_callback() is only required to perform the binding. So, the actual callback is #display_search_results_callback_void().
    Wxwidgets forbids GUI manipulation from a parallel thread. So, #display_search_results_callback_void() only emits a event that informs the
    main thread that the searching is complete. When catching this event, our main thread launches #FinishPageGeneration(wxCommandEvent &event).

 On it's turn, #FinishPageGeneration reads search results from #database_asker and draws the search results page. After this, #FinishPageGeneration
 calls
 @code
 database_asker_2_is_searching_right_now=true;
 page_num_that_database_asker_2_was_searching_last_or_is_searching_right_now=this->current_page+1;
 database_asker_2.GiveMePageParallelThread(this->current_page+1, display_search_results_callback_2, &InitPageGenerationStatus_2);
 @endcode
 So, we start to search for books that are required to display the next page immediately. This what the #database_asker_2 is for.
 The reason for the procedure is the performance considerations: while user studies the search results on the current page, we prepare the next page.
 When user will press the "Next" button in order to display the next page, this next page will be ready!!!.
 Of course, the search for the next page is performed in the background (parallel thread). The search
 is performed by #database_asker_2.

 So, what happens when #database_asker_2 finishes the searching that was launched by the code above? Obviously, #display_search_results_callback_2 gets executed.
 Actually, #display_search_results_callback() is bound to
  #display_search_results_callback_void_2(), so #display_search_results_callback_void_2() gets executed. Similar to
  #display_search_results_callback_void(), #display_search_results_callback_void_2()  emits a event that informs the
 main thread that the searching is complete. When catching this event, our main thread launches #FinishPageGeneration(wxCommandEvent &event).

 So, when the searching by #database_asker_2 is finished, #FinishPageGeneration gets executed. #FinishPageGeneration
 can determine which of the database askers (#database_asker or #database_asker_2) finished the searching. It does it like this:
 @code
 int num=event.GetInt();
 if(num==1)
    {
    // database_asker have finished the searching
    }
 else if(num==2)
    {
    // database_asker_2 have finished the searching
    }
 @endcode
 The corresponding event's int gets set by #display_search_results_callback_void() or #display_search_results_callback_void_2().


 So, when #database_asker_2 have finished searching, #FinishPageGeneration knows that it is #database_asker_2 who finished the searching.
 At the top of #FinishPageGeneration we have something like this:
 @code

    // database_asker_2 have finished the searching, we update database_asker_2_is_searching_right_now accordingly.
    if(event.GetInt()==2){database_asker_2_is_searching_right_now=false;};


    if(event.GetInt()==2 and !next_page_shold_be_generated_by_database_asker_2)
        {
        return;
        }
 @endcode
 By default #next_page_shold_be_generated_by_database_asker_2 is false, as a result #FinishPageGeneration do nothing. So, #database_asker_2 just finds books
 that belong to the next page; after this #database_asker_2 sits doing nothing.

 Now let us see what would happen if user preses the "Next" button. When user do this, he obviously want to see the next page with the search results.
 When this happens, #OnNextPageButtonPressed gets executed. #OnNextPageButtonPressed checks if #database_asker_2_is_searching_right_now is true;
 Let us consider a case when #database_asker_2_is_searching_right_now if false (#database_asker_2 have finished  searching, now it sits doing noting). If this is the case, than #OnNextPageButtonPressed sets #next_page_shold_be_generated_by_database_asker_2 to true and emits a event that
 makes #FinishPageGeneration to execute. #FinishPageGeneration(wxCommandEvent &event) would find out that \em event.GetInt() is 2 ("2" is the indication of
 #database_asker_2). Next, #FinishPageGeneration would find out that #next_page_shold_be_generated_by_database_asker_2 is true. In this case, #FinishPageGeneration
 would draw the next page using the search results provided by #database_asker_2; after this #FinishPageGeneration sets #next_page_shold_be_generated_by_database_asker_2
 to false and makes #database_asker_2 to search for the next page once again.

 Let us now discuss a case when #database_asker_2_is_searching_right_now is true when #OnNextPageButtonPressed gets executed (#database_asker_2 is still searching).
 In this case, #OnNextPageButtonPressed just sets #next_page_shold_be_generated_by_database_asker_2 to true and returns. When #database_asker_2 will
 finish searching,  #FinishPageGeneration(wxCommandEvent &event) is gonna to be executed (the corresponding event will be generated by #display_search_results_callback_void_2().
  #FinishPageGeneration(wxCommandEvent &event) would find out that \em event.GetInt() is 2 ("2" is the indication of
 #database_asker_2). Next, #FinishPageGeneration would find out that #next_page_shold_be_generated_by_database_asker_2 is true. In this case, #FinishPageGeneration
 would draw the next page using search results provided by #database_asker_2; after this #FinishPageGeneration sets #next_page_shold_be_generated_by_database_asker_2
 to false and makes #database_asker_2 to search for the next page once again.
 **/
class FullTextSearchResults : public wxPanel
    {
    public:

        /**
        Constructor
        @param parent standard wxwidgets parent
        @param path_to_xapian_db full path to the full text search index directory.
        @param path_to_main_config_file full path to the main config file.
        @param _tab_index See #tab_index
        **/
        FullTextSearchResults(wxWindow *parent, std::string path_to_xapian_db, wxString path_to_main_config_file,int _tab_index=0);

        /**
        Could be used to set #tab_index from the outside of this class.
        The code of this function is the following:
        @code
        void SetTabIndex(int id){this->tab_index=id;};
        @endcode
        See #tab_index for more info.
        **/
        void SetTabIndex(int id){this->tab_index=id;};

        /**
        Sets focus to #Search_input;
        Something like
        @code
        Search_input->SetFocus();
        @endcode
        is not working, i have no idea why.
        So we simulate a mouse click to set the focus.
        **/
        void SetFocusToSearchInput();


        /**

        As the description of this class say, we perform searching by using a parallel thread.
        So, this function returns true if such searching process is going on right now, false otherwise.
        **/
        bool IsIdle()
            {
            if(database_asker_2_is_searching_right_now) return false;
            return search_button->IsEnabled();
            };

        /**
        Destructor
        **/
        ~FullTextSearchResults()
            {
            if(topics_list_height_have_changed)
                {
                config->Write("FULL_TEXT_SEARCH/topic_list_height", topics_list_height_changer->GetValue());
                }
            delete config;
            }

        /**
        The search  input
        **/
        wxSearchCtrl *Search_input;


        /**
        Is used to perform searching from the "outside" world. Let's explain a little.
        The full text search window is a standalone window. But the main window of the program have
        the full text search input too. So, when user wants to perform a search from
        the main window, we have to transform data to the full text search window.
        This is what this function is for.

        In general this function does the following:
        - First, it inserts \a search_string into the search input (of the full text search window);
        - Next, it selects appropriate stemmer from the stemmers list (the \a language parameter)
        - Next, it inserts all of the topic listed in \a topics_array to the corresponding
        input of the full text search window ("restrict search by topics" input).
        - Finally, it performs the full text search.
        **/
        void Search(wxString _search_string, wxString _language, std::vector<wxString> &topics_array);



        /**
        Reads config file again and redraws the search results (applies new font sizes, new colors and so on)
        **/
        void ReDraw();


        /**
        Reads config file again and redraws #language_choise.
        **/
        void ReDrawStemmers(){language_choise->ReDraw();};

        /**
        Reconnects to the xapian database.
        **/
        void ReconnectToDB(std::string path_to_xapian_readable_db);

    private:


        /**
        Prepares #database_asker and #database_asker_2: the search string
        gets parsed, the query that will be used to get data from the database gets constructed.
        In general this function does the following:
        It calls #database_asker->InitNewSearch, #database_asker_2->InitNewSearch
        (See Khz::FTS::Engine::FullTextSearchDbQuery::InitNewSearch).

        After the #InitNewSearch function is called, we can use #InitPageGeneration() to start searching.

        See "THE PAGE GENERATION PROCEDURE" section for more info

        @param language Supported by xapian stemmer
        @param query_string search string as it was entered into the search input.
        @param _num_of_books_per_page number of books per page
        @param show_error_messages should we show error messages?
        **/
        bool InitNewSearch
            (
            std::string language,
            std::string query_string,
            Xapian::doccount _num_of_books_per_page,
            bool show_error_messages=true
            );


        /*
        Displays search results belonging to the particular page.
        We can call this function only after #InitNewSearch.
        */
        //void ShowMePage(int page_num);


        /**
        "use stemmer" checkbox. When user clicks this checkbox, #OnUseStemmerCheckboxClicked gets executed.
        **/
        wxCheckBox *full_text_search_use_stemmer_checkbox;


        /**
        The main object that performs queries to the database. This class (FullTextSearchResults)
        just displays results obtained by #database_asker.
        **/
        Khz::FTS::Engine::FullTextSearchDbQuery database_asker;

        /**
        The "Search" button. When user presses this button, #OnSearchButtonPressed gets executed.
        **/
        wxButton *search_button;

        /**
        WxChoise that allows to select stemmer. Placed near of the "use stemmer" checkbox.
        **/
        LanguageSelectorWxChoice *language_choise;


        /**
        Number of the search results per page.
        **/
        int books_per_page=10;


        /**
        The page that is being displayed currently.
        **/
        int current_page;



        /**
        Query string that was used as a parameter of #InitNewSearch.
        I.E. this is the search string that we are currently searching for.
        **/
        std::string query_string;



        /**
        Executes when user presses #search_button.
        In general, this function calls #InitNewSearch
        and #InitPageGeneration (1).
        **/
        void OnSearchButtonPressed(wxCommandEvent &event);



        /**
        Main sizer associated with the control.
        **/
        wxBoxSizer *only_sizer;



        // /*
        // The main paginator. Draws buttons that allow to change page.
        // */
        // MyPaginationClass *paginator;



        // /*
        // Catches event emitted by #paginator when user changes page. Updates the
        // search results accordingly.
        // */
        // void ChangePage(wxCommandEvent &event);

        /**
        wxHtmlWindow that displays search results.
        **/
        MywxHtmlWindow *results_list;


        /**
        Status text, displays how much books was found, which of the books ar shown.
        Is displayed at the top of the page.
        The example of the text: "100 books found, 1-10 shown".
        **/
        wxStaticText *status_text;


        /**
        reads #books_per_page, #current_page. Updates #status_text accordingly.
        @param this_is_last_page indicates that we are displaying the last page.
        If set to true, than we just modify the status string from something like
        "approx. 25 books found" to "exactly 25 books found".
        **/
        void UpdateStatusText(bool this_is_last_page=false);


        /**
        Stemmer that is currently used for the search.
        **/
        std::string language;


        /**
        Main scrolling window that is responsible for the search results scrolling.
        #scrolled holds #Search_input, #status_text, #language_choise,
        #full_text_search_use_stemmer_checkbox, #topics_list, #search_button, #results_list,
        #first_button,
        #next_button,
        #prev_button, #restrict_topics_checkbox. As a result, we can scroll
        all of these element as a single piece.
        **/
        wxScrolledWindow *scrolled;


        /**
        Catches wxMouseEvent events and scrolls #scrolled if user revolves the wheel of the mouse.
        **/
        void OnScrollEvent(wxMouseEvent& event);


        /**
        Executes when user clicks #full_text_search_use_stemmer_checkbox.
        Sets #language to a valid xapian stemmer or to "" depending on the
        checkbox state. Disables/enables #language_choise.
        **/
        void OnUseStemmerCheckboxClicked(wxCommandEvent &event);



        /**
        Shows popup window explaining what stemmer is.
        **/
        void OnWhatIsStemmerLinkClicked(wxHyperlinkEvent& event);


        /**
        List of topics for the  searching (together with buttons "Add" ,"Remove", "Clear").
        (near of the #restrict_topics_checkbox).
        Search is gonna to be restricted by topics listed here.
        **/
        FullTextSearchTopicsList *topics_list;


        /**
        The "restrict search by topics" checkbox. If checked, than
        full text searching is restricted by topics listed in #topics_list.
        When user clicks the checkbox, #OnRestrictTopicsCheckboxClicked gets executed.
        **/
        wxCheckBox* restrict_topics_checkbox;


        /**
        Executes when user clicks #restrict_topics_checkbox.
        Enables/disables #topics_list.
        **/
        void OnRestrictTopicsCheckboxClicked(wxCommandEvent &event);


        /**
        Appends a single html block displaying one book to #results_list.
        @param num the index of the book on the page. This parameter is an int
        that was used to get data from #database_asker, something like this:
        @code
        database_asker.GetTitle(num); // num is the index of the book on the page
        @endcode
        This num is stored in the title href. When user clicks book,  we use this num to
        get data that are required to open the book, something like this:
        @code
        database_asker.GetRelPath(num);// we extracted relative path to the book by num.
        @endcode
        @param author the author of the book
        @param title the title of the book
        @param year publication year
        @param topic topic
        @param extension extension
        @param relpath relative path to the book, for example "/B_Biology/BB_Biography, history"
        @param filename filename with extension
        @param abstract See  #Khz::FTS::Engine::FullTextSearchDbQuery::GetAbstract
        **/
        void AppendBookToPage
                (
                int num,
                std::string author,
                std::string title,
                std::string year,
                std::string topic,
                std::string extension,
                std::string relpath,
                std::string filename,
                std::vector<std::pair<std::string, int>> abstract
                //long long int *id,
                //int* weight_in_percents
                );

        /**
        The window of this class have the following structure:
        we have the top level scrolled window #scrolled that contains all of the other elements.
        #results_list has no scrollbars
        (wxHW_SCROLLBAR_NEVER style, see #Khz::FTS::Gui::MywxHtmlWindow constructor) and is a child of #scrolled.
        When we create a page displaying search results we do it like this:
        - First, we put all of the content into #results_list.
        - Next, we resize #results_list to fit it's content, something like this:
        @code
        results_list->SetMinSize(wxSize(-1,
                  results_list->GetInternalRepresentation()->GetHeight()));
        results_list->SetSize(wxSize(-1,
                  results_list->GetInternalRepresentation()->GetHeight()));

        @endcode
        - Finally, we recalculate sizes and make #scrolled to redraw scrollbar:
        @code
        scrolled->GetSizer()->RecalcSizes();
        scrolled->GetSizer()->Layout();
        this->GetSizer()->RecalcSizes();
        this->Layout();
        this->Refresh();
        @endcode

        As a result, we would have scrollbars adjusted properly.

        Now, let us assume that user just resized the full text search window.
        It's width could change, leading to changing of the #results_list internal representation height.
        In other words, we should expect the the following value would change:
        @code
        int height= results_list->GetInternalRepresentation()->GetHeight();
        // height would change.
        @endcode

        So, we should resize #results_list manually AFTER!!! user have resized the full text search window.

        We do it like this:
        - we catch wxSizeEvent that gets fired when user resizes the window.
        As a result, #OnSize gets executed. #OnSize just sets #just_resized to true.
        - AFTER resizing is complete, wxwidgets fires wxIdleEvent. We catch this event
        and execute #OnIdle function. If #just_resized is true, than
        #OnIdle resizes #results_list, sets #just_resized to false, refresh the
        whole full text search window.

        By using this method we resize #results_list AFTER!!! resizing of the full text search window.
        **/
        bool just_resized;


        /**
        Config associated with the main config file
        **/
        wxFileConfig *config;


        /**
        Executes when user clicks a link inside of the #results_list.
        If user uses right mouse button, than we show popup menu.
        If user uses left button, than we just open the clicked book.
        **/
        void OnLinkLeftOrRightMouseButtonClicked(wxHtmlLinkEvent& event);


        /**
        Just stops the propagation of the event.
        Without it #results_list would twitch when caught the focus.
        **/
        void OnChildFocus(wxChildFocusEvent& event);


        /**
        Executes when user resizes the window.
        See #just_resized for more info.
        **/
        void OnSize(wxSizeEvent& event);


        /**
        Executes when wxwidgets fires a wxIdleEvent event.
        See #just_resized for more info.
        **/
        void OnIdle(wxIdleEvent &event);


        /**
        Returns path to the icon
        for the particular file extension. Uses #File_icons_map variable to do this. If #File_icons_map does not
        have any information regarding this extension, than this function  updates #File_icons_map accordingly.
        The size of the icon is defined by #m_HtmlConfParams->icons_size.

        Used by #AppendBookToPage when generating the "<img>" tag.
        @param Extension extension of the file.
        **/
        wxString ChooseIcon(std::string Extension);


        /**
        Remembers path to the icon for the particular extension. The first element of the pair (std::string)
        is extension,
        the second element of the pair is path to the icon.
        **/
        std::map<std::string, wxString> File_icons_map;


        /**
        Full path to the main config file.
        **/
        wxString path_to_config;


        /**
        Shows mouse button right click popup menu.
        **/
        void ShowLinkRightClickMenu();


        /**
        When user right clicks book, we show a popup menu. Next, if user chooses the "Info" entry in this menu,
        we should show the info about this book. So, we remember which of the books was right clicked last.
        Next, we show the info about
        this particular book. This is why we have #last_right_clicked_link.

        P.S. We use something like this:
        @code
        database_asker->GetId(last_right_clicked_link);
        // we got id of the book in the main kolkhoznik database.
        @endcode
        **/
        int last_right_clicked_link;


        /**
        When user right clicks book, we show a popup menu. If user selects the "Gen bibtex"
        entry, than this function gets executed. It creates a window that shows bibtex info to user.
        **/
        void OnGenBibtexMenuSelected(wxCommandEvent &event);


        /**
        When user right clicks book, we show a popup menu. If user selects the "Info"
        entry, than this function gets executed. It creates window displaying info about the book.
        **/
        void OnInfoAboutBookWanted(wxCommandEvent &event);


        /**
        @brief html display parameters

        A bunch of parameters that define the font sizes, font colors and so on of the
        search results.
        **/
        class HtmlDisplayParams
            {
            public:

                /**
                Constructor do nothing, use #ReadConfigFile to fill the parameters.
                **/
                HtmlDisplayParams(){};


                /**
                Reads config file and fills parameters
                **/
                void ReadConfigFile(wxFileConfig **config, wxString path_to_config );


                /**
                Title/author font size
                **/
                wxString caption_font_size;

                /**
                Is title/author bold
                **/
                bool is_caption_bold;

                /**
                Title/author color
                **/
                wxString caption_color;

                /**
                should we show info (extension, year and so on).
                **/
                bool show_info;

                /**
                The color of the info string (displaying extension, year and so on).
                **/
                wxString info_color;

                /**
                The font size of the info string (displaying extension, year and so on).
                **/
                wxString info_font_size;

                /**
                Should we display topic? (only if #show_info=true)
                **/
                bool show_topic;

                /**
                Should we display year? (only if #show_info=true)
                **/
                bool show_year;


                /**
                Should we display extension? (only if #show_info=true)
                **/
                bool show_extension;


                /**
                Should we display relative path to the book? (only if #show_info=true)
                **/
                bool show_relpath;


                /**
                Font size of the text sample from the book.
                **/
                wxString snippet_font_size;

                /**
                The color of the search terms in the text sample from the book.
                **/
                wxString highlight_color;

                /**
                The color of the delimiter of the text samples
                **/
                wxString dots_color;

                /**
                Is samples delimiter bold?
                **/
                bool dots_bold;


                /**
                Samples delimiter symbol (3 dots by default)
                **/
                wxString dots_symbol;

                /**
                Are search terms in the text sample from the book bold?
                **/
                bool highlight_bold;

                /**
                Text samples number
                **/
                unsigned int  samples_number;

                /**
                single text sample words number
                **/
                unsigned int sample_size;

                /**
                Scrollrate
                **/
                int scrollrate;

                /**
                Icons size
                **/
                int icons_size;
            };

        /**
        Html display parameters
        **/
        HtmlDisplayParams m_HtmlConfParams;

    /**
    The "First" button. Displays search results belonging to the first page.
    **/
    wxButton *first_button;


    /**
    The "Next" button. Displays search results belonging to the "next" page.
    **/
    wxButton *next_button;

    /**
    The "Previous" button. Displays search results belonging to the "previous" page.
    **/
    wxButton *prev_button;


    /**
    Executes when user presses #next_button. Does something like this (very simplified):
    @code
    this->InitPageGeneration(this->current_page+1);
    @endcode
    **/
    void OnNextPageButtonPressed(wxCommandEvent &event);


    /**
    Executes when user presses #prev_button. Does something like this (very simplified):
    @code
    this->InitPageGeneration(this->current_page-1);
    @endcode
    **/
    void OnPrevPageButtonPressed(wxCommandEvent &event);


    /**
    Executes when user presses #first_button. Does something like this (very simplified):
    @code
    this->InitPageGeneration(1);
    @endcode
    **/
    void OnFirstPageButtonPressed(wxCommandEvent &event);





    /**
    Used to set #display_search_results_callback_void as a callback that gets executed when #database_asker finishes the searching.
    See "THE PAGE GENERATION PROCEDURE" section for more info.
    **/
    boost::function<void()> display_search_results_callback;




    /**
    This goes as a bool parameter ( \em output_status ) to #Khz::FTS::Engine::FullTextSearchDbQuery::GiveMePageParallelThread when it gets called by #InitPageGeneration(),
    something like this:
    @code
    database_asker.GiveMePageParallelThread(page_num, display_search_results_callback, &InitPageGenerationStatus);
    @endcode
    So, as you could get from #Khz::FTS::Engine::FullTextSearchDbQuery::GiveMePageParallelThread,
    #InitPageGenerationStatus gets set to true if there were no errors during the database search that was performed by #database_asker.
    FinishPageGeneration() uses this var to check out if we need to draw the search results at all, something like this:
    @code
    void FullTextSearchResults::FinishPageGeneration(wxCommandEvent &event)
    {
    if(!InitPageGenerationStatus)
        {
        return;
        }
    // the page generation code is omitted here.
    // We draw page only if there were no errors during the database search.
    }
    @endcode
    **/
    bool InitPageGenerationStatus;




    /**


    Disables all of the available buttons and launches the page generation (searching).
    When called, this function only launches the searching and exits, it does not wait for searching to complete (we perform searching in
    a parallel thread).
    In order to launch searching, this function calls #database_asker->GiveMePageParallelThread()
    (see #Khz::FTS::Engine::FullTextSearchDbQuery::GiveMePageParallelThread)

    @note
    See "THE PAGE GENERATION PROCEDURE" section for more info.
    **/
    void InitPageGeneration(int page_num);



    /**
    Draws search results when #database_asker or #database_asker_2 finishes searching. See "THE PAGE GENERATION PROCEDURE"
    section for more info.
    **/
    void FinishPageGeneration(wxCommandEvent &event);



    /**
    We cannot calculate the total number of the matches (the total number of the pages) that are consistent with the search string
    (see #Khz::FTS::Engine::FullTextSearchDbQuery::GiveMePage() for more info). So, we assume that we have no more search results
    if #FinishPageGeneration() finds out the current page is empty. If this is the case, than #FinishPageGeneration()
    launches
    @code
    InitPageGeneration(this->current_page-1)
    @endcode
    and sets #this_is_the_last_page to true. When the previous page search results generation is complete, #this_is_the_last_page is used
    by #UpdateStatusText(): if #this_is_the_last_page is true, than #UpdateStatusText() would produce something like this:
    "exactly 5 results found". If #this_is_the_last_page is false, than #UpdateStatusText() will produce something like this:
    "approx. 5 results found".
    **/
    bool this_is_the_last_page;




    /**
    Callback that gets executed when #database_asker finishes searching.
    See "THE PAGE GENERATION PROCEDURE" section for more info.
    **/
    void display_search_results_callback_void();




    /**
    !!! read the "THE PAGE GENERATION PROCEDURE" section first for the explanation of the page generation procedure.

    When #FinishPageGeneration() is finished with a search results drawing, it emits a event informing the parent class
    (#Khz::FTS::Gui::FullTextSearchWindow) that searching is complete. #Khz::FTS::Gui::FullTextSearchWindow
     uses this event to set the caption of the corresponding tab.
    (While searching, the caption of the the tab is "searching...". When searching is finished, we set the caption of the tab
    to the cropped search string).
    So, the event carries the following information:

    1) the search string

    2) the index of the tab ( to set a caption, #Khz::FTS::Gui::FullTextSearchWindow must know the index of the tab)

    So, this is why we have #tab_index. Tabs are created by #Khz::FTS::Gui::FullTextSearchWindow, each tab
    is represented by Khz::FTS::GUI::FullTextSearchResults (this) class. When creating a tab,
    #Khz::FTS::Gui::FullTextSearchWindow passes the tab index to this class constructor (see the \em _tab_index
    constructor parameter). The constructor of this class stores this info inside of the #tab_index variable.
    When #FinishPageGeneration() emits the event that makes #Khz::FTS::Gui::FullTextSearchWindow to set the caption
    of the tab, #FinishPageGeneration()  uses #tab_index to store the corresponding info inside of the event.
    @note #tab_index can be changed from outside by  #SetTabIndex().

    **/
    int tab_index;



    /**
    Spinctrl that allows change size of #topics_list
    **/
    wxSpinCtrl *topics_list_height_changer;



    /**
    Executes when user changes something inside of #topics_list_height_changer. Reads data from
    #topics_list_height_changer and changes height of #topics_list
    **/
    void ChangeTopicsListHeight(wxSpinEvent& event);



    /**
    Gets set to false by the class constructor. If #ChangeTopicsListHeight gets executed even once, than
    #topics_list_height_have_changed gets set to true. The destructor of this class checks if #topics_list_height_have_changed is true.
    If it is, than destructor writes information about the height of #topics_list  into the config file. When creating the class object,
    #SetTopicsListHeightFromConfigFile() gets called. It reads the config file and restores the height of #topics_list.
    **/
    bool topics_list_height_have_changed;



    /**
    Reads config file and sets height of #topics_list accordingly.
    **/
    void SetTopicsListHeightFromConfigFile();




    /**
    A text informing user how much topics are currently listed inside of #topics_list.
    Something like "0 topics in list"
    **/
    wxStaticText *BooksNumberInTopicsListText;


    /**
    updates #BooksNumberInTopicsListText. The event gets send by FullTextSearchTopicsList.
    The id of the event is my_wx_id__FULL_TEXT_TOPICS_LIST_SOMETHING_CHANGED.
    The new number of the topics is
    @code
    event.GetInt();
    @endcode
    **/
    void UpdateInfoAboutBooksInTopicsList(wxCommandEvent &event);


    /**
    Stores all info about books (search results) that are currently displayed
    **/
    Khz::FTS::Engine::FullTextSearchDbQuery::CurrentPageData current_page_data;


    /**
    Searches for the next page while user is browsing search results displayed on the current page.

    See "THE PAGE GENERATION PROCEDURE" section for more info.
    **/
    Khz::FTS::Engine::FullTextSearchDbQuery database_asker_2;



    /**
    true if #database_asker_2 is searching right now, false otherwise. See "THE PAGE GENERATION PROCEDURE" section for more info.
    **/
    bool database_asker_2_is_searching_right_now;


    /**
    the name of the var is self-explanatory
    **/
    int page_num_that_database_asker_2_was_searching_last_or_is_searching_right_now;



    /**
    Informs #FinishPageGeneration that we want use #database_asker_2 to display search results.
    See "THE PAGE GENERATION PROCEDURE" section for more info
    **/
    bool next_page_shold_be_generated_by_database_asker_2;


    /**
    Used to set #display_search_results_callback_void_2 as a callback that gets executed when #database_asker_2 finishes the searching.
    See "THE PAGE GENERATION PROCEDURE" section for more info.
    **/
    boost::function<void()> display_search_results_callback_2;

    /**
    Callback that gets executed when #database_asker_2 finishes searching. See "THE PAGE GENERATION PROCEDURE" section for more info.
    **/
    void display_search_results_callback_void_2();


    /**
    The same as #InitPageGenerationStatus, but is related to #database_asker_2, not #database_asker.
    **/
    bool InitPageGenerationStatus_2;
    };


}}}// namespaces


#endif //#ifndef KOLKHOZNIK_FULLTEXTSEARCHRESULTS_HEADER_INCLUDED
