/*
Copyright (C) Ivan Ivanov, 2015-2016

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef KOLKHOZNIK_FULLTEXTSEARCHTOPICSLIST_HEADER_INCLUDED
#define KOLKHOZNIK_FULLTEXTSEARCHTOPICSLIST_HEADER_INCLUDED

#include <wx/panel.h>
#include <wx/listbox.h>
#include <xapian.h>
#include <wx/menu.h>


namespace Khz{ namespace FTS { namespace Gui{


/**
@brief a wxListbox combined with "Add", "Remove", "Clear" buttons.
User can add valid kolkhoz topics to the listbox by pressing "Add" button.
Is used to select topics when "restrict search by topics" mode is enabled.
**/
class FullTextSearchTopicsList : public wxPanel
{
public:
    /**
    Constructor
    @param parent standard wxwidgets parent
    @param id standard wxwidgets ID
    @param orientation coud be "vertical" or "horizontal".
    If \a orientation is "vertical", than "Add", "Remove", "Clear" button
    would be placed BELOW of the listbox.
    If orientation is "horizontal", than these buttons would be placed to the right of the listbox.
    **/
    FullTextSearchTopicsList(wxWindow *parent, wxWindowID id, std::string orientation="vertical");


    /**
    Just
    @code
    int GetCountPerPage(){return topics_list_itself->GetCountPerPage();};
    @endcode
    **/
    int GetCountPerPage(){return topics_list_itself->GetCountPerPage();};


    /**
    Returns query than would select all of the books belonging to the topics listed
    in  #topics_list_itself. See #Khz::FTS::Engine::FullTextSearchDbQuery::InitNewSearch
    (the \a restriction_query parameter).
    **/
    Xapian::Query GetXapianQuery();



    /**
    Returns array of topics that are currently listed in the #topics_list_itself listbox.
    @param output output array to fill.
    **/
    void GetArrayOfTopics(std::vector<wxString> &output);


    /**
    Clears #topics_list_itself.
    **/
    void Clear(){topics_list_itself->Clear();};


    /**
    Inserts topics to the #topics_list_itself listbox.
    the code of the function is:
    @code
    void InsertItems(wxArrayString temp_wx_array, unsigned int position)
        {
        topics_list_itself->InsertItems(temp_wx_array,position);
        };
    @endcode

    @param temp_wx_array array of wxstrings to insert.
    @param position All of the items are gonna to be inserted after this position.
    **/
    void InsertItems(wxArrayString temp_wx_array, unsigned int position)
        {
        topics_list_itself->InsertItems(temp_wx_array,position);
        };


    /**
    As the brief description of this class say, this class
    consists of wxListbox combined with "Add", "Remove", "Clear" buttons.
    So, #topics_list_itself is a standard wxLisbox where topics are supposed to
    be listed.
    **/
    wxListBox *topics_list_itself;



private:





    /**
    Executes when user presses "Add" button.
    Opens topic selection dialog than allows user to
    select topic(s).
    Inserts selected topic(s) to #topics_list_itself.
    **/
    void OnAddPressed(wxCommandEvent &event);



    /**
    Executes when user presses "Remove" button.
    Removes the currently selected topics from #topics_list_itself.
    **/
    void OnRemovePressed(wxCommandEvent &event);


    /**
    Executes when user presses "Clear" button. Just calls #Clear().
    **/
    void OnClearPressed(wxCommandEvent &event);


    /**
    Checks if #topics_list_itself contains this topic
    **/
    bool IsTopicInList(wxString topic);

    /**
    Executes right button mouse click (when user releases the right button)
    Generates popup menu.
    **/
    void OnRighMouseButtonUp(wxMouseEvent &event);


    /**
    Executes when user clicks any item of the popup menu generated by #OnRighMouseButtonUp
    **/
    void OnPopupClick(wxCommandEvent &event);

    /**
    Prevents deselection of the currently selected items when user presses the right button of the mouse.
    Without it we would not be able to remove the currently selected items from the right click popup menu.
    **/
    void OnRighMouseButtonDown(wxMouseEvent &event);
};


}}}// namespaces
#endif
