/*
Copyright (C) Ivan Ivanov, 2015-2016

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/



#include <wx/frame.h>
#include <wx/splitter.h>
#include <wx/fileconf.h>
#include "FullTextSearchResults.h"
#include <wx/aui/auibook.h>
#include<wx/menu.h>
namespace Khz{ namespace FTS { namespace Gui{


/**
@brief !!!!!  The main full text search window. !!!!! <br>   <br>                NOTE: the search input, the search results and so on are drawn
by the #Khz::FTS::Gui::FullTextSearchResults class.
This class draws tabs, each of the tab holds a #Khz::FTS::Gui::FullTextSearchResults object inside.
**/
class FullTextSearchWindow : public wxFrame
{
public:

    /**
    Constructor
    @param parent standard wxwidgets parent
    **/
    FullTextSearchWindow(wxWindow *parent);



    /**
    Destructor
    **/
    ~FullTextSearchWindow()
        {
        delete config;
        if(!is_menubar_shown) delete menuBar;// if it is shown than wxwidgets will take care of it
        }


    /**
    Reads config file again and redraws the search results.
    In additions, shows/hides toolbar.
    **/
    void ReDraw();


    /**
    Reads config file again and redraws the wxListBox with available stemmers.
    **/
    void ReDrawStemmers()
        {
        for(unsigned int i=0;i<(tabs->GetPageCount()-1);i++)
            {
            ((FullTextSearchResults *)(  tabs->GetPage(i)  ))  ->ReDrawStemmers();
            }
        //win->ReDrawStemmers();
        };


    /**
    Used to perform a search from the "outside" world. Let's explain a little.
    The full text search window is a standalone window. But the main window of the program have
    the full text search input too. So, when user wants to perform a search from
    the main window, we have to transform data to the full text search window.
    This is what this function is for.

    In general this function does the following:
    - First, it inserts \a search_string into the search input (of the full text search window);
    - Next, it selects appropriate stemmer from the stemmers list (the \a language parameter)
    - Next, it inserts all of the topic listed in \a topics_array to the corresponding
    input of the full text search window ("restrict search by topics" input).
    - Finally, it performs the full text search.
    **/
    void OnSearchWanted(wxString search_string, wxString language, std::vector<wxString> &topics_array);


    /**
    Reconnects to the xapian database. Reads new path to the database from the main config file
    **/
    void ReconnectToDB()
        {
        delete config;
        config =new wxFileConfig( "", "", ini_filename);
        for(unsigned int i=0;i<(tabs->GetPageCount()-1);i++)
            {
            ((FullTextSearchResults *)(  tabs->GetPage(i)  ))  ->
                            ReconnectToDB(config->Read("FULL_TEXT_SEARCH/database_path", "").ToStdString());
            }
        //win->ReconnectToDB(config->Read("FULL_TEXT_SEARCH/database_path", "").ToStdString());
        }
private:

    /**
    Executes when user selects "Help->how to search" in the menubar. Shows window explaining how to search.
    **/
    void OnHowToSearchPressed(wxCommandEvent &event);

    /**
    Remembers the new size of the window (puts data to the config file).
    On the next launch we restore the size of the window.
    **/
    void OnResize(wxSizeEvent &event);


    /**
    Remembers if window is maximized (puts data to the config file).
    We would restore the state on the next launch.
    **/
    void OnMaximize(wxMaximizeEvent& event);


    /**
    Config associated with the main config file.
    **/
    wxFileConfig *config;


    /**
    Full path to the main config file
    **/
    wxString ini_filename;


    /**
    The creation/destruction of the full text search window is managed by the
    Khz::MainWin::MyMainWindow class (we do it to ensure that there only one
                                      instance of the full text search window exists).
    So, this function executes when user tries to close the window.
    It prevents the window closing; instead it sends a wxEVT_BUTTON event that tells that
    user wants to close the full text search window (the event have the specified id, the id
                                                     is Khz::my_wx_id__CLOSE_FULL_TEXT_SEARCH_WINDOW).
    The event is going to be caught by the parent class (Khz::MainWin::MyMainWindow).
    Khz::MainWin::MyMainWindow will close the window (See Khz::MainWin::MyMainWindow::OnFullTextSearchWindowQuit)
    @note This function would not send the event if #FullTextSearchResults::IsIdle() returns false for any of the tab.
    The reason is the following: if #FullTextSearchResults::IsIdle() returns false, it means that a  parallel thread
    performing a search is
    running right now. If we close the window, we would get a segmentation fault when the thread will try to signal #FullTextSearchResults
    to draw the search results (because the corresponding #FullTextSearchResults object would not exist anymore).
    **/
    void OnQuit(wxCloseEvent& event);

    /*
    As the description of this class says, this class is just a top level container
    for the #Khz::FTS::Gui::FullTextSearchResults class object.
    So, we should have an instance of the #Khz::FTS::Gui::FullTextSearchResults class.
    */
    //FullTextSearchResults *win;


    /**
    Executes when user select "File->Close" in the menubar.
    Just calls #OnQuit()
    **/
    void OnCloseMenuSelected(wxCommandEvent &event);


    /**
    Overloaded version of #OnQuit(wxCloseEvent& event). See OnQuit(wxCloseEvent& event).
    **/
    void OnQuit();


    /**
    The tab with the "plus" symbol. When user clicks this tab, a new full text search tab appear.
    This works like this: when user clicks any tab, wxwidgets emit a wxEVT_AUINOTEBOOK_PAGE_CHANGING event.
    We catch this event, as a result #OnTabChenging gets executed. #OnTabChenging checks if user clicked
    #new_tab_tab (this tab is the last one). If this is the case, than #OnTabChenging inserts a new full
    text search tab and BLOCKS THE EVENT. As a result #new_tab_tab cannot be activated.
    **/
    wxPanel *new_tab_tab;

    /**
    the notebook with tabs
    **/
    wxAuiNotebook *tabs;

    /**
    full path to the full text search index
    **/
    std::string path_to_xapian_db;

    /**
    see #new_tab_tab
    **/
    void OnTabChanging(wxAuiNotebookEvent& event);

    /**
    When user presses the "search" button ( object inside of the #FullTextSearchResults class), we manually send
    a wxEVT_BUTTON event with the id my_wx_id__EVENT_TO_CHANGE_TEXT_OF_THE_TAB. This event stores the search string
    that user have entered into the search input. Here (in this class) we catch this event, extract the search string
    and set the label of the corresponding tab.
    **/
    void SetTabText(wxCommandEvent &event);


    /**
    Executes when user closes a tab. If it's the last tab that gets closed, than this function
    closes the full text search window.
    The more interesting thing happens when user closes the next to the last tab.
    In this case wxwidgets automatically activates the last tab. But in our case,
    the last tab is the #new_tab_tab tab. If this tabs gets activated, than
    a new tab will be created. So this function prevents such activation.

     @note This function would block the tab closing if #FullTextSearchResults::IsIdle() returns false for the corresponding tab.
    The reason is the following: if #FullTextSearchResults::IsIdle() returns false, than it means that a  parallel thread
    performing a search is
    running right now. If we close the tab, we would get a segmentation fault when the thread will try to signal #FullTextSearchResults
    to draw the search results (because the corresponding #FullTextSearchResults object would not exist anymore).
    **/
    void OnTabClosing(wxAuiNotebookEvent& event);


    /**
    Images list associated with #tabs. Currently contain only one icon with the "plus"
    symbol for the last tab.
    **/
    wxImageList* m_pImageList;

    /**
    Menubar of the full text search window
    **/
    wxMenuBar *menuBar;


    /**
    Indicates if #menuBar is shown. Used by destructor. The reason for this var is the following:
    user can hide/show menubar. If menubar is attached to the window(shown) than destructor
    will automatically delete it, we do not need to call delete manually. If menubar is not shown
    (is not attached to the window), than we have to call delete manually.
    **/
    bool is_menubar_shown;

    /**
    Executes after user closes a tab.
    Updates tab indexes #FullTextSearchResults::SetTabIndex() for each of the tabs.
    **/
    void OnTabClosed(wxAuiNotebookEvent& event);

    /**
    If \em str is shorter than \em length characters, than adds spaces to the end of the string
    to make it's characters number be equal to \em length. If \em str is longer than \em length characters,
    than crops the string and adds three dots to it's end. As a result, the length of the produced string (
    together with the dots) will be equal to \em length.
    **/
    wxString ellipsize_str(wxString &str, unsigned int length);


};


}}}// namespaces
