/*
Copyright (C) Ivan Ivanov, 2015

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/



#ifndef KOLKHOZNIK_LANGUAGESELECTORWXCHOICE_HEADER_INCLUDED
#define KOLKHOZNIK_LANGUAGESELECTORWXCHOICE_HEADER_INCLUDED

#include <wx/choice.h>
#include <wx/fileconf.h>
#include <vector>
namespace Khz{ namespace FTS { namespace Gui{


/**
@brief wxChoice allowing user to select xapian stemmer.

List of supported stemmers can be generated from the config file
or by Xapian::Stem::get_available_languages().
If config file contains VALID stemmers list,
than stemmers list is gonna to be generated based on the the config file.
if stemmers list in the config file is NOT VALID, than Xapian::Stem::get_available_languages() will be used.

P.S. we use config file to specify the stemmers order.
**/
class LanguageSelectorWxChoice : public wxChoice
    {
    public:
        /**
        constructor
        @param parent standard wxwidgets parent
        **/
        LanguageSelectorWxChoice(wxWindow *parent);


        /**
        Destructor
        **/
        ~LanguageSelectorWxChoice(){delete config;};


        /**
        Clears stemmers list and redraws it. We want call this
        function if we made some modifications in the config file.
        **/
        void ReDraw();
    private:



    /**
    config associated with the main config file.
    **/
    wxFileConfig *config;



    /**
    Checks if the @a stemmers parameter contains the same stemmers that are supported by xapian.
    We use this function inside of #GetStemmers().
    @param stemmers list of stemmers to check.
    **/
    bool AreStemmersGood(std::vector<std::string> stemmers);




    /**
    Gets available stemmers list either from the config file
    or from xapian.
    Basically this function does the following:
    - first, it extracts stemmers list from the config file.
    - second, it creates list of supported by the xapain stemmers. GetStemmersFromXapian() is used for this.
    - third, if checks if extracted from the config file stemmers list
    contain the same stemmers that are supported by the xapian (order does not matter).
    - finally. If extracted from the config file stemmers list
    contain the same stemmers that are supported by the xapian, than this function returns
    the extracted from the config file stemmers list. Otherwise it returns stemmers
    returned by #GetStemmersFromXapian ()
    **/
    std::vector<std::string> GetStemmers();



    /**
    Gets default stemmer from the config file.
    **/
    std::string GetDefaultStemmerFromConfig();


    /**
    Gets stemmers list from xapian. See also #GetStemmers();
    **/
    std::vector<std::string> GetStemmersFromXapian();


    /**
    full path to the main config file.
    **/
    wxString ini_filename;

    };


}}}

#endif
