/*
Copyright (C) Ivan Ivanov, 2014-2015

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/



#include <wx/progdlg.h>
#include <wx/filename.h>
#include <wx/textfile.h>
#include <boost/regex.hpp>
#include <wx/checkbox.h>
#include <wx/frame.h>



namespace Khz{namespace GenDB{




/**
@brief The database creation interface

Popup window, user can generate the database here
**/
class SqliteDBGenerator  : public wxFrame
{
public:
    /**
    Constructor, creates the database generation window
    **/
    SqliteDBGenerator(wxWindow* parent, const wxString& title);
private:
    /**
    Removes classificator from the kolkhoz library folder name. For example, transforms string "CsLn_Lecture notes" to "Lecture notes".
    **/
    wxString clear_topic(wxString topic) const;

    /**
    Input for the  path to the library directory
    **/
    wxTextCtrl *library_dir_input;


    /**
    Input for the database path
    **/
    wxTextCtrl *where_to_store_database_input;


    /**
    Just a text, something like "Choose kolkhoz library directory"
    **/
    wxStaticText *choose_path_to_library_text;

    /**
    Just a text, something like "Choose where to store the database".
    **/
    wxStaticText *choose_path_to_database_text;


    /**
    Progress bar indicating the database creation status
    **/
    wxGenericProgressDialog *progress_bar;


    /**
    Just a text, something like "Choose full_md5_len.txt"
    **/
    wxStaticText *choose_path_to_full_md5_len_text;


    /**
    Input for the path to the full_md5_len.txt
    **/
    wxTextCtrl *full_md5_len_input;

    /**
    Just a text, something like "Choose where to store the log file"
    **/
    wxStaticText *choose_path_to_log_text;

    /**
    Input for the path to the log file
    **/
    wxTextCtrl *where_to_store_log_input;


    /**
    "Create database" button. If user presses this button, the #OnCreateDatabaseButtonPressed gets executed.
    **/
    wxButton *create_database_button;

    /**
    "Cancel button". If user presses this button, the #OnCancel function gets executed.
    **/
    wxButton *cancel_creating_database_button;


    /**
    Main window sizer
    **/
    wxBoxSizer *main_sizer;

    //void OnSize(wxSizeEvent &event);

    /**
    Executes when user presses "Choose" button to select the library path.
    Shows the directory selection dialog.
    **/
    void DoKolhozPathChoose(wxCommandEvent& event);

    /**
    Executes when user presses "Choose" button to select the database path.
    Shows the directory selection dialog.
    **/
    void DoKolhozDatabasePathChoose(wxCommandEvent &event);


    /**
    Executes when user presses "Choose" button to select the log file path.
    Shows the file selection dialog.
    **/
    void DoKolhozLogPathChoose(wxCommandEvent &event);



    /**
    Executes when user presses "Choose" button to select the full_md5_len.txt path.
    Shows the file selection dialog.
    **/
    void DoKolhoz_full_md5_len_Choose(wxCommandEvent &event);

    /**
    Closes the window
    **/
    void OnCancel(wxCommandEvent &event);



    /**
    Launches the database generation
    **/
    void OnCreateDatabaseButtonPressed(wxCommandEvent &event);


    /**
    @note We do not use this function currently, see #fill_database_v2 instead

    Generates the database based on the dir with books. Does not need the full_md5_len.txt file.
    Recursively goes through the kolkhoz library directory, finds files,
    fills database based on the filenames.
    @param dir_with_books full path to the kolkhoz library directory
    @param db database
    @param parsed_size used internally during the recursive calls (to display the progressbar). Just provide a pointer to zero here.
    @param total_size_of_library used internally during the recursive calls(to display the progressbar). Just provide a pointer to zero here.
    @param abs_path used internally during the recursive calls. Use the default parameter.
    @param last_id used internally during the recursive calls. Use the default parameter.

    **/
    void fill_database(wxString dir_with_books,sqlite3 *db,long double *parsed_size,  long double *total_size_of_library, wxString abs_path="empty",long int last_id=1);

    /**
    Gets md5 of the file
    **/
    std::string md5(wxFileName 	 current_file);



    /**
    Used by the #fill_database_v2 function. Takes line from the full_md5_len.txt,
    extracts Author, Title, Topic and so on, puts data to the database.
    returns true if everything is ok, false otherwise
    If #check_for_missing_files is true, than checks if the according file exists; checks md5 of the file.
    **/
    bool process_line(std::string line, wxString path_to_library, std::ofstream &log_file, sqlite3 *db);

    /**
    Used by #process_line function. Puts data to the database.
    @param filename filename with extension
    @param topic_id The id of the topic ("topics" table of the database) that book must belong to.
    @param Topic the topic that book must belong to.
    @param RelPath Relative path to the book, for example "/P_Physics". Path separator must be "/".
    @param md5  md5 of the file
    @param filesize the size of the file
    @param db sqlite3 database (connected)
    **/
    void put_file_to_db
        (
        std::string filename,
        long long int topic_id,
        std::string Topic,
        std::string RelPath,
        std::string md5,
        std::string filesize,
        sqlite3 *db);

    /**
    Removes last "/" sumbol if exists, adds leading sumbol "/" if not exists.
    **/
    std::string clear_relative_path(std::string relative_path_std);


    /**
    Inserts topic to the database if it is not there.
    Returns ID of the topic in the database. Used by #fill_database_v2.
    In case of error returns -1;
    **/
    long long int insert_topic_to_db_if_necessary(std::string topic, sqlite3 *db);


    /**
    Fills database based on the full_md5_len.txt.  If #check_for_missing_files is true, than checks for missing files,
    writes log file
    **/
    bool fill_database_v2( wxString path_to_library,  wxString path_to_full_md5_len_txt, wxString path_to_log_file, sqlite3 *db);

    /**
     We check for missing files if #check_for_missing_files is true. If we detect a missing file, we increment the #errors_count.
     When the database generation is finished, we show a message to the user. Depending on the #errors_count we choose one or another message.
     **/

    long long  int errors_count;


    /**
    Used by #fill_database_v2.
    Executes when database creation is finished. Checks that the number of the lines in the full_md5_len.txt is equal to the number of the rows in the database.
    Returns true if everything is ok, false otherwise
    **/
    bool check_counts(long long int counter,sqlite3 *db);

    /**
     If true, than #fill_database_v2 checks for missing files during the database generation
    **/
    bool check_for_missing_files;


    /**
    "Check my library for missing files" checkbox. If user clicks this checkbox, #OnCheckFilesCheckboxClicked gets executed.
    **/
    wxCheckBox *check_files_checkbox;

    /**
    "Choose" path to the library button. If user presses this button, the #DoKolhozPathChoose function gets executed.
    **/
    wxButton *choose_path_to_library_button;

    /**
    Executes when user checks/unchecks the "check my library for missing files" checkbox.
    Sets #check_for_missing_files to true/false, enables/disables two last inputs:library path  input, path to the log file input
    **/
    void OnCheckFilesCheckboxClicked(wxCommandEvent& event);
    /**
    "Choose" path to the log file button. If user presses this button, #DoKolhozLogPathChoose gets executed.
    **/
    wxButton *choose_path_to_log_button;


    /**
    add missing topics to the "topics" table of the main kolkhoznik database.
    For example, is "topics" table have topic "/Something/Something_else"
    but does not have topic "/Something" than this function will add
    topic "/Something to the "topics" table of the database.
    @param db sqlite3 database
    @param min_id You have to use the default value, this parameter is used internally during recursive calls.
    **/
    void add_missing_topics(sqlite3 *db, unsigned int min_id=1);


    /**
    "Check files existence" checkbox. Do nothing, just for aesthetic purpose.
    **/
    wxCheckBox *check_for_missing_files_checkbox;

    /**
    "Verify files_md5" checkbox. If checked, than we would verify md5 sums of files. When
    user clicks this checkbox, #On_check_md5_checkbox_clicked gets executed.
    **/
    wxCheckBox *check_md5_checkbox;

    /**
    If we should verify md5 sums of files. We set this var in constructor and by #On_check_md5_checkbox_clicked
    **/
    bool virify_files_md5;


    /**
    Just prevents #check_for_missing_files_checkbox from being unchecked.
    **/
    void On_check_for_missing_files_checkbox_clicked(wxCommandEvent &event)
        {
        check_for_missing_files_checkbox->SetValue(true);
        }


    /**
    Executes when user clicks #check_md5_checkbox. Sets #virify_files_md5.
    **/
    void On_check_md5_checkbox_clicked(wxCommandEvent &event)
        {
        if (check_md5_checkbox->GetValue())
            {
            virify_files_md5=true;
            }
        else
            {
            virify_files_md5=false;
            }
        }
};



}}// end of namespaces
