/*
Copyright (C) Ivan Ivanov, 2014-2015

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/




#ifndef KOLKHOZNIK_GET_BOOKS_FROM_DB_BY_QUERY_HEADER_INCLUDED
#define KOLKHOZNIK_GET_BOOKS_FROM_DB_BY_QUERY_HEADER_INCLUDED



#include "Mdb_fuctions.h"
#include <wx/string.h> // only for get_book_from_db_by_id(long long int id, wxString database_path, std::string dbtable);
#include "clear_database_path.h" // only for get_book_from_db_by_id(long long int id, wxString database_path, std::string dbtable);


namespace Khz{namespace Libs{ namespace DB{


/**
@brief gets books from db in multiply queries, N books per query.


Let us consider an example. Assume that there is a database that consists of 100000 rows.
Also assume that you want to get all of the books from the database.
As the database is big, it would be a bad idea to get all of the books by a single query.
By using this class one can get all of the books in several queries. To get all of the books by TEN books per query one can do this:
@code
    std::string table="main";
    std::string where_statement="topic=?"; // "SELECT * FROM main WHERE " + "topic=?"
    //we have one placeholder here. The query is "SELECT * FROM main WHERE topic=?
    std::vector<std::string> placeholder_parameters;
    placeholder_parameters.push_back("/Biology");  //the query is going to be "SELECT * FROM main WHERE topic='/Biology'"

    get_books_from_db_by_query database_asker(db, "main", where_statement, 10, placeholder_parameters); //10 books per query
    while(database_asker.Step())
        {
        for(long i=0;i<database_asker.current_result.size();i++)
            {
            std::string Author=database_asker.current_result[i]["Author"];
            ... //do whatever you want
            }
        }
@endcode
**/
class get_books_from_db_by_query
{
public:
    /**
    constructor, see example above
    **/
    get_books_from_db_by_query(sqlite3 *_db, std::string _dbtable,
                               std::string _where_statement, long _num_of_books_per_step, std::vector<std::string> _placeholder_parameters);
    ~get_books_from_db_by_query() {this->db=NULL;};

    /**
    The result of the query for the current step
    **/
    mapDB current_result;

    /**
    Gets next #num_of_books_per_step from the database
    Returns false if there are no more books, true otherwise.
    **/
    bool Step();

    /** Returns #total_steps **/
    long long GetTotalSteps(){return total_steps;};

    /** Returns #step **/
    long long GetCurrentStep(){return step;};

    /** Returns #total_lines **/
    long long GetTotalLines(){return total_lines;};

    /**
    returns total number of books in the library
    **/
    static long long int get_num_of_books_in_library(sqlite3 *db, std::string dbtable);

    /** Gets one book from the database by id **/
    static Khz::Libs::DB::mapDB get_book_from_db_by_id(sqlite3 *db, std::string dbtable, long long int id);

    /** Gets one book from the database by id **/
    static Khz::Libs::DB::mapDB get_book_from_db_by_id(long long int id, wxString database_path, std::string dbtable);

private:
    /**
    Total number of books that are consistent with where_statement (the constructor parameter)
    **/
    long long total_lines;

    /**
    How many steps is needed to get all of the books from the database
    **/
    long long total_steps;

    /**
    Current step
    **/
    long long step;

    /**
    How many books we are getting from the database by a single query
    **/
    long num_of_books_per_step;


    /**
    The where statement that is used to get books from the main kolkhoznik database.
    The sql query to the database is going to be
    @code
    std::string query ="SELECT * FROM " + dbtable + " WHERE " + where_statement +";";
    @endcode

     @note The WHERE statement could have placeholders. There is not need to use placeholders for integer values, so you
        should bind a string for each of the placeholders (use sqlite3_bind_text for it). The parameters for the placeholders
        are stored in #parameters_for_WHERE_placeholders.
    **/
    std::string where_statement;


    /**
    The placeholder parameters for #where_statement. You should use them like this:
    @code
    for(unsigned int i=0; i< parameters_for_WHERE_placeholders.size(); i++)
        {
        sqlite3_bind_text(stmt,  i+1, parameters_for_WHERE_placeholders[i].c_str(), -1, SQLITE_TRANSIENT);
        }
    @endcode
    **/
    std::vector<std::string> parameters_for_WHERE_placeholders;

    /**
    The table of the database that we going to query.
    **/
    std::string dbtable;

    /**
    pointer to the database.
    **/
    sqlite3 *db;
};

}}} // end of namespaces
#endif
