/*
Copyright (C) Ivan Ivanov, 2014-2015

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/



#include<vector>
#include<string>
#include<sqlite3.h>
#include<iostream>
#include "Mdb_fuctions.h"
//get books with this topic

namespace Khz{namespace Libs{ namespace DB{

/**
@brief Selects books associated with the particular topic from the database. Allows access the result of the database query.

This class allows to select books that are inside of the particular topic from the database.
The result of the database query is stored in the #result variable.
**/
class show_books_in_topic {
public:

    //gets only one line from db

    /**
    Queries database, gets the num-th row. Assume for example that some topic contain, for example, 100 rows. This
    function gets only the num-th row(assuming num<100).
    @param db sqlite3 database
    @param dbtable database table
    @param Topic_id the id of the topic in the database (table "topics")
    @param num number of the row in the database (counting starts from zero). If num=-1 than only #total_lines is going to be filled
    (use it to get the total books number within the particular topic)
    @param order Sorting order, for example "ORDER by AuthorLowerCase" (order goes into the sql query directly).

    @note Let us note that sqlite does not support a case-insensitive search for non-latin characters.
    To enable such search, we have columns like "TitleLowerCase". When searching, we transform a
    search string to the lowercase one, than search in the *LowerCase column.

    The result of the database query can be retrieved like this:
    @code
    show_books_in_topic books=show_books_in_topic(database, table, some_id, some_num, some_order);
    std::string author_of_book=books.result[0]["Author"];
    //we should give zero as index to the result (result contains only one row).
    @endcode
    **/
    show_books_in_topic(sqlite3 *db, std::string dbtable, long int Topic_id,long long num, std::string order);



    /**
    Queries the database, gets rows from num_first to num_last.
    @param db sqlite3 database
    @param dbtable database table
    @param Topic_id the id of the topic in the database (table "topics")
    @param num_first first row to get. Be sure that num_first is less than #total_lines
    @param num_last last row to get. Be sure that num_last is less than #total_lines
    @param order Sorting order, for example "ORDER by AuthorLowerCase" (order goes into the sql query directly).

    @note Let us note that sqlite does not support a case-insensitive search for non-latin characters.
    To enable such search, we have columns like "TitleLowerCase". When searching, we transform a
    search string to the lowercase one, than search in the *LowerCase column.

    The result of the database query can be retrieved like this:
    @code
    show_books_in_topic books=show_books_in_topic(database, table, some_id, 20, 30, some_order);  //gets rows from 20 to 30
    std::string author_of_book=books.result[0]["Author"];  // Author of 20-th row
    //P.S. The result of the query does not know anything about the starting and last rows.
    //Count there always starts from zero, so books.result[0] is actually the 20-th row.
    @endcode
    **/
    show_books_in_topic(sqlite3 *db, std::string dbtable, long int Topic_id, long long num_first, long long num_last, std::string order);

    /**
    The result of the database query. See Mdb::mapDB
    **/
    Khz::Libs::DB::mapDB result; // using:: result[1]["Title"] to get value; result[int n]["someitle"].  sometitle is a column name in database.
    /**
    Total number of books within the particular topic in the database.
    You can get it like this:
    @code
    long int total_num_of_books_in_topic=show_books_in_topic(db, dbtable, 10,-1, "").total_lines;
    //gets the number of books within the topic that have ID=10 ("topics" table in the database). Forth parameter is -1,
    //meaning than we need only the books count.
    @endcode
    **/
    long long  total_lines;

    /**
    The "WHERE" statement that is used to get books from the "main" table of the database.
    The complete query to the database is:
    @code
    "SELECT * FROM main WHERE " + where_statement + "ORDER BY..."
    @endcode
    @note #where_statement does not have any placeholders, you can use it safely without any bindings.
    **/
    std::string where_statement;
};





/**
@brief Searchs in the database

Searches the database. See the class's constructors description for more info.
**/
class search_books {
public:
    /**
    Searchs the database, returns the num-th row.
    Assume for example that user searches for the word "electron". The result of the search can contain, for example, 100 rows. This
    function gets only the num-th row(assuming num<100).
    @param db database
    @param dbtable table in the database
    @param search_string search string as it was entered into the search input (several vords divided by spaces)
    @param num the number of the row in the database (counting starts from zero). If num=-1 than only #total_lines is going to be filled
    (use it to get the total number of books that contain the search string.)
    @param search_type Search type can be "or", "and". User can enter several words into the search input.
    If search type is "or", than we select rows that have one of these words from the database.
    If search type is "and", than we select only rows that have all of the words.
     @param fields_to_search fields to search (columns in the database) as it should be inserted into sql query.
     For example, valid values are "TitleLowerCase", "AuthorLowerCase", "AuthorLowerCase || TitleLowerCase".
     So, if searching should be done in several fields than these fields should be splitted by the "||" symbol.
    @note Let us note that sqlite does not support a case-insensitive search for non-latin characters.
    To enable such search, we have columns like "TitleLowerCase". When searching, we transform a search string to
    the lowercase one, than search in the *LowerCase column.
    @param order Sorting order, for example "ORDER by AuthorLowerCase" (order goes into the sql query directly).

    The result of the database query can be retrieved like this:
    @code
    search_books books=search_books(database, table, "electron positron", some_num, "and", "Title", "");
    //searches for books that have words (both of the words) "electron" and "positron" in the title. Returns some_num-th row.
    std::string author_of_book=books.result[0]["Author"];
    //we should give zero as index to the result (result contains only one row).
    @endcode
    **/
    search_books(sqlite3 *db, std::string dbtable, std::string search_string,long long num, std::string search_type,
                          std::string fields_to_search, std::string order);





        /**
    Searchs the database, gets rows from num_first to num_last.
    @param db database
    @param dbtable table in the database
    @param search_string search string as it was entered into the search input (several vords divided by spaces)
    @param num_first first row to get. Be sure that num_first is less than #total_lines
    @param num_last last row to get. Be sure that num_last is less than #total_lines
        @param search_type Search type can be "or", "and". User can enter several words into the search input.
    If search type is "or", than we select rows that have one of these words from the database.
    If search type is "and", than we select only rows that have all of the words.
     @param fields_to_search fields to search (columns in the database) as it should be inserted into sql query.
     For example, valid values are "TitleLowerCase", "AuthorLowerCase", "AuthorLowerCase || TitleLowerCase".
     So, if searching should be done in several fields than these fields should be splitted by the "||" symbol.
    @note Let us note that sqlite does not support a case-insensitive search for non-latin characters.
    To enable such search, we have columns like "TitleLowerCase". When searching, we transform a search string to
    the lowercase one, than search in the *LowerCase column.
    @param order Sorting order, for example "ORDER by AuthorLowerCase" (order goes into the sql query directly).

    The result of the database query can be retrieved like this:
    @code
    search_books books=search_books(database, table, "electron positron", 20, 30, "and", "Title", "");
    //searches for books that have words (both of the words) "electron" and "positron" in the title. Returns 20-30 rows.
    std::string author_of_book=books.result[0]["Author"];  //author of the 20-th row
    //P.S. The result of the query does not know anything about the starting and last rows.
    //Count there always starts from zero, so books.result[0] is actually the 20-th row.
    @endcode
    **/
    search_books(sqlite3 *db, std::string dbtable, std::string search_string,long long num_first,long long num_last, std::string search_type,
                          std::string fields_to_search, std::string order="ORDER by AuthorLowerCase, TitleLowerCase");


    /**
    The result of the database query. See Mdb::mapDB
    **/
    Khz::Libs::DB::mapDB result; // using:: result[1]["Title"] to get value; result[int n]["someitle"].  sometitle is a column name in database.
        /**
    The total number of books that contain the search string.
    You can get it like this:
    @code
    long int total_num_of_books=search_books(db, dbtable, "electron positron" , -1, "and","Title","");.total_lines;
    //gets the number of books that have words (both of the words) "electron" and "positron" in the title. The forth parameter is -1,
    //meaning than we need only the books count.
    @endcode
    **/
    long long total_lines;


    /**
    The "WHERE" statement that is used to get books from the "main" table of the database.
    The complete query to the database is:
    @code
    "SELECT * FROM main WHERE " + where_statement + "ORDER BY..."
    @endcode

    @note The WHERE statement could have placeholders. There is not need to use placeholders for integer values, so you
    should bind a string for each of the placeholders (use sqlite3_bind_text for it). The parameters for the placeholders
    are stored in #parameters_for_WHERE_placeholders.
    **/
    std::string where_statement;

    /**
    The placeholder parameters for #where_statement. You should use them like this:
    @code
     for(unsigned int i=0; i< parameters_for_WHERE_placeholders.size(); i++)
            {
            sqlite3_bind_text(stmt,  i+1, parameters_for_WHERE_placeholders[i].c_str(), -1, SQLITE_TRANSIENT);
            }
    @endcode
    **/
    std::vector<std::string> parameters_for_WHERE_placeholders;
};


}}}// namespaces
