/*
Copyright (C) Ivan Ivanov, 2014-2015

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/



#include <wx/srchctrl.h>
#include <wx/stattext.h>
#include <wx/checkbox.h>
#include <wx/radiobut.h>
#include "LanguageSelectorWxChoice.h"
#include <wx/hyperlink.h>
#include "FullTextSearchTopicsList.h"
#include <wx/button.h>
#include <wx/scrolwin.h>



namespace Khz{namespace MainWin{



/**
@brief The search tab of the application. Includes Search input, Search button and so on.

The search tab of the application. Icludes Search input, Search button and so on.
- When user hits the "search" button (not fulltext search, normal search), this
class sends an wxEVT_BUTTON event with the special id, the id is my_wx_id__SEARCH_BOOKS_BUTTON.
The event with this id gets caught by the #Khz::MainWin::MyMainWindow, as a result
#Khz::MainWin::MyMainWindow::OnSearchWanted gets executed. #Khz::MainWin::MyMainWindow::OnSearchWanted
Reads the search string from the search input and performs the searching.

P.S. When user hits ENTER to search book  (not fulltext search, normal search), this
class sends an wxEVT_TEXT_ENTER event with the special id, the id is my_wx_id__SEARCH_BOOKS_INPUT.
The event with this id also gets caught by the #Khz::MainWin::MyMainWindow, as a result
#Khz::MainWin::MyMainWindow::OnSearchWanted gets executed. #Khz::MainWin::MyMainWindow::OnSearchWanted
Reads the search string from the search input and performs the searching as before.
**/
class LeftSearchPanel : public wxScrolledWindow
{
public:
    /**
    The Search input
    **/
    wxSearchCtrl *Search_input;

    /**
    The full texts search input
    **/
    wxSearchCtrl *full_text_search_input;

    /**
    Gets string from the search input and returns it
    **/
    wxString GetSearchText();

    /**
    Gets string from the full text search input and returns it
    **/
    wxString GetFullTextSearchText(){return full_text_search_input->GetValue();};



    /**
    Allows to select language for the full text search stemmer
    **/
    Khz::FTS::Gui::LanguageSelectorWxChoice *full_text_search_language_selector;

    /**
    Reads config again and updates the order of the stemmers of the #full_text_search_language_selector
    accordingly. Just calls LanguageSelectorWxChoice::ReDrawStemmers
    **/
    void ReDrawStemmers(){full_text_search_language_selector->ReDraw();};

    /**
    Gets selected value from #full_text_search_language_selector
    **/
    wxString GetLanguageForFullTextSearch(){
            if(full_text_search_use_stemmer_checkbox->GetValue())
                {
                return full_text_search_language_selector->GetString(  full_text_search_language_selector->GetSelection()  );
                }
            else
                {
                return "";
                }
            };

    /**
    Gets array of topics where user wants to perform the full text search.
    Extracts topics from #full_text_search_topics_list.

    @param array Give an empty vector here. The function will fill it.
    **/
    void GetArrayOfTopicsForFullTextSearch(std::vector<wxString> &array)
        {
        if(!full_text_search_restrict_topics_checkbox->GetValue()) return;
        full_text_search_topics_list->GetArrayOfTopics(array);
        }

    /**
    Constructs the search tab
    @param parent standard wxWidgets parent
    **/
    LeftSearchPanel(wxNotebook *parent);

    /**
    Returns true if the Author checkbox is checked, false otherwise
    **/
    bool IsAuthorChecked();

    /**
    Returns true if the Title checkbox is checked, false otherwise
    **/
    bool IsTitleChecked();

     /**
    Returns the search type based on radiobuttons positions.
    @return wxString.
    @note Search type can be "or", "and". User can enter several words into the search input.
    If search type is "or", than we select rows that have one of these words from the database.
    If search type is "and", than we select only rows that have all of the words.
    **/
    wxString GetSearchType();

    /**
    Disables the "Search" button. We should call this function before actually searching.
    **/
    void on_search_begin();

    /**
    Disables the "Search" button of the full text search section.
    We should call this function before actually searching.
    **/
    void on_full_text_search_begin(){full_text_search_button->Disable();};


    /**
    Enables the "Search" button of the full text search section (disabled previously by #on_full_text_search_begin() )
    **/
    void on_full_text_search_end(){full_text_search_button->Enable();};

    /**
    Enables the "Search" button (disabled previously by #on_search_begin() )
    **/
    void on_search_end();

private:
    /**
    The Search button.
    **/
    wxButton *Search_button;



    /**
    The full text search button
    **/
    wxButton *full_text_search_button;

    /**
    "use stemmer " checkbox
    **/
    wxCheckBox *full_text_search_use_stemmer_checkbox;

    /**
    Executes when user checks/unchecks #full_text_search_use_stemmer_checkbox.
    Enables/Disables #full_text_search_language_selector
    **/
    void OnUseStemmerCheckboxClicked(wxCommandEvent &event);

    /**
    Executes when user checks/unchecks #full_text_search_restrict_topics_checkbox.
    Enables/Disables #full_text_search_topics_list;=
    **/
    void OnRestrictTopicsCheckboxClicked(wxCommandEvent &event);

    /**
    shows message box explaining what stemmer is
    **/
    void OnWhatIsStemmerLinkClicked(wxHyperlinkEvent& event);

    /**
    The "Title" checkbox. If checked, than we search by title.
    **/
    wxCheckBox *Title_checkbox;

    /**
    The "Author" checkbox. If checked, than we search by author.
    **/
    wxCheckBox *Author_checkbox;


    /**
    radio button, "AND" (search all of the words)
    **/
    wxRadioButton *search_type;

    /**
    radio button, "OR" (search any word)
    **/
    wxRadioButton *search_type2;

    /**
    The "restrict search by topics" checkbox. If checked, than
    full text searching is restricted by topics listed in #full_text_search_topics_list
    **/
    wxCheckBox *full_text_search_restrict_topics_checkbox;

    /**
    List of topics for the full text searching (together with buttons "Add" ,"Remove", "Clear").
    **/
    Khz::FTS::Gui::FullTextSearchTopicsList *full_text_search_topics_list;

};


}} // end of namespace
