/*
Copyright (C) Ivan Ivanov, 2014-2015

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/



#include <wx/listctrl.h>
#include <vector>
#include<sqlite3.h>
#include <wx/fileconf.h>
#include <wx/menu.h>
#include "books_cache.h"



namespace Khz{namespace MainWin{




/**
@brief Creates the books list (right panel), can show books in particular topic or search results.

Creates the books list (right panel), can show search results or books associated with the particular topic. A user can open a book by double-clicking it.
The book is going to be opened by the external application associated with the file extension.

===================================================================
Let us describe briefly how the books list is working. It is inherited from the wxListCtrl control (wxWidgets library). We c
create a virtual list (wxLC_VIRTUAL, see constructor). A
virtual control can load only books that are to be diplayed on the monitor and unload them when they go out of the screen.
Such configuration allows to work with a very big dataset without(almost) performance issues.
When creating the control we should add colums and set their headers text.
Next, we should set the number of the rows, this is done by the  SetItemCount(int num_of_rows) function (this function is part of the wxListCtrl class).
When the number of the rows is set, wxWidgets will ajust  the size of the scrollbar.
To display text in rows wxWidgets calls the OnGetItemText(long item, long column) function. It is a pure virtual function
of the wxListCtrl class(MyBooksList is inhereted from it). For example we can define this function like this:
@code
wxString MyBooksList::OnGetItemText(long item, long column) const
    {
    return "test";
    }
@endcode
In this case all of the cells will contain the word "test".

Let's consider a more practical example. Assume that we want to show all of the book associated with the topic "/Physics".
First, we should query the database and get the id of the topic. For this we could use this query:
@code
SELECT id from topics where Topic='/Physics'
@endcode

So, we got the id(assume for our example that id=1), next step is to select the books number. For this we could run the simple query:
@code
SELECT COUNT(*) from main where ID_of_Topic=1
@endcode
Assume for example that the number is 10. So, we call SetItemCount(10). This will set the scrollbar size.

To display books we steel have to define the OnGetItemText function.
Schematically it could look like this:
@code
wxString MyBooksList::OnGetItemText(long item, long column) const
    {
    //first we query the database: SELECT * FROM dbtable WHERE ID_of_Topic=1 LIMIT item,1;
    //this query will select only one row. Suppose that we have a variable "result" that can return title, autor and so on by
    // this way: result["Title"], result["Autor"] ("Title", "Author" are names of the fields in the database)
    // P.S. the query_database function below is a pure hypothetical function, we do not have such function in the real world.
    result=query_database("SELECT * FROM dbtable WHERE ID_of_Topic=1 LIMIT "+item+",1;");

    //second, we shoud know which of the fields is associated with the current column.
    //Assume that we know it, for example suppose that we got it by a hypothetical function GetlumnName(column):
    wxString column_name=GetColumnName(column);  //for example column_name is "Title" now

    //finally, we return the result
    return result[column_name];  //returns the title of the book
    }
@endcode
This will do it, our list contains books now. But it is going to  work very slowly. The reason for this is obvious: To display
list we have to make columns_number*rows_number queries to the database (for example, if the number of the rows is 20,  and the number of the columns is 5,
than we are going to make 20*5=100 queries to the database).

Fortunately there is a way to make only one query. When wxWidgets are about to display rows,
  it sends a special, so called "wxEVT_LIST_CACHE_HINT" event (BEFORE calling of the OnGetItemText function).
  We can catch this event and get first and last rows that are to be displayed.
  For example, assume that our list consists of 1000 rows and rows 90-120 are currently displayed.
  When user scrolls list, some rows go out of the screen, some rows appear on the screen. Suppose that user scrolled down a bit, and now
  wxWidgets are about to draw items 100-130. Before drawing items it sends a wxEVT_LIST_CACHE_HINT event. We can catch this event
  and do this:
  @code
    long item_min=event.GetCacheFrom();  //return 100
    long item_max=event.GetCacheTo();   //returns 130

    //next, we can select rows 100-130 from the database.
    //assume that we have a class named cache, it can return author, title and so on by
    // cache[110]["Author"]  (returns Author of the 110-th row)
    cache=query_database("SELECT * FROM dbtable WHERE ID_of_Topic=1 LIMIT " + item_min + "," + (1+item_miax-item_min) + ";");
  @endcode
  Finally, we need the OnGetItemText function:
  @code
  wxString MyBooksList::OnGetItemText(long item, long column) const
    {

    wxString column_name=GetColumnName(column);  //for example column_name is "Title" now
    return cache[item][column_name];  //returns title of the book
    }
@endcode
That's it. Our books list is working based on this principle, it have the cache. <br>

Now we are ready for details.<br>
- Let us look into the constructor of the MyBooksList class. That is what is going on there:
    -# First, we initialize #config that provides an interface for the config file management.
    -# Second, we prepare #m_pImageList and set it as an images list for the control.
    -# Next, we set the #do_not_sort_list_if_books_more_than_this_number variable. Let us describe the reason
    why we have this variable. Normally, when user clicks header of the column,  we sort books (if user
                                    clicks "Author" column, than we sort books by author).
    But books list can be very big, in this case sorting will take a long time. As a result the performance issues could appear. So, we
    refuse to sort book if list contains more than #do_not_sort_list_if_books_more_than_this_number books (
                                    of course, user always can change this number by pressing File->Configure->Advanced)
    -# Next, we set the #max_size_of_cache variable. Let us explain a little. In the example above we cached only
    rows that are currently displayed on the screen. But we can cache more items for the better performance.
    For example, we can cache  all of the visible items + 100 items below + 100 items above. As a result, if user scrolls list
    up or down  a bit, we do not need to query the database. We query the database only if user scrolls up or down on more that 100 items.
    So, what is the #max_size_of_cache variable for. If, for example, the #max_size_of_cache is 1000, and  screen can diplay 20 rows,
    than we cache all of the visible rows,  (1000-20)/2 rows below of the visible rows, and   (1000-20)/2 rows above of the
    visible rows.
    -# Next, we connect to the database
    -# Next, we initialize the #full_rows_number variable by zero and call the SetItemCount function.  As a result we have a empty list for
    a while.
    -# Next, we call the #GenItemsToShow() function. It fills the #items_to_show variable (reads the config file for it). Let us explain a little.
    The #items_to_show variable contains list of the columns (Author, Title and so on) that are to be displayed.  #items_to_show[i].first returns the database entry associated with the i-th column,
    #items_to_show[i].second returns the visible caption.
    -# Next, we set  the #users_wanded_order and #sort_order variables. Se description of the #UpdateOrder function to understand the
    difference.
    -# Next, we create columns based on the #items_to_show variable.
    -# By now we have an initialized books list, but this list is empty. Finally we call the #OnBooksInTopicWanted (1) function.
    As a result, the list gets filled with the books that are inside of the topic with the index "1"
    ("1" is the index of the topic in the database (table "topics")).
    Let us note that  "1" is the index of the top level topic.

Let us consider now what happens when we call the #OnBooksInTopicWanted function.
This function clears cache,  queries the database and gets the number of the books in the topic. Next, it sets #mode=TOPICS_MODE, deletes all of the items
and calls the SetItemCount function (also, #OnBooksInTopicWanted calls the #UpdateOrder function to disable on enable sorting, see #UpdateOrder
                                      for details).
As one could guess, the wxWidgets library sends the wxEVT_LIST_CACHE_HINT event when the SetItemCount function gets executed.
We catch this event and call the #PutDataToCache function. This function have the following structure:
@code
void MyBooksList::PutDataToCache(wxListEvent& event)
    {
    long max_cache_size=max_size_of_cache;
    long item_min=event.GetCacheFrom();
    long item_max=event.GetCacheTo();
    if (  //all of the items are already cached
        (cache.cached_min==item_min && cache.cached_max== item_max) ||  ( cache.cached_min< item_min && cache.cached_max>item_max)
        )
        {
        return;
        }
   if(mode==SEARCH_MODE)
        {
        //query dabase, search books according to the #search_string, put result to cache.
        //code is omitted
        }
    else if (mode=TOPIC_MODE)
        {
        //query dabase, select books with #Topic_id, put result to cache.
        //code is omitted
        }
    }
@endcode
So, we have cached all of the books that should be displayed.
Next, the #OnGetItemText function gets executed (automatically, wxWidgets executes this function on its own):
@code
wxString MyBooksList::OnGetItemText(long item, long column) const
    {
    //gets database row associted with current column
    std::string column_name=items_to_show[column].first; //let's assume that this is "Title".
    return cache.get_cached_line(item)[column_name];  //returns title of the book
    }
@endcode
@note
The code above is only the simplified version of the actual #OnGetItemText function code.

So, this is the pricipial scheme.
**/
class MyBooksList : public wxListCtrl
    {
    public:


        /**
        Constructs the books list. Just constructed list shows books in the
        root directory of the library (user
        can update list list by choosing topic on the left panel or by searching).
        **/
        MyBooksList(wxWindow* parent, wxWindowID id);
        ~MyBooksList();


        /**
        Call this function if you have changed the database path in the config file. This function will read the config file
        again, reconnect to the database and rebuild the books list
        **/
        void Reconfigure();



        /**
        you should call this if you have changed icons or visible columns(Title, Author) of the right panel. This function will read the config file
        again and refresh the books list accordingly.
        **/
        void ReDrawList();




        /**
        Reads the config and updates the current config values.
        Call this function if you have changed application associated with some file extension in the Configuration dialog.
        @note When user doble clicks book, we should open it. So we look into config to find out if some application is associated
        with the extension of the file. If it is, we open file with this application. Without calling the JustReadConfigAgain function
        kolkhoznik will not know that you have changed the application in the config file.
        **/
        void JustReadConfigAgain();


         /**
        Searchs for books with the provided search string. Makes the #MyBooksList to display the search results.
        @param _search_string search string as it was entered into the search input (several vords divided by spaces)
        @param _search_type Search type can be "or", "and". User can enter several words into the search input.
        If search type is "or", than we select rows that have one of these words from the database.
        If search type is "and", than we select only rows that have all of the words.
        @param _fields_to_search fields to search (columns in the database)
        as it should be inserted into sql query. For example, valid values are
        "TitleLowerCase", "AuthorLowerCase", "AuthorLowerCase || TitleLowerCase". So, if searching should
        be done in several fields than these fields should be splitted by  the "||" symbol.
        @note Let us note that sqlite does not support a case-insensitive search
        for non-latin characters. To enable such search, we have columns like "TitleLowerCase".
        When searching, we transform a search string to the lowercase one, than search in the *LowerCase column.
        **/
        void OnSearchWanted(
                            std::string _search_string,
                             std::string _search_type, std::string _fields_to_search);





        /**
        Makes the #MyBooksList to show books in the particular topic.
        @param _Topic_id id of topic in the sqlite database.
        **/
        void OnBooksInTopicWanted(long int _Topic_id);



        /**
        Opens book on doubleclick. Checks if there is an application associated with the particular file type (
        reads kolkhoznik config file for this). If it is, launches this application.
        If kolkhoznik config file have no information about application associated with the filetype, than this function
        launches
        the default system application associted with the filetype.
        **/
        void OpenThisBook(wxListEvent& event);


        /**
        The current "WHERE" statement that is used to get books from the "main" table of the database.
        The complete query to the database is:
        @code
        "SELECT * FROM main WHERE " + where_statement + "ORDER BY..."
        @endcode

        We use this function when generating a bibtex database for books
        "that are currently displayed on the right panel" (see tools->GenBibtex in the menubar).

        P.S. When generating the bibtex database we query the main kolkhoznik database
        to get book that we want to include to the bibtex database. It's where the "WHERE"
        statement comes.

        returns #where_statement.

        P.P.S. The WHERE statement could have placeholders.
        There is not need to use placeholders for integer values, so you should bind a string for each of
        the placeholders (use sqlite3_bind_text for it).
        See  #GetPlaceHoldersParameters().


        P.P.P.S. We do not have something like SetWhere(). The where statements is
        constructed automatically depending on #mode, current topic, search string.

        **/
        std::string GetWhere();

        /**
        returns #parameters_for_WHERE_placeholders
        **/
        std::vector<std::string> GetPlaceHoldersParameters();
    private:


        /**
        Initializes #m_pImageList. Reads folder with icons.
        **/
        void InitAndSetIconsList(int size);



        /**
        Deletes #m_pImageList and creates a new one. Used when user changes icons size in the configure gui.
        @param new_size size of icons in pixels
        **/
        void ReBuildIMageList(int new_size);



        /**
        icons size of the control
        **/
        int icons_size;



        /**
        Executes when user right clicks book and selects the "Info" entry.
        Shows info about particular book. Creates an object of the InfoAboutThisBook class.
        @param event is generated by wxWidgets when user right clicks book and selects  the "Info" entry.
        **/
        void OnInfoAboutBookWanted(wxCommandEvent &event);




        /**
        Creates popup menu when user right clicks book
        @param event is generated when user right clicks book
        **/
        void RightClickMenu(wxListEvent& event);



        /**
        Executes when user clicks header of the column. As a result the sorting order changes.
        @param event is generated by wxWidgets automatically
        **/
        void UserWantsToChangeSortingOrder(wxListEvent& event);



        /**
        See #mode.
        **/
        enum state_mode{SEARCH_MODE, TOPIC_MODE};



        /**
        Describes columns of the list. Assume, for example, that list contains 2 columns: "Author", "Title".
        That we have:
        @code
        items_to_show[0].first="Author"  //field in the database
        items_to_show[0].second="Author" //visible caption


        items_to_show[1].first="Title"  //field in the database
        items_to_show[1].second="Title" //visible caption
        @endcode
        @note
        #items_to_show variable is generated by the #GenItemsToShow () function.
        **/
        std::vector<std::pair<wxString,wxString> > items_to_show;





        /**
        Books cache, we fill #cache when wxWidgets fires the wxEVT_LIST_CACHE_HINT event.
        See detailed class description for more info.
        **/
        books_cache cache;



        /**
        main database
        **/
        sqlite3 *db;  //without pointer does not work!!!!, FUCK


        /**
        Search string consists of words divided by spaces.
        This string is used by the PutDataToCache () function to query the database when #mode = SEARCH_MODE.
        We need to remember the search string as the PutDataToCache () function needs it to query the database while
        user is scrolling the list.  See detailed class descriptions for more info.
        **/
        std::string search_string;



        /**
        Search type can be "or", "and". User can enter several words into the search input.
        If search type is "or", than we select rows that have one of these words from the database.
        If search type is "and", than we select only rows that have all of the words.
        We need to remember the search type as the PutDataToCache () function needs it to query the database while
        user is scrolling the list.  See detailed class description for more info.
        **/
        std::string search_type;




        /**
       fields to search (columns in the database)
        as it should be inserted into sql query. For example, valid values are
        "TitleLowerCase", "AuthorLowerCase", "AuthorLowerCase || TitleLowerCase". So, if searching should
        be done in several fields than these fields should be splitted by  the "||" symbol.
        @note Let us note that sqlite does not support a case-insensitive search
        for non-latin characters. To enable such search, we have columns like "TitleLowerCase".
        When searching, we transform a search string to the lowercase one, than search in the *LowerCase column.
        **/
        std::string fields_to_search;



        /**
        Id of the current topic in the sqlite database.
        We need to remember this variable as the PutDataToCache () function needs it to query the database while
        user is scrolling the list.  See detailed class descriptions for more info.
        **/
        long int Topic_id;



        /**
        Current full rows number in the list.
        **/
        long long full_rows_number;



        /**
        Current mode, can be SEARCH_MODE, TOPICS_MODE (see #state_mode enum). Depending on #mode the PutDataToCache () function
        performs one or another query to the database when user is scrolling the list. See detailed class descriptions for more info.
        **/
        int mode;



        /**
        This function is called by wxWidgets automatically to get text for the current cell of the list.
        See detailed class descriptions for more info.
        **/
        wxString OnGetItemText(long item, long column) const;





        /**
         This function is called by wxWidgets automatically to get an image for the current row (similar to the OnGetItemText ()  function).
         @return index of the image in the #m_pImageList.
        **/
        int OnGetItemImage 	(long  item) 	const;





        /**
        Caches books when wxWidgets fires a wxEVT_LIST_CACHE_HINT event. Max size of the cache (number of rows to cache) is
        defined by the #max_size_of_cache variable.  See detailed class descriptions for more info.
        @param event is generated by wxwidgets automatically.
        **/
        void PutDataToCache(wxListEvent& event);



        /**
        Config associated with the main config file.
        **/
        wxFileConfig *config;




        /**
        Reads config file and generates #items_to_show
        **/
        void GenItemsToShow();




        /**
        See the UpdateOrder () function description.
        **/
        std::string order;



        /**
        See the UpdateOrder () function description.
        **/
        std::string users_wanded_order;




        /**
        Disables sorting if list contains too many books or enables it if list contains not so many books.
        The reason for the sorting restrictions is a performance issues. If list is too big, than
        sorting takes too long. User can define how many books it is allowed to sort by File->Configure->Advanced.
        @note
        We have the two order variables: #order, #users_wanded_order. Let's consider a simple example to understand how this function works and why we need the variables
        . Suppose that user launched the program, chose some topic, set sorting order (let it be by
                                                                                                                   "Author").
        In this case both #order and  #users_wanded_order are "ORDER by AuthorLowerCase" (order goes into the sql query directly).
        Now, assume that user have changed topic, as a result the #OnBooksInTopicWanted function is going to be executed.
        #OnBooksInTopicWanted function sets the number of the books in the list for the new topic (and sets #full_rows_number variable) and calls this function.
        This function checks if the number of the books in the list is more than #do_not_sort_list_if_books_more_than_this_number variable (
                        kolkhoznik gets #do_not_sort_list_if_books_more_than_this_number from the config file).
        If list is too big (contains more books than #do_not_sort_list_if_books_more_than_this_number) than we should disable sorting.
        So, This function sets #order="", #users_wanded_order does not change. As we use #order for sorting (not #users_wanded_order),
        sorting becomes disabled.
        Now, assume that user have chosen another topic. If this topic have not too many books, than we should restore order. So, this functions sets #order=
        #users_wanded_order. As a result, we have the sorting back.  So, #users_wanded_order remembers the sorting order, but actual sorting is
        governed by #order; this function sets #order depending on the number of the books in the list.
        **/
        void UpdateOrder();






        /**
        Executes when user drags border between columns. Puts data about new columns position into the config file.
        Constructor of this class restores columns positions on the next kolkhoznik launch.
        @param event is generated by wxWidgets automatically.
        **/
        void OnResizeColumn(wxListEvent& event);





        /**
        full path to the main config file.
        **/
        wxString ini_filename;



        /**
        images list associated with the control
        **/
        mutable wxImageList *m_pImageList;



        /**
        Number of the currently sorting column. When user clicks header of the column, we call the #UserWantsToChangeSortingOrder () function.
        This function gets the index of the clicked column, it can be done like this:
        @code
        int clicked_column=event.GetColumn();
        @endcode
        Next, this function executes something like this:
        @code
        if(number_of_sorting_item == clicked_column)
            {
            //change order of sorting
            //code omitted here
            }
        else
            {
            //change sorting column, for example from "Author" to "Title"
            //code omitted here.
            }
        @endcode
        This is why we need to remember the number of the currenly sorting column.
        **/
        int number_of_sorting_item;



        /**
        can be "up" or "down". Depending on it we constract the sql query to get the books from the database.
        In particular, if #sort_order=="down", than we add the word "DESC" to order part of the sql query.
        **/
        std::string sort_order;


        /**
        When user right clicks book, we show a popup menu. Next, if user chooses the "Info" entry in this menu,
        we should show the info about this book. So, we remember which of the books was right clicked last.
        Next, we show the info about
        this particular book. This is why we have the #last_right_clicked_item
        **/
        long long last_right_clicked_item;


        /**
        We should not call this fucntion directly, it is used by the #OnGetItemImage function. Returns index (in the #m_pImageList) of the icon
        for the particular file extension. Uses #File_icons_map variable for this. If #m_pImageList does not
        have icon for this extension, this function adds icon to the #m_pImageList and updates #File_icons_map
        @param Extension extension of the file.
        **/
        int ChooseIcon(std::string Extension) const;


        /**
        Helps to find the index of the icon in the #m_pImageList for the particular file extension.
        See code of the #ChooseIcon function for usage example.
        **/
        mutable std::map<std::string,int> File_icons_map;

        /**
        The maximal number of the books in the books list that kolkhoznik is allowed to sort.
        See #UpdateOrder function description for more info.
        **/
        int do_not_sort_list_if_books_more_than_this_number;


        /**
        See detailed class descriptions for more info.
        **/
        int max_size_of_cache;


        /**
        Generates bibtex entry for the right-clicked book. Shows popup window with the according bibtex entry.
        Executes when user right clicks book and selects the "Gen Bibtex" entry in the popup menu.
        **/
        void OnGenBibtexMenuSelected(wxCommandEvent &event);

        /**
        Gets one book from the database
        @param num the index of the book in the book's list (right panel) (see wxListCtrl)
        **/
        Khz::Libs::DB::mapDB  GetOneBookFromDB(long long num);


        /**
        The current "WHERE" statement that is used to get books from the "main" table of the database.
        The complete query to the database is:
        @code
        "SELECT * FROM main WHERE " + where_statement + "ORDER BY..."
        @endcode

         @note The WHERE statement could have placeholders. There is not need to use placeholders for integer values, so you
        should bind a string for each of the placeholders (use sqlite3_bind_text for it). The parameters for the placeholders
        are stored in #parameters_for_WHERE_placeholders.
        **/
        std::string where_statement;

        /**
        The placeholder parameters for #where_statement. You should use them like this:
        @code
        for(unsigned int i=0; i< parameters_for_WHERE_placeholders.size(); i++)
            {
            sqlite3_bind_text(stmt,  i+1, parameters_for_WHERE_placeholders[i].c_str(), -1, SQLITE_TRANSIENT);
            }
        @endcode
        **/
        std::vector<std::string> parameters_for_WHERE_placeholders;
    };


}} //end of namespaces
