/*
Copyright (C) Ivan Ivanov, 2014-2015

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/



//#include "my_TaskBar.h"
#include <wx/frame.h>
#include <wx/splitter.h>
#include <wx/scrolwin.h>
#include <wx/msgdlg.h>
#include "MyRightPanel.h"
#include<wx/menu.h>
#include <wx/treectrl.h>
#include "Configure_tabs_parent.h"
#include <wx/notebook.h>
#include "ConfigureGui.h"
#include "FullTextSearchWindow.h"


namespace Khz{namespace MainWin{




/**
@brief !!!!!!!!!!!!!!!!!!!!  The main window of the program !!!!!!!!!!!!!!!!!!!!

The main Window is splitted vertically into two parts. The left part consists of the "Search" tab, "Topics" tab.
The right part consists of the books list.
**/
class MyMainWindow : public wxFrame
    {
    public:
        /**
        @brief Constructor

        Constructor, creates the main window of the program
        @param title Visible caption of the window
        **/
        MyMainWindow(const wxString& title);
        ~MyMainWindow();


        /**
        Config associated with the main config file
        **/
        wxFileConfig *config;


        /**
        @brief Not quits kolkhoznik, just hides main window!!!

        This function is called when user tries to close the main window.
        @note We do not exit kolkhoznik, but just hide the main window to the tray.
         (User can exit the program by pressing File->Exit or by right clicking on the
         tray icon and choosing the "Exit" entry in the popup menu).
         @param event is generated by wxWidgets automatically.
        **/
        void OnQuit(wxCloseEvent& event);

        /**
        Destroys #full_text_search_window, sets #is_full_text_search_window_shown to false.

        P.S. the #full_text_search_window skips the wxCloseEvent. So it will not close for itself
        when user tries to close it. Instead, the #Khz::FTS::Gui::FullTextSearchWindow class sends a wxBUTTON_EVENT with a special id.
        As a result, this event gets caught by the  #MyMainWindow class.
        It follows than that the  #MyMainWindow class is responsible for the #full_text_search_window closing.

        This way we can be sure that only one instance of #full_text_search_window is allocated (by the "new" operator).
        See code of the #OnFullTextSearchWanted function to understand how it is done.
        **/
        void OnFullTextSearchWindowQuit(wxCommandEvent& event);




        /**
        Shows the "about kolkhoznik" popup window when user presses Help->About
        @param event is generated by wxWidgets automatically.
        **/
        void OnAbout(wxCommandEvent& event);

        /**
        Just calls #m_right->GetWhere();
        **/
        std::string GetWhere() {return m_right->GetWhere(); }

        /**
        Just calls #m_right->GetPlaceHoldersParameters()
        **/
        std::vector <std::string> GetPlaceHoldersParameters() { return m_right->GetPlaceHoldersParameters(); }

    private:

        /**
        The full text search window, appear when user presses the "search" button
        under the full text search input
        **/
        Khz::FTS::Gui::FullTextSearchWindow *full_text_search_window;

        /**
        true if #full_text_search_window is shown, false otherwise. Used by #OnFullTextSearchWanted
        **/
        bool is_full_text_search_window_shown;

        /**
        The configuration window, user can make it appear by pressing File->Configure
        **/
        Khz::CfgWin::ConfigureGui *config_gui;

        /**
        The right panel of the program. Consists of the books list
        **/
        MyRightPanel *m_right;

        /**
        The left panel of the program, consists of the "Search" tab, "Topics" tab
        **/
        MyleftPanel *m_left;

        /**
        Splits the main window into two parts: left (topics tree, search) and right( books list)
        **/
        wxSplitterWindow *m_splitter;

        /**
        Executes when user presses the "Seach" button.
        Gets string from the search input of the #m_left and makes #m_right
        to search books with the provided search string.
        As a result, the list of found books appear on the right panel.
        @param event is generated by the "Search" button or by the Search input(on pressing Enter)
        **/
        void OnSearchWanted(wxCommandEvent& event);



        /**
        Executes when user presses the full text search button of the #Khz::MainWin::LeftSearchPanel (see #m_left).
        Gets string from the full text search input of the #m_left. If an instance of the
        #full_text_search_window is not presented, than creates
        one (#is_full_text_search_window_shown can tell if this window is created).
        Finally, this function calls #full_text_search_window->OnSearcWanted (#m_left->GetFullTextSearchText ());
        to search inside books with the provided search string.
        @param event is generated by #Khz::MainWin::LeftSearchPanel
        **/
        void OnFullTextSearchWanted(wxCommandEvent &event);


        /**
        Executes When user presses File->Create database
        @param event is genereted by wxWidgets automatically
        **/
        void OnCreateDatabase(wxCommandEvent & event);
        /**
        When user changes topic on the left panel (#m_left), this function
        sends signal to #m_right and makes it to show books in the
        particular topic.
        @param event is generated by #m_left (event appear when user changes selection in the library tree).
         Event have information about which of the topics  the user
        wants to browse. This info can be obtaned by event->GetInt(). This int is the id of the topic in the database (table "topics").
        **/
        void OnBooksInTopicWanted(wxCommandEvent &event);

        /**
        Shows the configuartion window
        @param event is generated by wxWidgets automatically
        **/
        void OnConfigure(wxCommandEvent &event);

        /**
        Remembers the size of the main window when user changes the main window geometry (For example, by dragging one of the borders).
        The corresponding information is stored in the config file.
        On kolkhoznik launch the constructor of this class reads the config file and restores the main window geometry.
        @param event is generated by wxWidgets automatically
        **/
        void OnResize(wxSizeEvent &event);

        /**
        Remembers that the main window of the program should be maximized on the next kolkhoznik launch (puts the corresponding information into the config file).
        When launching kolkhoznik, the constructor of this class reads the config file and maximazes the window if necessary.
        @param event is generated by wxWidgets automatically
        **/
        void OnMaximize(wxMaximizeEvent& event);

        /**
        Remembers the position of the splitter between the left and right panels (puts the corresponding information into the config file).
        On kolkhoznik launch the constructor of this class reads the config file and restores the splitter position.
        @param event is generated by wxWidgets automatically
        **/
        void OnSashPositionChanged(wxSplitterEvent& event);


        /**
        The full path to the config file
        **/
        wxString ini_filename;

        /**
        Shows the statistics window when user presses File->View Statistics
        @param event is generated by wxWidgets Automatically
        **/
        void OnViewStatistics(wxCommandEvent& event);

        /**
        Reconfigures panels when user changes something in the configuration window.
        @param event is generated by #config_gui. Event have an information about what the user have changed. This information
        can be retrieved by the event->GetInt() function (returns int). All possible values are part of the enum in Custom_IDs.h. For example,
        one of the values is my_wx_id__RECONFIGURE_ICONS. Depengind on the value, this function makes #m_left and #m_right to change
        their appearance.
        **/
        void ReconfigurePanels(wxCommandEvent& event);


        /**
        Creates the bibtex database generation window (the #Khz::BibTex::BibTexGeneratorStandaloneWindow class).
        Executes when user selects "Tools->Gen Bibtex" in the menubar.
        **/
        void OnGenBibtexWanted(wxCommandEvent& event);

        /** Creates the full text search index generation window (the #Khz::FTS::Gui::FullTextSearchIndexerWindow  class).
        Executes when user selects "Tools->Gen full text search index" in the menubar
        **/
        void OnGenFullTextSearchIndexWanted(wxCommandEvent &event);


wxDECLARE_NO_COPY_CLASS(MyMainWindow);
    };




}}//end of namespaces
