/*
Copyright (C) Ivan Ivanov, 2014-2015

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/



#include <wx/panel.h>
#include <wx/notebook.h>
#include <wx/sizer.h>
#include "LeftSearchPanel.h"
#include "SearchForTopicDialog.h"
namespace Khz{namespace MainWin{


/**
@brief The left panel of the program, consists of the "Search" tab, "Topics" tab

The left panel of the program, consists of the "Topics" tab, "Search" tab.

- When user hits the "search" button (not fulltext search, normal search), this
class sends an wxEVT_BUTTON event with the special id, the id is my_wx_id__SEARCH_BOOKS_BUTTON.
The event with this id gets caught by the #Khz::MainWin::MyMainWindow, as a result
#Khz::MainWin::MyMainWindow::OnSearchWanted gets executed. #Khz::MainWin::MyMainWindow::OnSearchWanted
Reads the search string from the search input and performs the searching.

P.S. When user hits ENTER to search book  (not fulltext search, normal search), this
class sends an wxEVT_TEXT_ENTER event with the special id, the id is my_wx_id__SEARCH_BOOKS_INPUT.
The event with this id also gets caught by the #Khz::MainWin::MyMainWindow, as a result
#Khz::MainWin::MyMainWindow::OnSearchWanted gets executed. #Khz::MainWin::MyMainWindow::OnSearchWanted
Reads the search string from the search input and performs the searching as before.


When user change selection inside of the library tree, this class emits a wxEVT_BUTTON command event
with the special id, the id is my_wx_id__SHOW_BOOKS_IN_TOPIC_COMMAND_EVENT.
The event finally gets caught by the MyMainWindow class, as a result
the #Khz::MainWin::MyMainWindow::OnBooksInTopicWanted function gets executed, making the right panel
of the program to display books belonging to the selected topic.

P.S. One have to call event.GetInt() to get the id of the topic in the "topics"
table of the main kolkhoznik database.

**/
class MyleftPanel : public wxPanel
{
public:
    /**
    Constructor, puts together the "Search" tab, the "Topics" tab.
    @param parent typical wxWxidgets parent
    **/
    MyleftPanel(wxWindow *parent);

    /**
    "Search" tab, consists of the search input, search button and so on.
    **/
    LeftSearchPanel *search_panel;




    /**
    Reads config again and updates the order of the stemmers of the #search_panel
    accordingly (full text search section). Just calls LanguageSelectorWxChoice::ReDrawStemmers
    **/
    void ReDrawStemers(){search_panel->ReDrawStemmers();};



    /**
    Gets String from the search input
    **/
    wxString GetSearchText();


    /**
    Gets string from the full text search input. Just calls #search_panel->GetFullTextSearchText ()
    **/
    wxString GetFullTextSearchText(){return search_panel->GetFullTextSearchText();};

    /**
    Gets array of topics where user wants to perform the full text search.
    Just calls LefSearchPanel::GetArrayOfTopicsForFullTextSearch (array)

    @param array Give an empty vector here. The function will fill it.
    **/
    void GetArrayOfTopicsForFullTextSearch(std::vector<wxString> &array)
        {
        search_panel->GetArrayOfTopicsForFullTextSearch(array);
        }

    /**
    Gets selected language from #Khz::MainWin::LeftSearchPanel::full_text_search_language_selector
    **/
    wxString GetLanguageForFullTextSearch(){return search_panel->GetLanguageForFullTextSearch();};

    /**
    Returns true if the "Author" checkbox of the search tab is checked, false otherwise
    **/
    bool IsAuthorChecked();

    /**
    Returns true if the "Title" checkbox of the search tab is checked, false otherwise
    **/
    bool IsTitleChecked();
     /**
    Returns the search type based on the radiobutton state.
    @return wxString.
    @note The search type can be "or", "and". User can enter several words into the
    search input. If the search type is "or", than we select rows that have one of
    these words from the database. If the search type is "and", than we select only
     rows that have all of the words.
    **/
    wxString GetSearchType();

    /**
    Topics tree.  User can browse the topics of the library here.
    **/
    TreeCtrlLibrary *topics_tree;

    /**
    Unselects all of the topics. We do not use it currently.
    **/
    void UnselectAllTopics();

    /**
    We should call this function if we have changed the database path in the config file.
    The function reads the config file again and reloads the library tree(left panel, "topics" tab).
    **/
    void ReconfigureLibraryTree();

    /**
    We should call this function if we have changed the font size or the icons size of the library tree in the config file.
    This function reads the config file again and changes the appearance of the #topics_tree object.
    **/
    void SetNewLibraryTreeAppearance();

    /**
    Disables the "Search" button, we should call this before actually searching.
    **/

    void on_search_begin();

    /**
    Enables the "Search" button (disabled previously by #on_search_begin() )
    **/
    void on_search_end();



     /**
    Disables the "Search" button of the full text search section.
    We should call this function before actually searching.
    **/
    void on_full_text_search_begin(){search_panel->on_full_text_search_begin();};


    /**
    Enables the "Search" button of the full text search section (disabled previously by #on_full_text_search_begin() )
    **/
    void on_full_text_search_end(){search_panel->on_full_text_search_end();};


private:


    /**
    a popup dialog appearing when user starts typing while browsing topics tree.
    This dialog allows to search for a topic
    **/
    SearchForTopicDialog *topic_search_dlg;




     /**
    just calls
    @code
    wxString topic=event.GetString();
    topics_tree->ActivateTopicIfExists(topic);
    @endcode
    the corresponding event is generated by #topic_search_dlg
    **/
    void display_search_results_for_topic_selected_inside_of_SearchForTopicDialog(wxCommandEvent &event);



    /**
    shows #topic_search_dlg
    **/
    void OnKeyPressedInsideOfTopicsTab(wxKeyEvent&event);
};


}} // end of namespaces
