/*
Copyright (C) Ivan Ivanov, 2016

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/


#include <wx/dialog.h>
#include <wx/srchctrl.h>
#include <wx/listbox.h>
#include "sqlite3.h"
#include <wx/fileconf.h>
#include <boost/function.hpp>
#include <boost/thread.hpp>
#include "Mdb_fuctions.h"

namespace Khz{namespace MainWin{





/**
@brief the "search for topic" dialog. Appear when user starts typing when browsing topics.

We create an object of this class inside of #MyLeftPanel
@note the search itself is performed by a parallel thread. The idea is to prevent the gui freezing if
searching is time-consuming.
**/
class SearchForTopicDialog : public wxDialog
    {
    public:

        /**
        constructor
        **/
        SearchForTopicDialog
            (
            wxWindow *parent,
            wxWindowID id,
            wxString title,
            const wxPoint &pos=wxDefaultPosition,
            const wxSize &size=wxDefaultSize
            );
        /**
        destructor
        **/
        ~SearchForTopicDialog()
            {
            delete config;
            sqlite3_close(db);
            }
        /**
        Returns false if a parallel thread is running right now.
        You cannot destruct the object of this class
        if #IsIdle returns false.
        **/
        bool IsIdle(){return !parallel_thread_is_running_righ_now;};



        /**
        the searc input
        **/
        wxSearchCtrl *search_input;
    private:


        /**
        listbox holding the list of topics found during the search.
        **/
        wxListBox *topics_list;


        /**
        full path to the main config file.
        **/
        wxString ini_filename;


        /**
        WxFileConfig associated with the main config file.
        **/
        wxFileConfig *config;


        /**
        sqlite3 object associated with the main kolkhoznik database.
        **/
        sqlite3 *db;


        /**
        Searchs for a topic.
        @param search_string The search string, consist of words(fragments of words) divided by spaces.
        This search query is something like this:
        @code
        std::string search_string= "physics matem";
        std::string query_to_db= "SELECT * FROM main WHERE (Topic LIKE '%physics%' AND Topic LIKE '%matem%');
        @endcode
        @param finished_callback An arbitrary function that gets executed when searching is finished.

        This function gets called only by #InitSearch(). The function (#MainSearcherParallelThreadInternal)
         gets executed in the parallel thread. The results of the searching
        are stored in #current_result.

        See #InitSearch for more details.
        **/
        void MainSearcherParallelThreadInternal
                    (
                    std::string search_string,
                    boost::function<void()> &finished_callback
                    );


        /**
        #InitSearch() sets this var to true before it launches searching (in a parallel thread).
        When searching is finished, #OnSearchFinished sets this var to false.
        This var is used to prevent launching two parallel threads. If #parallel_thread_is_running_righ_now is true, than #InitSearch()
        would not start searching, it would just call return.
        **/
        bool parallel_thread_is_running_righ_now;




        /**
        See #InitSearch()
        **/
        bool a_search_was_skipped;




        /**

        Let me describe the search procedure first.
        When user types something inside of #search_input, #OnSearchInputChanged() gets executed. #OnSearchInputChanged() calls #InitSearch().

        #InitSearch() sets #parallel_thread_is_running_righ_now to true and launches the search.
        The searching is performed by a parallel thread.
        BUT!!!! if #parallel_thread_is_running_righ_now is true when #InitSearch() gets called, than
        #InitSearch() just  sets #a_search_was_skipped to true and returns. This way we ensure that only one parallel thread is running at
        any moment.


        In order to understand the logic let us consider first the case when #parallel_thread_is_running_righ_now is false while #InitSearch() gets executed.
        In this case #InitSearch() launches a search in the parallel thread.
        This function does not wait until the search is completed (The actual function that performs searching is
        #MainSearcherParallelThreadInternal, #InitSearch() just launches it.)
        When searching is competed, #display_search_results_callback gets executed (it goes as a
        second param to  #MainSearcherParallelThreadInternal).
        #display_search_results_callback is binded to #display_search_results_callback_void() in the constructor of this class.
        So, the actual callback is display_search_results_callback_void(). As wxwidgets does not allow the gui manipulation from the
        parallel thread, display_search_results_callback_void() just sends a event indicating that the searching process have finished.
        This event gets caught by the main thread. When main thread catches this event, it executes #OnSearchFinished() (we have the
        corresponding binding). #OnSearchFinished() sets #parallel_thread_is_running_righ_now to false and puts the results of the
        search into #topics_list.

        Now let's discuss the case  when #parallel_thread_is_running_righ_now is true while #InitSearch() gets executed.
        As it was said before, in this case #InitSearch() just #a_search_was_skipped to true and returns.
        That would mean that our search results would  get outdated when user stops typing inside of #search_input.
        To fix this we do the following: we catch a wxIdleEvent event that gets emitted by wxwidgets when user stops typing
        inside of #search_input. As a result, #OnIdle() gets executed. #OnIdle checks if #a_search_was_skipped is true. If it is,
        than #OnIdle() calls #InitSearch() once again. (The search string is of course the actual string from #search_input.)
        **/
        void InitSearch
                    (   std::string search_string
                    );





        /**
        See #InitSearch()
        **/
        void display_search_results_callback_void();


        /**
        See #InitSearch()
        **/
        boost::function<void()> display_search_results_callback;



        /**
        The current search results.
        @note You could use it only inside of #OnSearchFinished,
        otherwise you could get segfault (if a parallel thread would try to access it
        together with you)
        **/
        Khz::Libs::DB::mapDB current_result;


        /**
        See #InitSearch()
        **/
        void OnSearchFinished(wxCommandEvent &event);


        /**
        See #InitSearch()
        **/
        void OnIdle(wxIdleEvent &event);


        /**
        See #InitSearch()
        **/
        void OnSearchInputChanged(wxCommandEvent &event);


        /**
        Fires a event informing us that user have selected some topic.
        The id of the event is my_wx_id__SEARCH_FOR_TOPIC_DIALOG_ITEM_SELECTED,
        event.GetString() would return the topic.

        This function gets executed when user activates (double clicks or hits enter while item is selected) an item inside of #topics_list
        **/
        void  OnTopicActivated(wxCommandEvent &event);


        /**
        Remembers the size of the window when user changes the window geometry (For example, by dragging one of the borders).
        The corresponding information is stored in the config file.
        On kolkhoznik launch the constructor of this class reads the config file and restores the window geometry.
        @param event is generated by wxWidgets automatically
        **/
        void OnResize(wxSizeEvent &event);



        /**
        Remembers that the window should be maximized on the next kolkhoznik launch (puts the corresponding information into the config file).
        When launching kolkhoznik, the constructor of this class reads the config file and maximizes the window if necessary.
        @param event is generated by wxWidgets automatically
        **/
        void OnMaximize(wxMaximizeEvent& event);

        /**
        The same as #OnTopicActivated, but executes when user clicks a topic by a mouse.
        **/
        void OnTopicClickedByMouse(wxMouseEvent& event);


        /**
        If pressed key is ESCAPE than hides the dialog. Otherwise do nothing.
        **/
        void OnKeyPressed(wxKeyEvent &event);
    };


}}//namespaces

