/*
Copyright (C) Ivan Ivanov, 2014-2015

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef KOLKHOZNIK_TREECTRLLIBRARY_HEADER_INCLUDED
#define KOLKHOZNIK_TREECTRLLIBRARY_HEADER_INCLUDED



#include <wx/treectrl.h>
#include "sqlite3.h"
#include <wx/fileconf.h>
#include <vector>


namespace Khz{namespace MainWin{

/**
@brief The library tree. The tree is represented by the "Topics" tab of the left panel. User can browse topics (Physics, Mathematics and so on) here

The library tree. The tree is represented by the "Topics" tab of the left panel. User can browse topics (Physics, Mathematics and so on) here.
If user clicks a topic, than the books associated with the topic appear on the right panel.

When user change selection inside of the library tree, this class emits a wxEVT_BUTTON command event
with the special id, the id is my_wx_id__SHOW_BOOKS_IN_TOPIC_COMMAND_EVENT.
The event finally gets caught by the MyMainWindow class, as a result
the #Khz::MainWin::MyMainWindow::OnBooksInTopicWanted function gets executed, making the right panel
of the program to display books belonging to the selected topic.

P.S. One have to call event.GetInt() to get the id of the topic in the "topics"
table of the main kolkhoznik database.


**/
class TreeCtrlLibrary : public wxTreeCtrl
{
public:
    /**
    Constructs the library tree
    @param parent standard wxWidgets parent
    @param id standard wxWidgets id
    @param pos starndard wxWidgets window position
    @param size standard wxWidgets window size
    @param style standard wwWidgets window style
    **/
    TreeCtrlLibrary(wxWindow *parent, const wxWindowID id,
                       const wxPoint& pos, const wxSize& size,
                       long style);
    ~TreeCtrlLibrary();




    /**
    Recursively searches for an item with the specified topic.
    If item exists, activates it (SELECTS the corresponding item. As a result,
    the right panel of the main kolkhoznik window would display books belonging to a topic).

    @note kolkhoznik displays books belonging to a topic when an item gets selected.
    See #OnSelChanged () for more info.
    **/
    void ActivateTopicIfExists(wxString topic);



    /**
    Returns topic associated with the particular item
    @return int is the id of the topic in the database (table "topics")
    @param id is the id of the item (see the desription of the wxTreeCtrl class (wxWidgets library))
    **/
    long int GetTopic(wxTreeItemId id);
    /**
    Call this function if you have changed the database path in the config file.
    Connects to the new database and rebuilds the topics tree
    **/
    void Reconfigure();
    /**
    Remembers the state of the library tree (expanded items, current selection) and redraws it.
    You should call this function  if you have changed icons or fonts.
    <br>
    IMPORTANT!!! How the redrawing actually works is described in the #ReExpandItems function description

    @note I have no idea why, but one cannot set font from inside of the wxTreeCtrl, and, as the
    TreeCtrlLibrary is inhereted from wxTreeCtrl, one cannot set font from inside of the TreeCtrlLibrary class too.
    So, we set font from inside of the  MyleftPanel class and call the #ReDrawItems function to fit the items heights to the new font size.
    **/
    void ReDrawItems();
    /**
    When user changes selection inside of the library tree, we show books belonging to the new selected topic
    on the right panel (more precisely TreeCtrlLibrary generates event with special id. This events gets caught by the
                        #Khz::MainWin::MyMainWindow class object, #Khz::MainWin::MyMainWindow makes #Khz::MainWin::MyRightPanel to show books belonging to the particular topic).
    So, suppose that you were browsed topics, than you switched to the "Search" tab and made some searching.
    The right panel of the program shows the result of the searching now. Next, you decide to change the icons size of the library tree.
    As a result the #ReDrawItems function gets executed, this function removes all of the items from the library tree,
    regenerates them again and RESTORES  the SELECTION. What happens next? Right, the search results (right panel) will
    disappear and books from the selected topic will appear. To prevent this we have to block the selection event.
    It is done like this: #ReDrawItems sets #block_next_selection_event to true, delete items, restores them, restores SELECTION.
    When #ReDrawItems restores selection, the  wxWidgets library sends event, as a result the #OnSelChanged function gets executed.
    #OnSelChanged checks if #block_next_selection_event is false. If it is, #OnSelChanged sends special event to the #Khz::MainWin::MyMainWindow class
    object, as a result #Khz::MainWin::MyMainWindow makes #Khz::MainWin::MyRightPanel to show books belonging to the  particular topic.  If #block_next_selection_event is true, than
    #OnSelChanged sets it to false (the next selection changing should refresh the books list) and DOES NOT SEND the special event to #Khz::MainWin::MyMainWindow.
    As a result, the books list remains unchanged and the search results do not disappear.
    **/
    bool block_next_selection_event;


    /**
    Returns topic associated with the particular item
    @return full topic, for example "/Physics/Astronomy"
    @param id is the id of the item (see the desription of the wxTreeCtrl class (wxWidgets library))
    **/
    wxString GetTopicString(wxTreeItemId id);
    //void SetNewFont(wxFont font);
private:
    /**
    sqlite database object associated with the main database
    **/
    sqlite3 *db;  //without pointer does not work!!!!, FUCK
    /**
    config associated with the main config file
    **/
    wxFileConfig *config;
    /**
    Looks into the database and appends subitems (subtopics) to the particular item.
    See #OnItemExpanding for more info.
    @param parent an item to append subtopics
    **/
    void AppendSubtopicsToItem(const wxTreeItemId &parent);
    /**
    Topics are stored as strings in the database. for example, a topic could be "/Physics/Geophysics/Travel Guide" (subtopics
    are separated by the "/" symbol). For this topic, the visible on the screen text of the item should be "Travel Guilde".
    So, this function gets the text after the last "/" symbol (visible text).
    **/
    wxString GetTreeTextFromTopic(std::string topic_std);
    /**
    We do not load the whole library tree on the  program launch, we load topics only when they are needed.
    When browsing the top level topic,  we load only the top level items together with their childs (without childs
    one cannon know if an item has subtopics and if this item should be expandable).
    When user expands a topic, this topic already has childs. We should add subchilds to these childs to make
    these childs expandable if necessary. So, this function adds these childs (calls #AppendSubtopicsToItem function).
    @param event is generated by wxWidgets automatically
    **/
    void OnItemExpanding(wxTreeEvent& event);
    /**
    Builds #images_list and calls
    @code
    SetImageList(images_list);
    @endcode
    @param size size of the icons in pixels
    **/
    void SetImages(int size);
    /**
    Emits a special event (the event with the special id, the id is my_wx_id__SHOW_BOOKS_IN_TOPIC_COMMAND_EVENT).
    The event finally gets caught by the MyMainWindow class, as a result
    the #Khz::MainWin::MyMainWindow::OnBooksInTopicWanted function gets executed, making the right panel
    of the program to display books belonging to the selected topic.
    @param event is generated by wxWidgets automatically
    **/
    void OnSelChanged(wxTreeEvent &event);
    //void Expand_Collapse_item_onclick(wxMouseEvent &event);
    //void ShowBooksInTopic(wxMouseEvent &event);

    /**
    Normally, when user changes selection in the library tree, we refresh the books list (We display books belonging t
                                                                                                      to the new selected topic on the right panel).
    Suppose now that user was browsing topics for a while, than he switched to the "Search" tab and made some searching.
    When user presses the "Search" buttom,  the current topic of the library tree  remains selected.
    When user returns to the topics tree and clicks on the ALREADY SELECTED ITEM, than  this
    funcion fires an event (#OnSelChanged is not executed as the selection remains the same)
    . The MyMainWindow class catches this event and shows books belonging to the topic.

    P.S. the #OnSelChanged function sends the same event to MyMainWindow to show books in topic
    **/
    void FireAdditionalShowBooksInTopicEvent(wxMouseEvent &event);
    /**
    Full path to the config file
    **/
    wxString ini_filename;
    /**
    Index of the icon in the #images_list, can be -1 (no icons) or 0 (one icon for all of the topics)
    **/
    int icon_num;

    /**
    Deletes  #images_list, than calls #SetImages to create new #images_list.
    The function is used to change the size of the icons
    @param new_size is size of icons
    **/
    void ReBuildIMageList(int new_size);
    /**
    When user changes icons size or font in the config file (not manually, by GUI), we have to apply changes.
    To do this, we first remember which of the items are expanded. For this we call the #FindExpandedItems function.
    This function  Recursively checks all of the items and fills
    #expanded_items_remember_vector. In addition, this function finds a selected item and remembers it(fills #selected_item_remember_vector).
    Next, we call the #ReDrawItems function. This function
    deletes all of the items and draws a new ones. Next, we need to expand all of the items that were expanded before the redrawing.
    This is done by a #ReExpandItems (this) function. To reexpand items this function uses #expanded_items_remember_vector.
    After items are reexpanded we should restore the selection. This is done by
    the #ReStoreSelection function. This function takes #selected_item_remember_vector and selects the according item.
    @param path path to expand. You should provide the #expanded_items_remember_vector[i] here
    @param item item for the <strong class="paramname">path</strong>. You should provide the root item here


    Let us give an example for better understanding of the remember vectors structure. Suppose that you have this tree structure:
    @code
    -rootitem
        +subitem0
        -subitem1
            +1subitem0
            +1subitem1
            -1subtime2
                12subitem
            +1subitem3
        +subitem2
        -subitem3
            +3subtim0
        +subitem5
    @endcode
    In this case #expanded_items_remember_vector consists of two subvectors:
    @code
    expanded_items_remember_vector[0]=subvector1
    expanded_items_remember_vector[1]=subvector2

    subvector1[0]=pair(1,1)  //subitem1 is expanded, the first num is the index of the
    //item (subitem1 is the first subitem of the root item, counting starts from 0),
    //the second num tells us that the item IS EXPANDED.
    subvector1[1]=pair(2,1) //1subtime2 is expanded (1subitem2 is the second subitem of the subitem1, counting starts from zero)

    // subvector1 tells us that relative to the root item:
    // the first item (counting from 0) is expanded
    // the second subitem inside of the first item (inner layer) is expaned
    // So, the earch subvector of the expanded_items_remember_vector traces the path of the expanded items.
    // Each next element of the subvector describes the inner layer relative to the previous element of the subvector.

    subvector2[0]=pair(3,1) // subitem3 is expanded.
    @endcode
    To restore this expansion you have to call the #ReExpandItems function two times, it can be done like this:
    @code
    for(int i=0;i<expanded_items_remember_vector.size();i++)  // expanded_items_remember_vector has two elements
        {
        ReExpandItems(rootitem, expanded_items_remember_vector[i]);  //we count relative to the rootitem
        }
    @endcode

    One can ask: why the subvector  of the #expanded_items_remember_vector
    (subvector1, subvector2 in the example above) consists of two numbers? What the second number is for?
    Let us provide an example to clarify the issue.
    Assume that subitem1 is closed, but 1subtime2 is expanged (you can get this
    by expanding subitem1, expanding 1subtime2, closing subitem1).
    In this case subvector1 is:
    @code
    subvector1[0]=pair(1,0)  //subitem1, the first num is the index of the item, the second num tells  us that the item is NOT expanded

    subvector1[1]=pair(2,1) //1subtime2 is expanded
    @endcode

    What about the #selected_item_remember_vector? As it was mentioned before, the #FindExpandedItems function fills this vector too.
    Assume that 12subitem is selected in the example above. than #selected_item_remember_vector is:
    @code
    selected_item_remember_vector[0]=1; //top level part of the path to the selected item is subitem1 (first entry, counting from zero)
    selected_item_remember_vector[1]=2; //the next part of the path to the selected item is 1subtime2 (second entry, counting from zero)
    selected_item_remember_vector[2]=0; //the final part of the path to the selected item, 12subitem has index zero.
    @endcode
    **/
    void ReExpandItems(wxTreeItemId item, std::vector<std::pair<int,int>> path );
    /**
    Recursively checks all of the items and fills #expanded_items_remember_vector. In addition, finds the selected item and remembers it
    (fills #selected_item_remember_vector).
    @param item you should give the root item of the library
    @param path you should give an empty vector

    IMPORTANT!!! See the #ReExpandItems function description for more info.
    **/
    void FindExpandedItems(wxTreeItemId item, std::vector<std::pair<int,int>> path );
    /**
    Restores selection after the redrawing of the items, See the #ReExpandItems function description for more info
    @param item provide the root item here
    @param path provide  the previously genereated #selected_item_remember_vector here
    **/
    void ReStoreSelection(wxTreeItemId item, std::vector<int> path);
    /**
    Used only inside of the #FindExpandedItems function. In general,  converts #expanded_items_remember_vector into
    #selected_item_remember_vector (fills #selected_item_remember_vector)
    @param temp_vector subvector of #expanded_items_remember_vector, describes path to the selected item. Does not include the selected item.
    @param counter the index of the selected item in the most inner part of the path, counting starts from zero

    IMPORTANT!!! See the #ReExpandItems function description for more info.
    **/
    void remember_selected_item(std::vector<std::pair<int,int>> temp_vector,int counter);

    /**
    We remember the expanded items here,
    See the #ReExpandItems function description for more info
    **/
    std::vector<std::vector<std::pair<int,int>>> expanded_items_remember_vector;  //remebers which items are expanded, needed for redraw.
    /**
    We store information about which of the items is selected here,
    See the #ReExpandItems function description for more info
    **/
    std::vector<int> selected_item_remember_vector;
    /**
    wxWidgets standard images list associated with the control
    **/
    wxImageList *images_list;



    /**
    Used only inside of ActivateTopicIfExists(wxString topic). Gets called recursively.
    **/
    void ActivateTopicIfExists(std::vector<std::string> &exploded_topic, wxTreeItemId 	id, int depth);


    /**
    executes when user presses a key. Does different kind of things.
    **/
    void OnKeyPressed(wxKeyEvent &event);

};

/**
@brief Container for the TreeCtrlLibrary items data, contains topic (wxString), the id of the topic
in the database (Integer, table "topics").

Container for the TreeCtrlLibrary items data, contains topic (wxString), the id of the topic
in the database (Integer, table "topics"). The data are associated with an every item and could be exctracted.
 See the code of the
TreeCtrlLibrary::GetTopic function for the data extraction example.
**/
class MywxTreeItemData : public  wxTreeItemData
{
public:
    /**
    counstructor
    @param topic the topic associted with the item
    @param id the id of the topic in the database
    **/
    MywxTreeItemData(wxString topic, long int id);
    /**
    Topic
    **/
    wxString AssociatedTopic;
    /**
    The id of the topic in the database
    **/
    long int AssociatedID;
};




}} // end of namespaces






#endif
